function RecentPastDecompDLG(selector,Results,IsRecentPast,HeaderStr,ConEst,NameStr,AnnualStr,CurrINI,DSGEModel)
% RecentPastDecompDLG: Displays graphs of "Recent & Past" and "Sample-Split" decompositions of the observed
%                      variables.
%
% USAGE:
%
%       RecentPastDecompDLG(selector,Results,IsRecentPast,HeaderStr,ConEst,NameStr,AnnualStr,CurrINI,DSGEModel)
%
% REQUIRED INPUT: selector (string vector) taking on the values "init", "samplebegin", "sampleend", "showgraph", and "done".
%
% OPTIONAL INPUT: Results (structure) with fields "Y", "ObsSS", "ObsIS", "ObsPast", "ObsRecent", "MeasErr", "DateVector",
%                         "KeepVar", and "tauT0" which is tau if IsRecentPast is 1 and T0 otherwise (0).
%
%                 IsRecentPast (boolean) which is 1 if a "Recent & Past" decomposition has been performed and 0
%                         if a "Sample-Split" decomposition was employed.
%
%                 HeaderStr (string vector) with the header for the dialog.
%
%                 ConEst (boolean) that takes the value 0 for unconstrained estimation of the states and
%                         structural shocks and 1 otherwise. The constraints concern the state variables.
%
%                 NameStr (string vector) that takes the values "Initial Values" or "Posterior Mode".
%
%                 AnnualStr (string vector) which is either "Original" or "Annualized".
%
%                 CurrINI (structure), with initialization information.
%
%                 DSGEModel (structure) whose fields contain all required information
%                         about the model, such as input files, variable and sample
%                         data. Actual observations are not stored in this file.
%
%
%
% NOTE: When selector = "init", then all optional input arguments are required!
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2025 European Central Bank.
%
%                       First version: March 7, 2025.
%                        This version: March 17, 2025.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 10-03-2025: Continued work on the showgraph part of the function.
%
% * 12-03-2025: Further work on the function.
%
% * 13-03-2025: Finished the showgraph part of the function.
%
% * 14-03-2025: Updated the documentation.
%
% * 17-03-2025: Added the input variable "ConEst".
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

if strcmp(selector,'init')==1;
   %
   % check if we should use the annualized results
   %
   if strcmp(lower(AnnualStr),'original')==0;
      Results.Y = Results.AnnY;
      Results.ObsSS = Results.AnnObsSS;
      Results.ObsIS = Results.AnnObsIS;
      Results.ObsPast = Results.AnnObsPast;
      Results.ObsRecent = Results.AnnObsRecent;
      Results.MeasErr = Results.AnnMeasErr;
   end;
   DSGEModel.HeaderStr = HeaderStr;
   DSGEModel.NameStr = NameStr;
   DSGEModel.AnnualStr = AnnualStr;
   DSGEModel.Results = Results;
   DSGEModel.IsRecentPast = IsRecentPast;
   if ConEst==0;
      DSGEModel.ConEstStr = '';
   else;
      DSGEModel.ConEstStr = 'State-Constrained ';
   end;
else;
   GraphGUI = findobj('Type','figure','Tag','RecentPastDecompDLG');
   controls = get(GraphGUI,'UserData');
   CurrINI = get(controls.variables,'UserData');
   DSGEModel = get(controls.variablestext,'UserData');
end;
%
%
% Set screen size directly
%
switch selector;
   case 'init';
      GraphGUI = figure('Color',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'FileName','RecentPastDecompDLG.m', ...
         'MenuBar','none', ...
         'PaperUnits','points', ...
         'Units','pixels', ...
         'Position',[(CurrINI.scrsz(3)-525)/2 (CurrINI.scrsz(4)-240)/2 525 240], ...
         'Tag','RecentPastDecompDLG', ...
         'Visible','off', ...
         'Resize','off', ...
         'Name',[DSGEModel.ConEstStr HeaderStr ' - ' NameStr], ...
         'NumberTitle','off', ...
         'CloseRequestFcn','RecentPastDecompDLG done', ...
         'ToolBar','none');
      %
      if MatlabNumber>=7.0;
         set(GraphGUI,'DockControl','off');
      end;
      %
      % Check where the sample of data actually begins
      %
      [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
      if strcmp(lower(DSGEModel.AnnualStr),'original')==1;
         [NewStartYear,NewStartPeriod] = AdjustSampleStart(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.DataFrequency,DSGEModel.KalmanFirstObservation-1);
      else;
         [NewStartYear,NewStartPeriod] = AdjustSampleStart(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.DataFrequency,DSGEModel.KalmanFirstObservation-1+(max(DSGEModel.annual)-1));
      end;
      %
      % Outer axis
      %
      AxesBox([2 2 523 230],'Graphics',45,[0.5 1],'on',CurrINI);
      %
      % Select the sample
      %
      OldDSGEModel = DSGEModel;
      DSGEModel.SubBeginYear = NewStartYear;
      DSGEModel.SubBeginPeriod = NewStartPeriod;
      %
      [FirstPeriod,LastPeriod,SubSampleStrMat] = CreateSubSample(DSGEModel);
      %
      %
      DSGEModel = OldDSGEModel;
      %
      % Select the sample
      %
      controls.sampletxt = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 180 360 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','text', ...
         'String','Selected sample:', ...
         'HorizontalAlignment','left', ...
         'Tag','SampleText');
      %
      % sample start and sample end
      %
      controls.samplebeg = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[210 185 87 20], ...
         'Style','popupmenu', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'UserData',CurrINI, ...
         'String',SubSampleStrMat, ...
         'HorizontalAlignment','center', ...
         'Callback','RecentPastDecompDLG samplebegin', ...
         'Tag','SampleBeginPopup',...
         'Value',1);
      %
      if IsRecentPast==0;
         set(controls.samplebeg,'Value',Results.tauT0);
      end;
      %
      controls.sampleend = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[303 185 87 20], ...
         'Style','popupmenu', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'UserData',CurrINI, ...
         'String',SubSampleStrMat, ...
         'HorizontalAlignment','center', ...
         'Callback','RecentPastDecompDLG sampleend', ...
         'Tag','SampleEndPopup',...
         'Value',size(SubSampleStrMat,1));
      %
      % Select variable
      %
      controls.variablestext = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 145 180 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','text', ...
         'UserData',DSGEModel, ...
         'String','Select variable:', ...
         'HorizontalAlignment','left', ...
         'Tag','ObsVarText');
      %
      controls.variables = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[210 150 180 20], ...
         'Style','popupmenu', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'UserData',CurrINI, ...
         'String',DSGEModel.VariableNames, ...
         'HorizontalAlignment','center', ...
         'Tag','ObsVarPopup',...
         'Value',1);
      %
      % check box for plotting shock groups or not
      %
      controls.useshockgroups = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 120 366 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','checkbox', ...
         'Visible','on', ...
         'HorizontalAlignment','left', ...
         'String','Use shock groups', ...
         'Callback','RecentPastDecompDLG useshockgroups', ...
         'Value',0, ...
         'Enable','on', ...
         'Tag','UseShockGroupCheckBox');
      %
      % check box for decomposing the shock groups into individual shocks
      %
      controls.decomposegroups = uicontrol('Units','pixels', ...
            'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
            'Position',[24 100 366 20], ...
            'FontSize',CurrINI.gui_fontsize, ...
            'FontName',CurrINI.gui_fontname, ...
            'FontWeight',CurrINI.gui_fontweight, ...
            'FontAngle',CurrINI.gui_fontangle, ...
            'Style','checkbox', ...
            'Visible','on', ...
            'HorizontalAlignment','left', ...
            'String','Decompose shock groups into individual shocks', ...
            'Callback','RecentPastDecompDLG fixcontrols', ...
            'Value',0, ...
            'Enable','off', ...
            'Tag','DecomposeShockGroupCheckBox');
      %
      controls.useshockaliases = uicontrol('Units','pixels', ...
            'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
            'Position',[40 80 350 20], ...
            'FontSize',CurrINI.gui_fontsize, ...
            'FontName',CurrINI.gui_fontname, ...
            'FontWeight',CurrINI.gui_fontweight, ...
            'FontAngle',CurrINI.gui_fontangle, ...
            'Style','checkbox', ...
            'Visible','on', ...
            'HorizontalAlignment','left', ...
            'String','Use shock aliases instead of shock names', ...
            'Value',0, ...
            'Enable','off', ...
            'Tag','UseShockAliasesCheckBox');
      %
      controls.skipmeasurementerrors = uicontrol('Units','pixels', ...
            'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
            'Position',[24 60 366 20], ...
            'FontSize',CurrINI.gui_fontsize, ...
            'FontName',CurrINI.gui_fontname, ...
            'FontWeight',CurrINI.gui_fontweight, ...
            'FontAngle',CurrINI.gui_fontangle, ...
            'Style','checkbox', ...
            'Visible','on', ...
            'HorizontalAlignment','left', ...
            'String','Skip measurement errors', ...
            'Value',0, ...
            'Enable','on', ...
            'Tag','SkipMeasurementErrorsCheckBox');
      %
      controls.blackedgebars = uicontrol('Units','pixels', ...
            'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
            'Position',[24 40 366 20], ...
            'FontSize',CurrINI.gui_fontsize, ...
            'FontName',CurrINI.gui_fontname, ...
            'FontWeight',CurrINI.gui_fontweight, ...
            'FontAngle',CurrINI.gui_fontangle, ...
            'Style','checkbox', ...
            'Visible','on', ...
            'HorizontalAlignment','left', ...
            'String','Draw bars with black edges', ...
            'Value',1, ...
            'Enable','on', ...
            'Tag','BlackEdgeBarsCheckBox');
      %
      %
      controls.saveplotstofile = uicontrol('Units','pixels', ...
            'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
            'Position',[24 20 366 20], ...
            'FontSize',CurrINI.gui_fontsize, ...
            'FontName',CurrINI.gui_fontname, ...
            'FontWeight',CurrINI.gui_fontweight, ...
            'FontAngle',CurrINI.gui_fontangle, ...
            'Style','checkbox', ...
            'Visible','on', ...
            'HorizontalAlignment','left', ...
            'String','Save figures to file', ...
            'Value',0, ...
            'Enable','on', ...
            'Tag','SavePlotsToFileCheckBox');
      %
      % Create graph button
      %
      controls.show = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[420 55 90 20], ...
         'String','Display', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'CallBack','RecentPastDecompDLG showgraph', ...
         'Enable','on', ...
         'Tag','Done');
      %
      %  Done button
      %
      controls.done = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[420 20 90 20], ...
         'String','Done', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'CallBack','RecentPastDecompDLG done', ...
         'Enable','on', ...
         'UserData','waiting', ...
         'Tag','Done');
      %
      set(GraphGUI,'UserData',controls,...
         'HandleVisibility','callback');
   case 'samplebegin'
      %
      % check to make sure that the sample begins before it ends!
      %
      samplebeg = get(controls.samplebeg,'Value');
      sampleend = get(controls.sampleend,'Value');
      if sampleend<samplebeg;
         set(controls.sampleend,'Value',samplebeg);
      end;
   case 'sampleend'
      %
      % check to make sure that the sample begins before it ends
      %
      samplebeg = get(controls.samplebeg,'Value');
      sampleend = get(controls.sampleend,'Value');
      if sampleend<samplebeg;
         set(controls.samplebeg,'Value',sampleend);
      end;
   case 'useshockgroups'
      if get(controls.useshockgroups,'Value')==1;
         if length(unique(DSGEModel.ShockGroups(DSGEModel.Results.KeepVar)))==length(DSGEModel.Results.KeepVar);
            set(controls.decomposegroups,'Enable','off','Value',0);
         else;
            set(controls.decomposegroups,'Enable','on');
         end;
      else;
         set(controls.decomposegroups,'Enable','off','Value',0);
      end;
   case 'fixcontrols'
      ShockGroups = unique(DSGEModel.ShockGroups(DSGEModel.Results.KeepVar));
      if (get(controls.decomposegroups,'Value')==1)&(length(ShockGroups)>1);
         set(controls.useshockaliases,'Enable','on');
      else;
         set(controls.useshockaliases,'Enable','off','Value',0);
      end;
   case 'showgraph'
      %
      % collect variable info
      %
      VariableValue = get(controls.variables,'Value');
      CurrVariable = StringTrim(DSGEModel.VariableNames(VariableValue,:));
      n = DSGEModel.n;
      if VariableValue<n;
         set(controls.variables,'Value',VariableValue+1);
      else;
         set(controls.variables,'Value',1);
      end;
      %
      % check if the bars should have black edges or not
      %
      BlackEdgeBars = get(controls.blackedgebars,'Value');
      %
      % get the desired sample dates
      %
      samplebeg = get(controls.samplebeg,'Value');
      sampleend = get(controls.sampleend,'Value');
      SampleStrMat = get(controls.samplebeg,'String');
      FigureHandles = [];
      %
      % check if we should use shock groups or individual shocks
      %
      UseShockGroups = get(controls.useshockgroups,'Value');
      %
      % create a vector with dates
      %
      DateVector = MakeDateVector(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency);
      %
      % Make sure that DateVector is not longer than the number of periods for the shocks
      %
      if length(DateVector)>length(DSGEModel.Results.Y);
         DateVector = DateVector(1,length(DateVector)-length(DSGEModel.Results.Y)+1:length(DateVector));
      end;
      %
      % create the figure
      %
      FigHandle = figure('Visible','off', ...
                         'Position',[(CurrINI.scrsz(3)-750)/2 (CurrINI.scrsz(4)-500)/2 750 500], ...
                         'Tag',CurrVariable, ...
                         'Name',[DSGEModel.ConEstStr DSGEModel.HeaderStr ' - ' CurrVariable]);
      %
      FigureHandles = [FigureHandles FigHandle];
      SetFigureProperties(FigHandle,CurrINI);
      %
      % Create the Y-data matrix for the shock groups
      %
      if UseShockGroups==1;
         ShockGroups = DSGEModel.ShockGroups(DSGEModel.Results.KeepVar);
         Ymat = [DSGEModel.Results.ObsIS(VariableValue,:);zeros(size(DSGEModel.ShockGroupNames,1),size(DSGEModel.Results.Y,2))];
      else;
         ShockGroups = (1:length(DSGEModel.StateShockPositions));
         ShockGroups = (1:length(DSGEModel.Results.KeepVar));
         Ymat = [DSGEModel.Results.ObsIS(VariableValue,:);zeros(length(ShockGroups),size(DSGEModel.Results.Y,2))];
      end;
      for i=1:length(DSGEModel.Results.KeepVar);
         Ymat(1+ShockGroups(i),:) = Ymat(1+ShockGroups(i),:)+DSGEModel.Results.ObsRecent(VariableValue,:,i);
      end;
      %
      % add all past shocks to initial state
      %
      Ymat(1,:) = Ymat(1,:)+sum(DSGEModel.Results.ObsPast(VariableValue,:,:),3);
      %
      % remove rows from Ymat where the shock groups are zero because all individual shocks
      % in that group are zero for all t.
      %
      Ymat = Ymat([1 1+unique(ShockGroups)],:);
      %
      % set up the color scheme for the bars
      %
      UseAliases = 0;
      if (UseShockGroups==1)&(isempty(DSGEModel.ShockGroupColors)==0)&(size(DSGEModel.ShockGroupColors,1)==length(unique(ShockGroups)));
         ShockGroupColors = DSGEModel.ShockGroupColors(unique(ShockGroups),:);
      else;
         if (isempty(DSGEModel.ShockColors)==0)&(size(Ymat,1)-1==size(DSGEModel.ShockColors(DSGEModel.Results.KeepVar,:),1));
            ShockGroupColors = DSGEModel.ShockColors(DSGEModel.Results.KeepVar,:);
         else;
            ShockGroupColors = CreateDefaultColors(size(Ymat,1)-1);
         end;
         if (UseShockGroups==0)|(isequal(DSGEModel.ShockGroupNames,DSGEModel.StateShockNames)==1);
            UseAliases = 1;
         end;
      end;
      %
      ShockGroupColors = [0.3 0.3 0.3;ShockGroupColors];
      if DSGEModel.IsRecentPast==1;
         InitStateStr = ['Past shocks with lag=' int2str(DSGEModel.Results.tauT0)];
      else;
         InitStateStr = ['Past shocks up to ' DSGEModel.Results.SampleStrMat(DSGEModel.Results.tauT0,:)];
      end;
      if (isequal(DSGEModel.Results.MeasErr,zeros(size(DSGEModel.Results.Y)))==0)&(get(controls.skipmeasurementerrors,'Value')==0);
         Ymat = [Ymat;DSGEModel.Results.MeasErr(VariableValue,:)];
         MeasureStr = 'Measurement Error';
         ShockGroupColors = [ShockGroupColors;0.7 0.7 0.7];
      else;
         MeasureStr = '';
      end;
      %
      % Create the bars and collect the handles
      %
      if MatlabNumber>8.3;
         BarHandles = bar(DateVector',Ymat','stacked');
      else;
         if MatlabNumber>=7.0;
            %
            % make sure the bars are written as patches
            %
            BarHandles = bar('v6',DateVector',Ymat','stacked');
         else;
            BarHandles = bar(DateVector',Ymat','stacked');
         end;
      end;
      %
      % remake the bar patches such that all bars are displayed properly.
      %
      BarHandles = RemakeBars(BarHandles,ShockGroupColors,BlackEdgeBars);
%      Diff = DSGEModel.Results.Y-DSGEModel.Results.ObsSS-DSGEModel.Results.ObsIS-sum(DSGEModel.Results.ObsPast,3)-sum(DSGEModel.Results.ObsRecent,3)-DSGEModel.Results.MeasErr;
%      disp(Diff)
      %
      % the observed data in deviation from steady-state
      %
      y = DSGEModel.Results.Y(VariableValue,:)-DSGEModel.Results.ObsSS(VariableValue,:);
      %
      % plot the observed data
      %
      LineHandle = plot(DateVector,y,'.-','Color',[0 0 0],'LineWidth',2);
      if strcmp(lower(DSGEModel.AnnualStr),'original')==1;
         SetAxesFonts(CurrINI,'Title',[CurrVariable ' (original data) in deviation from steady-state at ' lower(DSGEModel.NameStr)]);
      else;
         SetAxesFonts(CurrINI,'Title',[CurrVariable ' (annual data) in deviation from steady-state at ' lower(DSGEModel.NameStr)]);
      end;
      axis('tight');
      %
      % check y-axis limites
      %
      ylim = get(gca,'YLim');
      for h=1:length(BarHandles);
         if h==1;
            YData = get(BarHandles(h),'YData');
            if size(YData,1)~=4;
               YData = YData';
            end;
            ylim(1) = min(min(YData(:,samplebeg:sampleend)));
            ylim(2) = max(max(YData(:,samplebeg:sampleend)));
         else;
            YData = get(BarHandles(h),'YData');
            if size(YData,1)~=4;
               YData = YData';
            end;
            ylim(1) = min([min(YData(:,samplebeg:sampleend)) ylim(1)]);
            ylim(2) = max([max(YData(:,samplebeg:sampleend)) ylim(2)]);
         end;
      end;
      %
      % see if we should add a zero line
      %
      add_on = 0.1*(ylim(2)-ylim(1));
      if add_on==0;
         Ylim = get(gca,'YLim');
         add_on = 0.1*(Ylim(2)-Ylim(1));
      end;
      ylim = [ylim(1)-add_on ylim(2)+add_on];
      set(gca,'YLim',ylim);
      if (ylim(1)<0)&(ylim(2)>0);
         hold('on');
         plot([DateVector(1) DateVector(length(DateVector))],[0 0],'-','Color',[0 0 0]);
         %
         % make sure the zero line is displayed below the observed variable
         %
         kids = get(gca,'Children');
         if size(kids,1)>size(kids,2);
            kids = kids';
         end;
         kids = [kids(2:length(kids)) kids(1)]';
         set(gca,'Children',kids);
      end;
      if UseAliases==0;
         ShockGroupNames = '';
         for i=1:size(DSGEModel.ShockGroupNames,1);
            ShockGroupNames = strvcat(ShockGroupNames,StringTrim(strrep(DSGEModel.ShockGroupNames(i,:),'_',' ')));
         end;
      else;
         ShockGroupNames = DSGEModel.ShockAliases(DSGEModel.Results.KeepVar,:);
         for j=1:size(ShockGroupNames,1);
            ShockGroupNames(j,:) = strrep(ShockGroupNames(j,:),'_',' ');
         end;
      end;
      %
      % make sure that only the selected sample is displayed
      %
      XData = get(BarHandles(1),'XData');
      if size(XData,1)>size(XData,2);
         XData = XData';
      end;
      set(gca,'XLim',[min(XData(:,samplebeg)) max(XData(:,sampleend))]);
      %
      % check is we should adjust ShockGroupNames
      %
      if (size(ShockGroupNames,1)>length(DSGEModel.Results.KeepVar))&(size(ShockGroupNames,1)>=max(DSGEModel.Results.KeepVar));
         ShockGroupNames = ShockGroupNames(DSGEModel.Results.KeepVar,:);
      elseif (size(ShockGroupNames,1)>length(DSGEModel.Results.KeepVar))&(size(ShockGroupNames,1)<max(DSGEModel.Results.KeepVar));
         ShockGroupNames = ShockGroupNames(1:length(DSGEModel.Results.KeepVar),:);
      end;
      if MatlabNumber>=7;
         legHandle = legend([BarHandles LineHandle],strvcat(InitStateStr,ShockGroupNames,MeasureStr,CurrVariable),'Location','NorthWest');
      else;
         legHandle = legend([BarHandles LineHandle],strvcat(InitStateStr,ShockGroupNames,MeasureStr,CurrVariable),2);
      end;
      SetAxesFonts(CurrINI,legHandle);
      hold('off');
      %
      % check if we should fix the X label
      %
      SetXTickIntegers(gca);
      %
      % check if we should save the figure to disk
      %
      if get(controls.saveplotstofile,'Value')==1;
         %
         if max(DSGEModel.annual)==12;
            SwapStr = 'M'
         elseif max(DSGEModel.annual)==4;
            SwapStr = 'Q';
         else;
            SwapStr = '';
         end;
         if DSGEModel.IsRecentPast==1;
            sStr = '';
            if DSGEModel.Results.tauT0>1;
               sStr = 's';
            end;
            DecompType = ['-Recent-Past-' int2str(DSGEModel.Results.tauT0) '-Lag' sStr '-'];
         else;
            DecompType = ['-Sample-Split-' strrep(DSGEModel.Results.SampleStrMat(DSGEModel.Results.tauT0,:),':',SwapStr) '-'];
         end;
         GraphDirectory = FixFilePath([DSGEModel.OutputDirectory '\graphs']);
         if DirExist(GraphDirectory)==0;
            stat = MakeDir(DSGEModel.OutputDirectory,'graphs');
         end;
         if isempty(findstr(lower(DSGEModel.NameStr),'mode'))==0;
            file = FixFilePath([GraphDirectory '\' CurrVariable DecompType DSGEModel.AnnualStr '-PosteriorMode.' CurrINI.graphicsfmt]);
         else;
            file = FixFilePath([GraphDirectory '\' CurrVariable DecompType DSGEModel.AnnualStr '-InitialValues.' CurrINI.graphicsfmt]);
         end;
         if strcmp(CurrINI.graphicsfmt,'eps')==1;
            saveas(FigHandle,file,'epsc');
         elseif strcmp(CurrINI.graphicsfmt,'ps')==1;
            saveas(FigHandle,file,'psc');
         else;
            saveas(FigHandle,file);
         end;
         %
         % save also as a fig file
         %
         saveas(FigHandle,strrep(file,['.' CurrINI.graphicsfmt],'.fig'));
      end;
      %
      % display the graph
      %
      SetToolbarImages(FigHandle,CurrINI.images);
      set(FigHandle,'Visible','on','CloseRequestFcn','delete(gcf); drawnow; pause(0.02);');
      drawnow;
      pause(0.02);
      %
      % check if we should decompose the shock groups into the individual shocks
      %
      if get(controls.decomposegroups,'Value')==1;
         if get(controls.useshockaliases,'Value')==0;
            AllShockNames = DSGEModel.StateShockNames(DSGEModel.Results.KeepVar,:);
         else;
            AllShockNames = DSGEModel.ShockAliases(DSGEModel.Results.KeepVar,:);
            for j=1:size(AllShockNames,1);
               AllShockNames(j,:) = strrep(AllShockNames(j,:),'_',' ');
            end;
         end;
         pos = [(CurrINI.scrsz(3)-750)/2 (CurrINI.scrsz(4)-500)/2];
         ShockMatrix = [];
         if isempty(DSGEModel.ShockColors)==0;
            AllShockColors = DSGEModel.ShockColors(DSGEModel.Results.KeepVar,:);
         else;
            if max(DSGEModel.ShockGroups(DSGEModel.Results.KeepVar))==length(DSGEModel.Results.KeepVar);
               AllShockColors = CreateDefaultColors(length(DSGEModel.Results.KeepVar));
            else;
               AllShockColors = CreateShockColors(DSGEModel.ShockGroups(DSGEModel.Results.KeepVar),DSGEModel.ShockGroupColors);
            end;
         end;
         %
         % go through the different shock groups
         %
         for i=1:size(ShockGroupNames,1);
            %
            % fix the figure position
            %
            pos = pos+[22 -22];
            if pos(1)+750>CurrINI.scrsz(3);
               pos(1) = 0;
            end;
            if pos(2)<=32;
               pos(2) = CurrINI.scrsz(4)-550;
            end;
            %
            % the group values are located in positions 1+i
            %
            GroupValues = Ymat(1+i,:);
            %
            % find out which shocks appear in group i
            %
            GroupShocks = (ShockGroups==i) .* (1:length(ShockGroups));
            GroupShocks = GroupShocks(GroupShocks>0);
            %
            ShockMat = zeros(length(GroupShocks),size(DSGEModel.Results.ObsRecent,2));
            for j=1:length(GroupShocks);
               ShockMat(j,:) = DSGEModel.Results.ObsRecent(VariableValue,:,GroupShocks(j));
            end;
            %
            % get the colors of the shocks
            %
            ShockColors = AllShockColors(GroupShocks,:);
            %
            % get the names of the shocks
            %
            ShockNames = AllShockNames(GroupShocks,:);
            %
            % create the figure
            %
            FigHandle = figure('Visible','off', ...
                               'Position',[pos 750 500], ...
                               'Tag',CurrVariable, ...
                               'Name',[DSGEModel.ConEstStr DSGEModel.HeaderStr ' - ' StringTrim(ShockGroupNames(i,:)) ' Shock Group - ' CurrVariable]);
            %
            FigureHandles = [FigureHandles FigHandle];
            SetFigureProperties(FigHandle,CurrINI);
            %
            % ensure that the handle to the current figure is correct (HG2 issue)
            %
            set(CurrINI.GraphicsRoot,'CurrentFigure',FigHandle);
            %
            % Create the bars and collect the handles
            %
            if MatlabNumber>8.3;
               BarHandles = bar(DateVector',ShockMat','stacked');
            else;
               if MatlabNumber>=7.0;
                  %
                  % make sure the bars are written as patches
                  %
                  BarHandles = bar('v6',DateVector',ShockMat','stacked');
               else;
                  BarHandles = bar(DateVector',ShockMat','stacked');
               end;
            end;
            %
            % remake the bar patches such that all bars are displayed properly.
            %
            BarHandles = RemakeBars(BarHandles,ShockColors,BlackEdgeBars);
            %
            % display the line with the combined shock groups values
            %
            LineHandle = plot(DateVector,GroupValues,'.-','Color',[0 0 0],'LineWidth',2);
            %
            % set the title
            %
            if strcmp(lower(DSGEModel.AnnualStr),'original')==1;
               SetAxesFonts(CurrINI,'Title',['Decomposition of recent ' lower(StringTrim(ShockGroupNames(i,:))) ' shock group for ' CurrVariable ' (original data) at ' lower(DSGEModel.NameStr)]);
            else;
               SetAxesFonts(CurrINI,'Title',['Decomposition of recent ' lower(StringTrim(ShockGroupNames(i,:))) ' shock group for ' CurrVariable ' (annual data) at ' lower(DSGEModel.NameStr)]);
            end;
            axis('tight');
            ylim = get(gca,'YLim');
            for h=1:length(BarHandles);
               if h==1;
                  YData = get(BarHandles(h),'YData');
                  if size(YData,1)~=4;
                     YData = YData';
                  end;
                  ylim(1) = min(min(YData(:,samplebeg:sampleend)));
                  ylim(2) = max(max(YData(:,samplebeg:sampleend)));
               else;
                  YData = get(BarHandles(h),'YData');
                  if size(YData,1)~=4;
                     YData = YData';
                  end;
                  ylim(1) = min([min(YData(:,samplebeg:sampleend)) ylim(1)]);
                  ylim(2) = max([max(YData(:,samplebeg:sampleend)) ylim(2)]);
               end;
            end;
            %
            % see if we should add a zero line
            %
            add_on = 0.1*(ylim(2)-ylim(1));
            if add_on==0;
               Ylim = get(gca,'YLim');
               add_on = 0.1*(Ylim(2)-Ylim(1));
            end;
            ylim = [ylim(1)-add_on ylim(2)+add_on];
            set(gca,'YLim',ylim);
            if (ylim(1)<0)&(ylim(2)>0);
               hold('on');
               plot([DateVector(1) DateVector(length(DateVector))],[0 0],'-','Color',[0 0 0]);
               %
               % make sure the zero line is displayed below the observed variable
               %
               kids = get(gca,'Children');
               if size(kids,1)>size(kids,2);
                  kids = kids';
               end;
               kids = [kids(2:length(kids)) kids(1)]';
               set(gca,'Children',kids);
            end;
            %
            % make sure the X axis limits follow the sample selection
            %
            set(gca,'XLim',[min(XData(:,samplebeg)) max(XData(:,sampleend))]);
            if MatlabNumber>=7;
               legHandle = legend([BarHandles LineHandle],strvcat(ShockNames,[StringTrim(ShockGroupNames(i,:)) ' shock group']),'Location','NorthWest');
            else;
               legHandle = legend([BarHandles LineHandle],strvcat(ShockNames,[StringTrim(ShockGroupNames(i,:)) ' shock group']),2);
            end;
            SetAxesFonts(CurrINI,legHandle);
            hold('off');
            %
            SetXTickIntegers(gca);
            %
            % check if we should save the figure to disk
            %
            if get(controls.saveplotstofile,'Value')==1;
               GraphDirectory = FixFilePath([DSGEModel.OutputDirectory '\graphs']);
               if isempty(findstr(lower(DSGEModel.NameStr),'mode'))==0;
                  file = FixFilePath([GraphDirectory '\' CurrVariable '-SingleShockGroup-' int2str(i) DecompType DSGEModel.AnnualStr '-PosteriorMode.' CurrINI.graphicsfmt]);
               else;
                  file = FixFilePath([GraphDirectory '\' CurrVariable '-SingleShockGroup-' int2str(i) DecompType DSGEModel.AnnualStr '-InitialValues.' CurrINI.graphicsfmt]);
               end;
               if strcmp(CurrINI.graphicsfmt,'eps')==1;
                  saveas(FigHandle,file,'epsc');
               elseif strcmp(CurrINI.graphicsfmt,'ps')==1;
                  saveas(FigHandle,file,'psc');
               else;
                  saveas(FigHandle,file);
               end;
               %
               % save also as a fig file
               %
               saveas(FigHandle,strrep(file,['.' CurrINI.graphicsfmt],'.fig'));
            end;
            %
            % display the graph
            %
            SetToolbarImages(FigHandle,CurrINI.images);
            set(FigHandle,'Visible','on','CloseRequestFcn','delete(gcf); drawnow; pause(0.02);');
            drawnow;
            pause(0.02);
         end;
      end;
   case 'done'
      set(controls.done,'UserData','done');
      delete(GraphGUI);
      drawnow;
      pause(0.02);
end;
%
if strcmp(selector,'init')==1;
   set(GraphGUI,'Visible','on');
   drawnow;
   SetYADAIcon(GraphGUI,CurrINI.ShowYADAIcon);
   if MatlabNumber>=7;
      uicontrol(controls.done);
   end;
   waitfor(controls.done,'UserData','done');
end;

%
% end of RecentPastDecompDLG.m
%