function [Status,ZLBYPaths,ZLBKsiPaths,KsitTh,PredData,PredEventData,BeliefStatus,TransStatus,SolutionStatus] = ZLBAdaptiveLearningDSGEPredictionPathsTheta(theta,thetaPositions,ModelParameters,BeliefSys,DSGEModel,CurrINI,X,h,NumPaths,AnnualizeData,TransData,NameStr)
% ZLBAdaptiveLearningDSGEPredictionPathsTheta: Calculates a certain number of possible paths that the endogenous variables
%                                              can take out-of-sample for a given value of the parameters in the DSGE model
%                                              and sibject to the ZLB.
%
% USAGE:
%
%       [Status,ZLBYPaths,ZLBKsiPaths,KsitTh,PredData,PredEventData,BeliefStatus,TransStatus,SolutionStatus] = ...
%                         ZLBAdaptiveLearningDSGEPredictionPathsTheta(theta,thetaPositions,ModelParameters,BeliefSys,...
%                                                                     DSGEModel,CurrINI,X,h,NumPaths,AnnualizeData,...
%                                                                     TransData,NameStr)
%
% REQUIRED INPUT:  theta (vector) with values of the original DSGE model parameters.
%
%                  thetaPositions (structure) with fields parameter and distribution. This function
%                                 needs the former field to know which parameter name corresponds to
%                                 position i in theta.
%
%                  ModelParameters (structure) whose fields have names given by the parameter names in the
%                                 DSGE model.
%
%                  BeliefSys (structure) with fields "m" an integer denoting the maximum number of lags
%                                 of the state variables in the belief equations; "Name" a string with the
%                                 name of the belief system; and G a cell array with f matrices, one for each
%                                 forward looking variable of dimension (k+r*m) x g_j where g_j is the number
%                                 of explanatory variables in the belief equation for forward looking variable
%                                 j. If empty, the function will compute the structure itself.
%
%                  DSGEModel (structure) with fields containing all the information about a DSGE Model for YADA.
%
%                  CurrINI (structure), with initialization information.
%
%                  X (matrix) of dimension kxMaxPH with the observations of the exogenous variables for
%                                 the prediction paths.
%
%                  h (integer) with the prediction horizon.
%
%                  NumPaths (integer)the number of paths to draw.
%
%                  AnnualizeData (boolean) that is 1 if the prediction paths should be annualized and 0 otherwise.
%
%                  TransData (boolean) that is 1 if the prediction paths should be transformed and 0 otherwise.
%
%                  NameStr (string vector) that is sent to the progress dialog name after the string
%                                 "Predictive Distribution".
%
% REQUIRED OUTPUT: Status (boolean) that takes the value 1 if the calculations were successful and 0 otherwise.
%
%                  ZLBYPaths (matrix) of dimension nxhxNumPaths with zero lower bound consistent prediction
%                                 paths for the observed variables.
%
%                  ZLBKsiPaths (matrix) of dimension rxhxNumPaths with zero lower bound consistent prediction
%                                 paths for the state variables.
%
%                  KsitTh (matrix) of dimension rxTh with the smooth estimates of the state variables taking
%                                 data from the prediction sample into account.
%
%                  PredData (structure) with fields:
%                                 PredMean   - (nxh) matrix with the mean predictions of the observed
%                                              variables.
%                                 etaShocks  - (qxh) matrix with the values of the shocks at the mean
%                                              predictions.
%                                 xiStates   - (rxh) matrix with the values of the states at the mean
%                                              predictions.
%                                 X          - (kxh) matrix with the deterministic variables for the
%                                              prediction horizon.
%                                 etaMean    - (qxh) matrix with the mean values for the economic shocks over
%                                              the forecast period.
%                                 wMean      - (nxh) matrix with the mean values for the measurement errors over
%                                              the forecast period.
%                                 A          - (kxn) matrix with the parameters on the exogenous variables in
%                                              the measurement equation.
%                                 H          - (rxn) matrix with parameters on the state variabels in the
%                                              measurement equation.
%                                 R          - (nxn) covariance matrix for the measurement errors.
%                                 MT1        - (rxk) or (rxkxh) matrix with parameters on the deterministic
%                                              variables in the state equation over the forecast horizon.
%                                              The 3rd dimension appears when beliefs are updated over the
%                                              forecast horizon.
%                                 FT1        - (rxr) or (rxrxh) matrix with parameters on the state variables
%                                              in the state equation over the forecast horizon.
%                                 BT1        - (rxq) or (rxqxh) matrix with parameters on the structural shocks
%                                              in the state equation over the forecast horizon.
%                                 ShockNames - (qxqstr) string matrix with the names of the economic shocks.
%                                 Shocks     - (qxhxNumPaths) matrix with the economic shocks for the paths.
%                                 MeasError  - (nxhxNumPaths) matrix with the measurement errors for the paths.
%                                 HistState  - (rxNumPaths) matrix with the time T period draw of the state
%                                              vector for the paths.
%                                 KernelX    - (3D matrix) of dimension n x 2^8 x h with the x-axis values
%                                              from kernel density estimation of the marginal predictive
%                                              densities.
%                                 KernelY    - (3D matrix) of dimension n x 2^8 x h with the y-axis values
%                                              from kernel density estimation of the marginal predictive
%                                              densities.
%
%                  PredEventData (n x 8) matrix where the first column holds the number of times
%                                 the ZLBYPaths data satisfies the event, the second holds the number
%                                 of times the ZLBYPaths data is greater than the event, the third
%                                 the number of time the data is above the upper bound, while the
%                                 fourth column holds the number of times the event has been tested.
%                                 The fifth and sixth column measure the average deviation from the
%                                 lower bound when below the lower bound and the square of this, while
%                                 the seventh and eighth columns hold similar data for the upper bound
%                                 of the event when the variables are above the upper bound.
%
% OPTIONAL OUTPUT: BeliefStatus (boolean) which is 1 if the Kalman filter of the belief system runs without
%                        error, and 0 otherwise.
%
%                  TransStatus (boolean) which takes the value 1 if the rewrite of the structural form of
%                        the DSGE model was successful and 0 otherwise.
%
%                  SolutionStatus (integer) Indicates whether the solution of the model is unique or not.
%                                 mcode is 1 when unique, 0 if not computed, and various other values when
%                                 there are problems with the solution. Specifically:
%
%                                      mcode = 2:      Roots not correctly computed by real_schur
%                                      mcode = 3:      Too many big roots
%                                      mcode = 35:     Too many big roots, and q(:,right) is singular
%                                      mcode = 4:      Too few big roots
%                                      mcode = 45:     Too few big roots, and q(:,right) is singular
%                                      mcode = 5:      q(:,right) is singular
%                                      mcode = 61:     Too many exact shiftrights
%                                      mcode = 62:     Too many numeric shiftrights
%                                      mcode = 7:      The a matrix has infinite or NaN values. Eigenvalues
%                                                      cannot be calculated.
%                                      mcode = 8:      The function "compute_aim_matrices" returns complex
%                                                      numbers.
%                                      mcode = -1:     No stable solution.
%                                      mcode = -2:     Too many large generalized eigenvalues.
%                                      mcode = -3:     Too few large generalized eigenvalues.
%                                      else            Return code not properly specified
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2025 European Central Bank.
%
%                       First version: November 26, 2024.
%                        This version: January 8, 2025.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 27-11-2024: Continued after the mean predictions under the ZLB.
%
% * 03-12-2024: Made sure that the original value of ZLBYMean is used for the test.
%
% * 13-12-2024: Made sure that ModelParameters is updated.
%
% * 08-01-2025: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
ZLBYPaths = [];
ZLBKsiPaths = [];
KsitTh = [];
PredData = [];
PredEventData = [];
Status = 0;
n = DSGEModel.n;
k = DSGEModel.k;
if (AnnualizeData==1)|(TransData==1);
   if TransData==0;
      TypeStr = 'Annualized';
   else;
      TypeStr = 'Transformed';
   end;
else;
   TypeStr = 'Original';
end;
if nargout>6;
   BeliefStatus = 1;
end;
if nargout>7;
   TransStatus = 1;
end;
%
if nargout<=8;
   %
   % Make sure that the ModelParameter structure takes the correct values.
   % This is not necessary to do when initial values are given to the function.
   % In that case, more substantial checking is required!
   %
   ModelParameters = ThetaToModelParameters(ModelParameters,theta,thetaPositions);
   %
   % copy files to the tmp folder
   %
   [stat,msg] = CopyFile([GetPath(DSGEModel.AIMDataFile) 'compute_aim_matrices.m'],[pwd '\tmp']);
   [stat,msg] = CopyFile(DSGEModel.MeasurementEquationFile,[pwd '\tmp']);
   if FileExist(DSGEModel.UpdateParameterFile)==1;
      [stat,msg] = CopyFile(DSGEModel.UpdateParameterFile,[pwd '\tmp']);
   end;
   if FileExist(DSGEModel.SystemPriorFile)==1;
      [stat,msg] = CopyFile(DSGEModel.SystemPriorFile,[pwd '\tmp']);
   end;
   %
   % check if we should copy model solver file
   %
   if DSGEModel.ModelSolver==4;
      if FileExist(DSGEModel.ModelSolverFile)==1;
         [stat,msg] = CopyFile(DSGEModel.ModelSolverFile,[pwd '\tmp']);
      end;
      if FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==1;
         [stat,msg] = CopyFile(DSGEModel.AdaptiveLearningModelSolverFile,[pwd '\tmp']);
      end;
   end;
   %
   % Update any parameteters that need updating
   %
   if FileExist(DSGEModel.UpdateParameterFile)==1;
      ModelParameters = feval(GetName(DSGEModel.UpdateParameterFile),ModelParameters);
   end;
else;
   SolutionStatus = 1;
end;
%
% check if we should also copy the belief system file
%
if DSGEModel.BeliefSystem==2;
   [stat,msg] = CopyFile(DSGEModel.BeliefSystemFile,[pwd '\tmp']);
end;
%
% load the AIMData file
%
ErrorStr = '';
if MatlabNumber>=8.0;
   try;
      AIMData = load(DSGEModel.AIMDataFile);
   catch ME;
      ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError(ME)];
   end;
else;
   try;
      AIMData = load(DSGEModel.AIMDataFile);
   catch;
      ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError];
   end;
end;
if isempty(ErrorStr)==0;
   if nargout>6;
      status = 0;
   end;
   About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
   return;
end;
%
% create the output directory
%
stat = MakeDir(DSGEModel.OutputDirectory,'predictions');
if stat~=1;
   if ispc==1;
      txt = ['YADA was for some reason unable to create the directory "' DSGEModel.OutputDirectory '\predictions". The computation of predictive distributions has therefore been aborted.'];
   else;
      txt = ['YADA was for some reason unable to create the directory "' DSGEModel.OutputDirectory '/predictions". The computation of predictive distributions has therefore been aborted.'];
   end;
   About(txt,'information','YADA - Directory Creation Problem',180,600,CurrINI);
   return;
end;
%
% set the InitialStateVector value
%
if DSGEModel.UseOwnInitialState==1;
   if length(DSGEModel.InitialStateValues)==size(AIMData.endog,1);
      DSGEModel.InitialStateVector = DSGEModel.InitialStateValues(DSGEModel.StateVariablePositions);
   else;
      DSGEModel.InitialStateVector = zeros(length(DSGEModel.StateVariablePositions),1);
   end;
else;
   DSGEModel.InitialStateVector = zeros(length(DSGEModel.StateVariablePositions),1);
end;
%
% Get the sub-sample
%
[DSGEModel.FirstPeriod,DSGEModel.LastPeriod] = CreateSubSample(DSGEModel);
FirstPeriod = DSGEModel.FirstPeriod;
LastPeriod = DSGEModel.LastPeriod;
%
% See if we can solve the model
%
%
% try to solve the model for the posterior mode estimates
%
ErrorStr = '';
if MatlabNumber>=8.0;
   try;
      [A,H,R,F,B0,mcode,ErrMsg,ModelParameters] = SolveDSGEModel(DSGEModel,theta,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
      if isempty(ErrMsg)==0;
         ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' ErrMsg];
      end;
   catch ME;
      ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' YADALastError(ME)];
   end;
else;
   try;
      [A,H,R,F,B0,mcode,ErrMsg,ModelParameters] = SolveDSGEModel(DSGEModel,theta,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
      if isempty(ErrMsg)==0;
         ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' ErrMsg];
      end;
   catch;
      ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' YADALastError];
   end;
end;
if nargout>8;
   SolutionStatus = mcode;
end;
if isempty(ErrorStr)==0;
   About(ErrorStr,'information','YADA - Error Message',CurrINI.scrsz(4)-50,600,CurrINI);
   return;
end;
if mcode~=1;
   if mcode==2;
      mcodeStr = 'Roots are not correctly computed by real_schur.'
   elseif mcode==3;
      mcodeStr = 'Too many big roots.';
   elseif mcode==35;
      mcodeStr = 'Too many big roots, and q(:,right) is singular.';
   elseif mcode==4;
      mcodeStr = 'Too few big roots.';
   elseif mcode==45;
      mcodeStr = 'Too few big roots, and q(:,right) is singular.';
   elseif mcode==5;
      mcodeStr = 'q(:,right) is singular.';
   elseif mcode==61;
      mcodeStr = 'Too many exact shiftrights.';
   elseif mcode==62;
      mcodeStr = 'Too many numeric shiftrights.';
   elseif mcode==7;
      mcodeStr = 'Infinite or NaN values detected.';
   elseif mcode==8;
      mcodeStr = 'The function "compute_aim_matrices" returns complex numbers.';
   elseif mcode==-1;
      mcodeStr = 'No stable solution.';
   elseif mcode==-2;
      mcodeStr = 'Too many large generalized eigenvalues.';
   elseif mcode==-3;
      mcodeStr = 'Too few large generalized eigenvalues.';
   else;
      mcodeStr = 'Return code not properly specified.';
   end;
   if DSGEModel.ModelSolver==1;
      SolverStr = 'AiM';
   elseif DSGEModel.ModelSolver==2;
      SolverStr = 'Klein';
   elseif DSGEModel.ModelSolver==3;
      SolverStr = 'Gensys';
   elseif DSGEModel.ModelSolver==4;
      SolverStr = ['Own algorithm (' GetName(DSGEModel.ModelSolverFile) ')'];
   end;
   txt = ['The ' SolverStr ' solver provided the return code: ' num2str(mcode) ', i.e., "' mcodeStr '"'];
   About(txt,'information',['YADA - ' SolverStr ' Solver Error'],500,600,CurrINI);
   return;
end;
if length(size(H))==3;
   if size(H,3)<DSGEModel.LastPeriod+h;
      %
      % too few data points in the 3D H matrix.
      %
      ErrorStr = ['The number of time periods for the time-varying measurement matrix H (' int2str(size(H,3)) ') is lower than the number of observations plus forecasts (T+h = ' int2str(DSGEModel.LastPeriod+h) '). YADA has therefore aborted from the unconditional predictions.'];
      About(ErrorStr,'error','Error - Measurement Equation',500,600,CurrINI);
      return;
   end;
end;
[S,KeepShocks] = RemoveRedundantColumns(B0);
if isempty(S)==1;
   txt = 'The model doesn''t have any structural shocks. YADA has aborted from the lower bound computations.';
   About(txt,'information',['Zero Lower Bound Forecasts - ' NameStr],200,600,CurrINI);
   return;
end;
[r,q] = size(B0);
%
% test if the measurement equation for the policy rate maps to the correct state variables only
%
if length(size(H))==2;
   ZLBLink = H(DSGEModel.RtildePosition,DSGEModel.ZLBPosition);
   NonZLBLink = H(setdiff((1:r),DSGEModel.RtildePosition),DSGEModel.ZLBPosition);
   if r>1;
      TestValue = isequal(NonZLBLink,zeros(r-1,1));
   else;
      TestValue = isempty(NonZLBLink);
   end;
   txt = '';
   BadValue = 0;
   if ZLBLink==0;
      txt = ['The observed monetary policy interest rate (' StringTrim(DSGEModel.VariableNames(DSGEModel.ZLBPosition,:)) ') doesn''t map to the claimed unrestricted monetary policy rate (' StringTrim(DSGEModel.StateVariableNames(DSGEModel.RtildePosition,:)) ') in the DSGE model. YADA is therefore unable to use the lower bound and the steady-state interest rate and will abort.'];
      BadValue = 1;
   elseif TestValue==0;
      txt = ['The measurement equation for ' StringTrim(DSGEModel.VariableNames(DSGEModel.ZLBPosition,:)) ' maps to other state variables than the claimed unrestricted monetary policy rate (' StringTrim(DSGEModel.StateVariableNames(DSGEModel.RtildePosition,:)) ') in the DSGE model. Are you sure you want to use this mapping?'];
   end;
   if isempty(txt)==0;
      if BadValue==1;
         About(txt,'information',['Zero Lower Bound Forecasts - ' NameStr],300,600,CurrINI);
         return;
      else;
         answer = Query(txt,'question',300,['Zero Lower Bound Forecasts - ' NameStr],600,'no',CurrINI);
         if strcmp(lower(answer),'no')==1;
            return;
         end;
      end;
   end;
else;
   Hlength = size(H,3);
   ZLBLink = shiftdim(H(DSGEModel.RtildePosition,DSGEModel.ZLBPosition,DSGEModel.LastPeriod:Hlength),2);
   ZLBLTest = min(ZLBLink~=zeros(Hlength-DSGEModel.LastPeriod+1,1));
   NonZLBLink = shiftdim(H(setdiff((1:r),DSGEModel.RtildePosition),DSGEModel.ZLBPosition,DSGEModel.LastPeriod:Hlength),2);
   if r>1;
      TestValue = isequal(NonZLBLink,zeros(Hlength-DSGEModel.LastPeriod+1,r-1))
   else;
      TestValue = isempty(NonZLBLink);
   end;
   txt = '';
   BadValue = 0;
   if ZLBLink==0;
      txt = ['The observed monetary policy interest rate (' StringTrim(DSGEModel.VariableNames(DSGEModel.ZLBPosition,:)) ') doesn''t map to the claimed unrestricted monetary policy rate (' StringTrim(DSGEModel.StateVariableNames(DSGEModel.RtildePosition,:)) ') in the DSGE model. YADA is therefore unable to use the lower bound and the steady-state interest rate and will abort.'];
      BadValue = 1;
   elseif TestValue==0;
      txt = ['The measurement equation for ' StringTrim(DSGEModel.VariableNames(DSGEModel.ZLBPosition,:)) ' maps to other state variables than the claimed unrestricted monetary policy rate (' StringTrim(DSGEModel.StateVariableNames(DSGEModel.RtildePosition,:)) ') in the DSGE model for some time periods. Are you sure you want to use this mapping?'];
   end;
   if isempty(txt)==0;
      if BadValue==1;
         About(txt,'information',['Zero Lower Bound Forecasts - ' NameStr],300,600,CurrIN);
         return;
      else;
         answer = Query(txt,'question',300,['Zero Lower Bound Forecasts - ' NameStr],600,'no',CurrINI);
         if strcmp(lower(answer),'no')==1;
            return;
         end;
      end;
   end;
end;
PredData.A = A;
PredData.H = H;
PredData.R = R;
PredData.F = F;
PredData.B0 = B0;
%
% determine the steady-state interest rate path.
%
if k>0;
   XPred = X;
   if size(DSGEModel.X,2)>DSGEModel.LastPeriod;
      X = DSGEModel.X(:,DSGEModel.LastPeriod+1:size(DSGEModel.X,2));
      LastValue = X(:,size(X,2));
   else;
      X = [];
      LastValue = DSGEModel.X(:,DSGEModel.LastPeriod);
   end;
   if isempty(DSGEModel.PredictedX)==0;
      X = [X DSGEModel.PredictedX];
   end;
   Tx = size(X,2);
   Xold = X;
   if Tx>DSGEModel.ZLBSampleT+1;
      X = X(:,1:DSGEModel.ZLBSampleT+1);
   elseif Tx<DSGEModel.ZLBSampleT+1;
      X = [X LastValue*ones(1,DSGEModel.ZLBSampleT+1-Tx)];
   end;
   Rbar = A(:,DSGEModel.ZLBPosition)'*X;
else;
   Rbar = zeros(1,DSGEModel.ZLBSampleT+1);
   X = zeros(1,DSGEModel.ZLBSampleT+1)
end;
if size(X,2)<h;
   X = [X XPred(:,DSGEModel.ZLBSampleT+2:h)];
end;
%
% setup the zero lower bound path
%
Tzlb = length(DSGEModel.ZLBData);
if Tzlb==1;
   Rzlb = DSGEModel.ZLBData*ones(h,DSGEModel.ZLBSampleT+1);
else;
   if Tzlb>DSGEModel.LastPeriod;
      Rzlb = zeros(h,Tzlb-DSGEModel.LastPeriod);
      for i=1:h;
         Rzlb(i,1:Tzlb-DSGEModel.LastPeriod-i+2) = DSGEModel.ZLBData(DSGEModel.LastPeriod+i-1:Tzlb);
      end;
      TZLB = size(Rzlb,2);
      if TZLB>=DSGEModel.ZLBSampleT+1;
         RzlbLast = Rzlb(1,size(Rzlb,2));
         % make the matrix have the correct dimension
         Rzlb = Rzlb(:,1:DSGEModel.ZLBSampleT+1);
         % replace zeros with the last value of the lower bound
         % provided the locations appear after the last nonzero
         for i=1:h;
            for j=2:DSGEModel.ZLBSampleT+1;
               if (Rzlb(i,j)==0)&(Rzlb(i,j-1)~=0);
                  Rzlb(i,j) = RzlbLast;
               end;
            end;
         end;
         %Rzlb((Rzlb==0)) = RzlbLast;
      else;
         %
         % we should add the last value in each row to fill the matrix
         %
         RzlbLast = Rzlb(1,size(Rzlb,2));
         Rzlb((Rzlb==0)) = RzlbLast;
         Rzlb = [Rzlb RzlbLast*ones(h,DSGEModel.ZLBSampleT+1-TZLB)];
      end;
   else;
      Rzlb = DSGEModel.ZLBData(Tzlb)*ones(h,DSGEModel.ZLBSampleT+1);
   end;
end;
%
% since H may involve estimated parameters we save a copy of Rzlb for later use
%
RzlbMat = Rzlb;
%
% Transform the steady-state interest rate and the lower bound to the same units
% as the unrestricted monetary policy rate
%
if length(size(H))==2;
   Hvalue = H(DSGEModel.RtildePosition,DSGEModel.ZLBPosition);
   Rzlb = (1/Hvalue)*Rzlb;
   Rbar = (1/Hvalue)*Rbar;
else;
   Hvalue = shiftdim(H(DSGEModel.RtildePosition,DSGEModel.ZLBPosition,DSGEModel.LastPeriod:Hlength),2)';
   Th = length(Hvalue);
   if Th>DSGEModel.ZLBSampleT;
      Hvalue = Hvalue(1,1:DSGEModel.ZLBSampleT);
   elseif Th<DSGEModel.ZLBSampleT;
      Hvalue = [Hvalue Hvalue(1,Th)*ones(1,DSGEModel.ZLBSampleT-Th)];
   end;
   Rzlb = Rzlb./Hvalue;
   Rbar = Rbar./Hvalue;
end;
%
% setup the parameters that determine the projected restricted policy rate
%
e_pr = zeros(r+1,1);
e_pr(DSGEModel.RtildePosition) = 1;
%
% Determine the belief system matrices
%
if DSGEModel.BeliefSystem==1;
   BeliefSys = YADADefaultBeliefSystem(DSGEModel.X,DSGEModel.XVariableNames,DSGEModel.StateVariableNames,DSGEModel.ForwardVariableNames);
else;
   BeliefSys = feval(GetName(DSGEModel.BeliefSystemFile),DSGEModel.XVariableNames,DSGEModel.StateVariableNames,DSGEModel.ForwardVariableNames);
end;
%
% Compute the transformation of the structural from that we use when solving the model
%
r = length(DSGEModel.StateVariablePositions);
f = length(DSGEModel.ForwardVariablePositions);
m = BeliefSys.m;
q = size(B0,2);
%
% get the structural form matrices
%
[Hlag,H0,Hexp,D] = DSGEStructuralForm(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions);
%
% reshuffle the equations to separate the restricted from the unrestricted policy rate
%
[Hlag,H0,Hexp,D,DA,MA] = ZLBDSGEStructuralForm(Hlag,H0,Hexp,D,DSGEModel.ZLBSampleT,DSGEModel.RtildePosition,DSGEModel.REqPosition,DSGEModel.ZLBRuleType);
%
% solve the model with the new matrices
%
[F,B0,BA,mcode] = SolveExtendedDSGEModel(Hlag,H0,Hexp,D,DA,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
%
% rewrite the G-matrices for the belief equations that match the position of the unrestricted policy rate being last and the restricted policy rate
% being in position "DSGEModel.RtildePosition". The latter variable should have zeros in the belief equations, while the former should have the
% coefficients from "DSGEModel.RtildePosition"
%
G = ZLBBeliefSystem(BeliefSys.G,m,k,r,DSGEModel.RtildePosition,DSGEModel.ZLBRuleType);
%
SMatrix = eye(r+1);
Sperp = SMatrix(:,setdiff((1:r+1),DSGEModel.ForwardVariablePositions));
SMatrix = SMatrix(:,DSGEModel.ForwardVariablePositions);
%
% Determine the first observation to use when computing log-likelihood
%
KalmanFirstObservation = DSGEModel.KalmanFirstObservation;
%
% Setup all the basic data matrix for the deterministic variables
% used for initializing Sigmax
%
if DSGEModel.k>0
   x = DSGEModel.X(:,FirstPeriod+KalmanFirstObservation-1:LastPeriod);
else;
   x = [];
end;
T = LastPeriod-(FirstPeriod+KalmanFirstObservation-1)+1;
%
% Compute the initial values of betavec, Sigmau, R10 and Sigmaeps
%
[betavec,Sigmau,R10,Sigmaeps,Gtilde,Kgf] = AdaptiveLearningInitialValuesDSGE(F,B0,ModelParameters.sigmarAL,ModelParameters.sigmaeAL,SMatrix,G,m,x,f,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue);
%
% call the transformation function
%
[tstatus,Hstarlag,Hstar0,Hstar1f] = AdaptiveLearningTransformDSGEModel(Hlag,H0,Hexp,SMatrix,Sperp);
if nargout>7;
   TransStatus = tstatus;
   if TransStatus==0;
      return;
   end;
end;
%
% compute initial values for the solution matrices
%
if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
   [Mt1,Ft1,Bt1,InvH0H1star] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betavec);
else;
   [Mt1,Ft1,Bt1,InvH0H1star] = feval(GetName(DSGEModel.AdaptiveLearningModelSolverFile),Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betavec,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
end;
%
% compute the solution matrix BAt1 on the anticipated shocks
%
BAt1 = InvH0H1star*DA;
%
ConstPos = 0;
if k>0;
   ConstPos = LocateStr(DSGEModel.XVariableNames,'const');
   if ConstPos==0;
      %
      % we will now check if X has a unit vector
      %
      Xhist = DSGEModel.X(:,FirstPeriod:LastPeriod);
      T = size(Xhist,2);
      for j=1:k;
         ConstTest = (Xhist==ones(1,T));
         if sum(ConstTest)==T;
            ConstPos = j;
            break;
         end;
      end;
   end;
   xbar = zeros(k,1);
   xbar(ConstPos) = 1;
else;
   xbar = [];
end;
%
% we need to fix the H-matrix when using the extended form of the model
% by adding zeros to 
%
HOld = H;
if length(size(H))==2;
   H = [H;zeros(1,n)];
else;
   HT = zeros(size(H));
   for t=1:size(H,3);
      HT(:,:,t) = [H(:,:,t);zeros(1,n)];
   end;
   H = HT;
end;
%
% check if the adaptive learning parameters allows for updating of the beliefs
%
IsALUpdate = (CheckAdaptiveLearningFixedBeliefs(ModelParameters)==0);
%
% run the Kalman filter
%
if MatlabNumber>=8.0;
   try;
      if IsALUpdate==1;
         if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
            [lnL,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningKalmanFilter(m,SMatrix,betavec,Sigmau,R10,Sigmaeps,ModelParameters.rhoAL,Hstar0,Hstarlag,Hstar1f,D,Mt1,Ft1,Bt1,Gtilde,Kgf,DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,KalmanFirstObservation,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs);
         else;
            [lnL,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningUserSolverKalmanFilter(GetName(DSGEModel.AdaptiveLearningModelSolverFile),m,SMatrix,betavec,Sigmau,R10,Sigmaeps,ModelParameters.rhoAL,Hstar0,Hstarlag,Hstar1f,D,Mt1,Ft1,Bt1,Gtilde,Kgf,DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,KalmanFirstObservation,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
         end;
      else;
         [lnL,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningSingleKalmanFilter(m,betavec,Mt1,Ft1,Bt1,DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,KalmanFirstObservation,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs);
         LastSolution.InvMatT1 = InvH0H1star;
         LastSolution.InvMatT = InvH0H1star;
      end;
   catch ME;
      if nargout>6;
         BeliefStatus = bstatus;
      end;
      txt = ['YADA caught the following error message from the Kalman filter routine under adaptive learning: ' YADALastError(ME)];
      About(txt,'error','Error - Kalman Filter',200,500,CurrINI);
      return;
   end;
else;
   try;
      if IsALUpdate==1;
         if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
            [lnL,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningKalmanFilter(m,SMatrix,betavec,Sigmau,R10,Sigmaeps,ModelParameters.rhoAL,Hstar0,Hstarlag,Hstar1f,D,Mt1,Ft1,Bt1,Gtilde,Kgf,DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,KalmanFirstObservation,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs);
         else;
            [lnL,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningUserSolverKalmanFilter(GetName(DSGEModel.AdaptiveLearningModelSolverFile),m,SMatrix,betavec,Sigmau,R10,Sigmaeps,ModelParameters.rhoAL,Hstar0,Hstarlag,Hstar1f,D,Mt1,Ft1,Bt1,Gtilde,Kgf,DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,KalmanFirstObservation,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
         end;
      else;
         [lnL,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningSingleKalmanFilter(m,betavec,Mt1,Ft1,Bt1,DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,KalmanFirstObservation,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs);
         LastSolution.InvMatT1 = InvH0H1star;
         LastSolution.InvMatT = InvH0H1star;
      end;
   catch;
      if nargout>6;
         BeliefStatus = bstatus;
      end;
      txt = ['YADA caught the following error message from the Kalman filter routine under adaptive learning: ' YADALastError];
      About(txt,'error','Error - Kalman Filter',200,500,CurrINI);
      return;
   end;
end;
%
% determine initial conditions for the forecasts
%
T = size(Ksitt,2);
KsiTT = Ksitt(:,T);
PTT = Ptt(:,:,T);
PTT = diag(abs(diag(PTT)))+tril(PTT,-1)+tril(PTT,-1)';
%
[S,L] = SchurDecomposition(PTT);
%
% compute KsitTh
%
if DSGEModel.T>=LastPeriod;
   %
   % run the filter at most until LastPeriod+h
   %
   hbar = min(h,DSGEModel.T-LastPeriod);
   if IsALUpdate==1;
      if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
         [lnLh,bstatush,lnLth,Yerrorh,Yhath,MSEYh,InvMSEYh,Ksitt1h,Ptt1h,Ksitth,Ptth,betatth,Zlagh,Mtildeh,Ftildeh,Btildeh,LastSolutionh] = AdaptiveLearningKalmanFilter(m,SMatrix,betavec,Sigmau,R10,Sigmaeps,ModelParameters.rhoAL,Hstar0,Hstarlag,Hstar1f,D,Mt1,Ft1,Bt1,Gtilde,Kgf,DSGEModel.Y,DSGEModel.X,FirstPeriod,LastPeriod+hbar,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,1,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs);
      else;
         [lnLh,bstatush,lnLth,Yerrorh,Yhath,MSEYh,InvMSEYh,Ksitt1h,Ptt1h,Ksitth,Ptth,betatth,Zlagh,Mtildeh,Ftildeh,Btildeh,LastSolutionh] = AdaptiveLearningUserSolverKalmanFilter(GetName(DSGEModel.AdaptiveLearningModelSolverFile),m,SMatrix,betavec,Sigmau,R10,Sigmaeps,ModelParameters.rhoAL,Hstar0,Hstarlag,Hstar1f,D,Mt1,Ft1,Bt1,Gtilde,Kgf,DSGEModel.Y,DSGEModel.X,FirstPeriod,LastPeriod+hbar,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,1,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
      end;
   else;
      [lnLh,bstatush,lnLth,Yerrorh,Yhath,MSEYh,InvMSEYh,Ksitt1h,Ptt1h,Ksitth,Ptth,betatth,Zlagh,Mtildeh,Ftildeh,Btildeh,LastSolutionh] = AdaptiveLearningSingleKalmanFilter(m,betavec,Mt1,Ft1,Bt1,DSGEModel.Y,DSGEModel.X,FirstPeriod,LastPeriod+hbar,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,1,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs);
   end;
   %
   % now apply the smoother
   %
   [KsitTh,PtTh,etatth,etatTh,wtth,wtTh] = AdaptiveLearningKalmanSmoother(Ksitt1h,Ptt1h,Yerrorh,InvMSEYh,H,R,Ftildeh,Btildeh,FirstPeriod,LastPeriod+hbar,1,DSGEModel.YNaNs);
   %
   % make sure that the extra row is skipped
   %
   KsitTh = KsitTh(1:r,:);
   if hbar<h;
      KsitTh = [KsitTh NaN*ones(r,h-hbar)];
   end;
else;
   %
   % the are no additional data points
   %
   [KsitTh,PtTh,etatth,etatTh,wtth,wtTh] = AdaptiveLearningKalmanSmoother(Ksitt1,Ptt1,Yerror,InvMSEY,H,R,Ftilde,Btilde,FirstPeriod,LastPeriod,1,DSGEModel.YNaNs);
   %
   % make sure that the extra row is skipped
   %
   KsitTh = [KsitTh(1:r,:) NaN*ones(r,h)];
end;
%
% deal with seeting up random draws of measurement errors
%
if rank(R)>0;
   R(:,DSGEModel.ZLBPosition) = 0;
   R(DSGEModel.ZLBPosition,:) = 0;
   [Rm,Lm] = SchurDecomposition(R);
   Rmeasure = Rm*sqrt(Lm);
   NumMeasure = size(Lm,1);
else;
   Rmeasure = [];
   wTh = 0;
   NumMeasure = 0;
end;
%
% set up the data matrices
%
if isempty(DSGEModel.X)==0;
   Xdata = DSGEModel.X(:,FirstPeriod+KalmanFirstObservation-1:LastPeriod);
else;
   Xdata = [];
end;
if strcmp(DSGEModel.DataFrequency,'q')==1;
   Yinit = DSGEModel.Y(:,LastPeriod-2:LastPeriod);
elseif strcmp(DSGEModel.DataFrequency,'m')==1;
   Yinit = DSGEModel.Y(:,LastPeriod-10:LastPeriod);
else;
   Yinit = [];
end;
maxYinit = max(max(isnan(Yinit)));
if DSGEModel.RunPredictionEvent==1;
   PredEventData = zeros(n,8);
end;
%
if TransData==1;
   Yinitial = DSGEModel.Y(:,1:DSGEModel.LastPeriod);
   %
   % try to covert: h
   %
   YTmp = [Yinitial DSGEModel.Y(:,DSGEModel.LastPeriod)*ones(1,h)];
   [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,YTmp',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
   Tt = size(YData,1);
   maxYinitial = max(max(isnan(Yinitial)));
else;
   maxYinitial = [];
   Yinitial = [];
   Tt = [];
end;
%
% deal with possible missing observations in the historical sample if we
% should annualize or transform the predictions
%
if AnnualizeData==1;
   if maxYinit==1;
      %
      % take into account that there are missing obs for the
      % historical data which can affect annualization
      %
      YSmooth = DSGEModel.Y(:,FirstPeriod+KalmanFirstObservation-1:LastPeriod);
      nanpos = isnan(YSmooth);
      %
      % compute the Kalman smoother
      %
      [KsitT,PtT,etatt,etatT,wtt,wtT] = AdaptiveLearningKalmanSmoother(Ksitt1,Ptt1,Yerror,InvMSEY,H,R,Ftilde,Btilde,FirstPeriod,LastPeriod,KalmanFirstObservation,DSGEModel.YNaNs);
      if length(size(H))==2;
         if k>0;
            YHistPred = (A'*Xdata)+(H'*KsitT);
         else;
            YHistPred = H'*KsitT;
         end;
      else;
         if k>0;
            for t=1:T;
               YHistPred = (A'*Xdata(:,t))+(H(:,:,FirstPeriod+KalmanFirstObservation-2+t)'*KsitT(:,t));
            end;
         else;
            for t=1:T;
               YHistPred = H(:,:,FirstPeriod+KalmanFirstObservation-2+t)'*KsitT(:,t);
            end;
         end;
      end;
      YSmooth(nanpos) = YHistPred(nanpos);
      if FirstPeriod+KalmanFirstObservation-1>1;
         YSmooth = [DSGEModel.Y(:,1:FirstPeriod+KalmanFirstObservation-2) YSmooth];
      end;
      if strcmp(DSGEModel.DataFrequency,'q')==1;
         Yinit = YSmooth(:,LastPeriod-2:LastPeriod);
      elseif strcmp(DSGEModel.DataFrequency,'m')==1;
         Yinit = YSmooth(:,LastPeriod-10:LastPeriod);
      else;
         Yinit = [];
      end;
   end;
elseif TransData==1;
   if maxYinitial==1;
      %
      % take into account that there are missing obs for the
      % historical data which can affect general transformations
      %
      YSmooth = DSGEModel.Y(:,FirstPeriod+KalmanFirstObservation-1:LastPeriod);
      nanpos = isnan(YSmooth);
      %
      % compute the Kalman smoother
      %
      [KsitT,PtT,etatt,etatT,wtt,wtT] = AdaptiveLearningKalmanSmoother(Ksitt1,Ptt1,Yerror,InvMSEY,H,R,Ftilde,Btilde,FirstPeriod,LastPeriod,KalmanFirstObservation,DSGEModel.YNaNs);
      if length(size(H))==2;
         if k>0;
            YHistPred = (A'*Xdata)+(H'*KsitT);
         else;
            YHistPred = H'*KsitT;
         end;
      else;
         if k>0;
            for t=1:T;
               YHistPred = (A'*Xdata(:,t))+(H(:,:,FirstPeriod+KalmanFirstObservation-2+t)'*KsitT(:,t));
            end;
         else;
            for t=1:T;
               YHistPred = H(:,:,FirstPeriod+KalmanFirstObservation-2+t)'*KsitT(:,t);
            end;
         end;
      end;
      YSmooth(nanpos) = YHistPred(nanpos);
      if FirstPeriod+KalmanFirstObservation-1>1;
         YSmooth = [DSGEModel.Y(:,1:FirstPeriod+KalmanFirstObservation-2) YSmooth];
      end;
      Yinitial = YSmooth(:,1:LastPeriod);
   end;
end;
%
% write down the solution matrices for projections using ZLB format, with Mt, Gt and C1
%
[Mt,Gt,C1,Mtilde,Pt] = ZLBProjectionMatrices(Mtilde(:,:,T),Ftilde(:,:,T),Btilde(:,:,T),LastSolution.InvMatT1*DA,MA);
%
% deal with deterministic variables
%
if k>0;
   Ax = A'*X;
else;
   Ax = zeros(n,h);
   X = zeros(r,max(DSGEModel.ZLBSampleT+1,h))
end;
%
% setup projections matrices for the restricted policy rate
%
% each row in ePG is the row vector multiplied by V(t) for tau=0,1,...,DSGEModel.ZLBSampleT
% each element in ePGx is the deterministic term of the projection
%
ePG = zeros(DSGEModel.ZLBSampleT+1,size(Gt,2));
ePGx = zeros(DSGEModel.ZLBSampleT+1,1);
Pmats = zeros(r+1,q+r+2+DSGEModel.ZLBSampleT,max(DSGEModel.ZLBSampleT+1,h));
Mmats = zeros(r+1,h);
if DSGEModel.FixedBeliefsForProj==1;
   ePG(1,:) = e_pr'*Pt(1:r+1,:);
   xbart = zeros(size(Mt,1),1);
   ePGx(1) = e_pr'*Mtilde*X(:,1);
   Mmats(:,1) = Mtilde*X(:,1);
   Pmats(:,:,1) = Pt;
   for tau=1:DSGEModel.ZLBSampleT;
      ePG(tau+1,:) = ePG(tau,:)*Gt;
      xbart = (Mt*X(:,tau))+Gt*xbart;
      ePGx(tau+1) = e_pr'*((Mtilde*X(:,tau+1))+(Pt*xbart));
      Mmats(:,tau+1) = Mtilde*X(:,tau+1);
      Pmats(:,:,tau+1) = Pt;
   end;
   if h>DSGEModel.ZLBSampleT+1;
      for tau=DSGEModel.ZLBSampleT+1:h-1;
         Pmats(:,:,tau+1) = Pt;
         Mmats(:,tau+1) = Mtilde*X(:,tau+1);
      end;
   end;
else;
   %
   % beliefs are now updated, take the values of the
   % solution matrices from period T that have been updated for T+1
   %
   MTiT = LastSolution.MTT;
   FTiT = LastSolution.FTT;
   BTiT = LastSolution.BTT;
   betaTiT = LastSolution.betaTT;
   InvMatT = LastSolution.InvMatT;
   %
   % write down the solution matrices for projections using ZLB format, with Mt, Gt and C1
   %
   [Mt,Gt,C1,Mtilde,Pt] = ZLBProjectionMatrices(MTiT,FTiT,BTiT,InvMatT*DA,MA);
   ePG(1,:) = e_pr'*Pt(1:r+1,:);
   xbart = zeros(size(Mt,1),1);
   ePGx(1) = e_pr'*Mtilde*X(:,1);
   Mmats(:,1) = Mtilde*X(:,1);
   Pmats(:,:,1) = Pt;
   for tau=1:DSGEModel.ZLBSampleT;
      %
      % update betaTiT
      %
      betaTiT = betavec+(LastSolution.M*(betaTiT-betavec));
      %
      % update the solution matrices
      %
      if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
         [MTiT,FTiT,BTiT,InvMatT] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betaTiT);
      else;
         [MTiT,FTiT,BTiT,InvMatT] = feval(GetName(DSGEModel.AdaptiveLearningModelSolverFile),Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betaTiT,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
      end;
      %
      % obtain the new projection matrices
      %
      [Mt,Gt,C1,Mtilde,Pt] = ZLBProjectionMatrices(MTiT,FTiT,BTiT,InvMatT*DA,MA);
      ePG(tau+1,:) = ePG(tau,:)*Gt;
      xbart = (Mt*X(:,tau))+Gt*xbart;
      ePGx(tau+1) = e_pr'*((Mtilde*X(:,tau+1))+(Pt*xbart));
      Mmats(:,tau+1) = Mtilde*X(:,tau+1);
      Pmats(:,:,tau+1) = Pt;
   end;
   if h>DSGEModel.ZLBSampleT+1;
      for tau=DSGEModel.ZLBSampleT+1:h-1;
         %
         % update betaTiT
         %
         betaTiT = betavec+(LastSolution.M*(betaTiT-betavec));
         %
         % update the solution matrices
         %
         if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
            [MTiT,FTiT,BTiT,InvMatT] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betaTiT);
         else;
            [MTiT,FTiT,BTiT,InvMatT] = feval(GetName(DSGEModel.AdaptiveLearningModelSolverFile),Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betaTiT,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
         end;
         %
         % obtain the new projection matrices
         %
         [Mt,Gt,C1,Mtilde,Pt] = ZLBProjectionMatrices(MTiT,FTiT,BTiT,InvMatT*DA,MA);
         Pmats(:,:,tau+1) = Pt;
         Mmats(:,tau+1) = Mtilde*X(:,tau+1);
      end;
   end;
end;
%
% compute the mean predictions
%
ZLBYMean = zeros(n,h);
etaMean = zeros(q,h);
KsiTMean = KsiTT;
ZLBKsiMean = zeros(r,h);
%
At = zeros(DSGEModel.ZLBSampleT+1,1);
Xt = [etaMean(:,1);KsiTMean];
for t=1:h;
   [At,status,AlgorithmInformation] = ForwardBackShootingAlgorithmAdaptiveLearning(Xt,At,ePG,ePGx,Rzlb(t,:),Rbar,DSGEModel.AIMTolerance);
   if status==1;
      Ksit = Mmats(:,t)+(Pmats(:,:,t)*[Xt;At]);
      ZLBKsiMean(:,t) = Ksit(1:r);
      if length(size(H))==2;
         ZLBYMean(:,t) = Ax(:,t)+(H'*Ksit);
      else;
         ZLBYMean(:,t) = Ax(:,t)+(H(:,:,DSGEModel.LastPeriod+t)'*Ksit);
      end;
      if t+1<=h;
         At = MA*At;
         Xt = [etaMean(:,t+1);Ksit];
      end;
   else;
      break;
   end;
end;
%
% check if the mean shocks and state failed to find the anticipated shocks.
%
if status==0;
   if AlgorithmInformation(1)==1;
      AddTxt = 'YADA has recorded an invertibility problem when attempting to solve for the anticipated shocks.';
   elseif AlgorithmInformation(2)==1;
      AddTxt = 'YADA has recorded that a solution cannot be found, i.e., Step 3 (ii) holds (see the YADA Manual).';
   elseif AlgorithmInformation(3)==1;
      AddTxt = 'YADA has recorded that the anticipated shocks are nonnegative, that the restricted policy rate satisfies the zero lower bound, but the complementary slackness condition is violated.';
   else;
      AddTxt = ['YADA has recorded that the maximum number of attempts (' int2str(AlgorithmInformation(5)) ') to find the anticipated shocks was reached.'];
   end;
   txt = ['The forward-back shooting algorithm failed to solve for the anticipated shocks for period ' int2str(t) ' using the population means of the structural shocks and the state variables in the last historical period. ' AddTxt ' Are you sure you want to continue?'];
   answer = Query(txt,'question',300,['Zero Lower Bound Forecasts - ' NameStr],600,'no',CurrINI);
   if strcmp(lower(answer),'no')==1;
      return;
   end;
end;
%
% check if we should annualize or transform
%
ZLBYMeanOrig = ZLBYMean;
if AnnualizeData==1;
   ZLBYMean = Annualize([Yinit ZLBYMean],DSGEModel.annual,DSGEModel.annualscale);
elseif TransData==1;
   YTmp = [Yinitial ZLBYMean];
   [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,YTmp',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
   ZLBYMean = YData(Tt-h+1:Tt,:)';
end;
%
PredData.ZLBYMean = ZLBYMean;
PredData.ZLBKsiMean = ZLBKsiMean;
PredData.Mt = Mt;
PredData.Gt = Gt;
PredData.C1 = C1;
PredData.Mtilde = Mtilde;
PredData.Pt = Pt;
%
% check if we should run parallel computations
%
if CurrINI.NumWorkers<2;
   %
   % check if we should setup a wait handle
   %
   if DSGEModel.ShowProgress==1;
      ProgressStructure.title = ['Progress for ' int2str(NumPaths) ' Prediction Paths'];
      ProgressStructure.facecolor = CurrINI.progress_facecolor;
      ProgressStructure.startfacecolor = CurrINI.progress_startfacecolor;
      ProgressStructure.edgecolor = CurrINI.progress_edgecolor;
      ProgressStructure.bgcolor = CurrINI.progress_bgcolor;
      ProgressStructure.stop = 0;
      ProgressStructure.clock = DSGEModel.ShowProgressClock;
      ProgressStructure.label = 'Path Number:';
      %
      ProgressStructure.name = ['ZLB Predictive Distributions - ' TypeStr ' Data'];
      ProgressStructure.CurrINI = CurrINI;
      WaitHandle = ProgressDLG(0,ProgressStructure);
      set(WaitHandle,'Color',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'));
      drawnow;
   else;
      txt = ['Please wait while YADA computes predictive distributions subject to the zero (effective) lower bound under adaptive learning for the ' lower(TypeStr) ' data. Computations started at: ' StringTrim(datestr(now,14))];
      WaitHandle = WaitDLG(txt,'information',['ZLB Predictive Distributions - ' TypeStr ' Data'],600,150,CurrINI,0);
      WaitControls = get(WaitHandle,'UserData');
   end;
   %
   % start the simulations
   %
   abort = '';
   StopEstimation = 0;
   ZLBYPaths = zeros(n,h,NumPaths);
   ZLBKsiPaths = zeros(r,h,NumPaths);
   AtMean = zeros(DSGEModel.ZLBSampleT+1,h);
   XtMean = zeros(q+r+1,h);
   %
   for it=1:NumPaths;
      %
      % check if we should update the progress dialog
      %
      if DSGEModel.ShowProgress==1;
         abort = get(WaitHandle,'UserData');
         if (strcmp(abort,'cancel')==1)|(strcmp(abort,'done')==1);
            ZLBYPaths = [];
            ZLBKsiPaths = [];
            StopEstimation = 1;
            break;
         else;
            ProgressDLG([it/NumPaths it]);
         end;
      end;
      %
      % Draw a sequence of measurement errors
      %
      if isempty(Rmeasure)==0;
         wTh = Rmeasure*randn(NumMeasure,h);
         wTh(DSGEModel.ZLBPosition,:) = 0;
      end;
      %
      % we are now ready to compute possible paths at theta
      %
      ShockMatrix = zeros(q,h);
      NumZLBTimes = zeros(1,h);
      %
      % set up a loop to allow for new draws of the state vector
      % and if that fails to use the thetaMode parameter values
      %
      RunLoop = 1;
      while RunLoop==1;
         %
         % Draw a state vector for time T using the N(KsiTT,PTT) distribution.
         %
         KsiT = KsiTT+(S*sqrt(L)*randn(size(L,2),1));
         PredictY = zeros(n,h);
         At = zeros(DSGEModel.ZLBSampleT+1,1);
         Xt = [zeros(q,1);KsiT];
         t = 1;
         ZLBKsiPath = zeros(r,h);
         while t<=h;
            %
            % add shocks to Xt
            %
            Xt(1:q) = randn(q,1);
            [AtNew,status,AlgorithmInformation] = ForwardBackShootingAlgorithmAdaptiveLearning(Xt,At,ePG,ePGx,Rzlb(t,:),Rbar,DSGEModel.AIMTolerance);
            if status==1;
               At = AtNew;
               AtMean(:,t) = AtMean(:,t)+((1/NumPaths)*At);
               XtMean(:,t) = XtMean(:,t) + ((1/NumPaths)*Xt);
               if max(At)>0;
                  %
                  % add the shocks when ZLB is binding and count how many times for given horizon
                  %
                  ShockMatrix(:,t) = ShockMatrix(:,t)+Xt(1:q,1);
                  NumZLBTimes(1,t) = NumZLBTimes(1,t)+1;
               end;
               Ksit = Mmats(:,t)+(Pmats(:,:,t)*[Xt;At]);
               ZLBKsiPath(:,t) = Ksit(1:r);
               if length(size(H))==2;
                  PredictY(:,t) = Ax(:,t)+(H'*Ksit);
               else;
                  PredictY(:,t) = Ax(:,t)+(H(:,:,DSGEModel.LastPeriod+t)'*Ksit);
               end;
               %
               % test if the prediction is "reasonable"
               %
               TestValue = (PredictY(:,t)-ZLBYMeanOrig(:,t))./ZLBYMeanOrig(:,t);
               if max(abs(TestValue))<100;
                  if t+1<=h;
                     At = MA*At;
                     Xt = [zeros(q,1);Ksit];
                  end;
                  t = t+1;
                  if t==h+1;
                     RunLoop = 0;
                  end;
               else;
                  t = h+1;
%                  break;
               end;
%            else;
%               t = h+1;
%               break;
            end;
         end;
      end;
      ZLBKsiPaths(:,:,it) = ZLBKsiPath;
      %
      % add measurement error draw (or 0) to the forecast path
      %
      PredictY = PredictY+wTh;
      if AnnualizeData==1;
         PredictY = Annualize([Yinit PredictY],DSGEModel.annual,DSGEModel.annualscale);
      elseif TransData==1;
         YTmp = [Yinitial PredictY];
         [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,YTmp',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
         PredictY = YData(Tt-h+1:Tt,:)';
      end;
      %
      % add the projected path to the variable to be saved
      %
      ZLBYPaths(:,:,it) = PredictY;
      drawnow;
   end;
   %
   % close any wait dialog
   %
   if DSGEModel.ShowProgress==1;
      set(WaitHandle,'UserData','done');
      close(WaitHandle);
      drawnow;
      pause(0.02);
   elseif DSGEModel.RunDriver==0;
      set(WaitControls.text,'UserData','done');
      delete(WaitHandle);
      drawnow;
      pause(0.02);
   end;
else;
   %
   % we can do parallel computations over the parameter draws
   %
   txt = ['Please wait while YADA computes predictive distributions subject to the zero (effective) lower bound under adaptive learning for the ' lower(TypeStr) ' data using parallel computations with ' int2str(CurrINI.NumWorkers) ' workers. Computations started at: ' StringTrim(datestr(now,14))];
   WaitHandle = WaitDLG(txt,'information',['ZLB Predictive Distributions - ' TypeStr ' Data'],600,150,CurrINI,0);
   WaitControls = get(WaitHandle,'UserData');
   %
   % initialize
   %
   abort = '';
   StopEstimation = 0;
   ZLBYPaths = zeros(n,h,NumPaths);
   ZLBKsiPaths = zeros(r,h,NumPaths);
   AtPaths = zeros(DSGEModel.ZLBSampleT+1,h,NumPaths);
   XtPaths = zeros(q+r+1,h,NumPaths);
   %
   if DSGEModel.RandomNumberValue==1;
      %
      % without eval statements the spmd-code gives an error in older matlab systems
      %
      eval('spmd; rng(0,''combRecursive''); end;');
   else;
      eval('spmd; rng(''default''); end;');
   end;
   %
   parfor it=1:NumPaths;
      %
      % Draw a sequence of measurement errors
      %
      if isempty(Rmeasure)==0;
         wTh = Rmeasure*randn(NumMeasure,h);
         wTh(DSGEModel.ZLBPosition,:) = 0;
      else;
         wTh = zeros(n,h);
      end;
      %
      % we are now ready to compute possible paths at theta
      %
      ShockMatrix = zeros(q,h);
      NumZLBTimes = zeros(1,h);
      %
      % set up a loop to allow for new draws of the state vector
      % and if that fails to use the thetaMode parameter values
      %
      RunLoop = 1;
      while RunLoop==1;
         %
         % Draw a state vector for time T using the N(KsiTT,PTT) distribution.
         %
         KsiT = KsiTT+(S*sqrt(L)*randn(size(L,2),1));
         PredictY = zeros(n,h);
         At = zeros(DSGEModel.ZLBSampleT+1,1);
         Xt = [zeros(q,1);KsiT];
         t = 1;
         ZLBKsiPath = zeros(r,h);
         AtPath = zeros(DSGEModel.ZLBSampleT+1,h);
         XtPath = zeros(q+r+1,h);
         while t<=h;
            %
            % add shocks to Xt
            %
            Xt(1:q) = randn(q,1);
            [AtNew,status,AlgorithmInformation] = ForwardBackShootingAlgorithmAdaptiveLearning(Xt,At,ePG,ePGx,Rzlb(t,:),Rbar,DSGEModel.AIMTolerance);
            if status==1;
               At = AtNew;
               AtPath(:,t) = At;
               XtPath(:,t) = Xt;
               if max(At)>0;
                  %
                  % add the shocks when ZLB is binding and count how many times for given horizon
                  %
                  ShockMatrix(:,t) = ShockMatrix(:,t)+Xt(1:q,1);
                  NumZLBTimes(1,t) = NumZLBTimes(1,t)+1;
               end;
               Ksit = Mmats(:,t)+(Pmats(:,:,t)*[Xt;At]);
               ZLBKsiPath(:,t) = Ksit(1:r);
               if length(size(H))==2;
                  PredictY(:,t) = Ax(:,t)+(H'*Ksit);
               else;
                  PredictY(:,t) = Ax(:,t)+(H(:,:,DSGEModel.LastPeriod+t)'*Ksit);
               end;
               %
               % test if the prediction is "reasonable"
               %
               TestValue = (PredictY(:,t)-ZLBYMeanOrig(:,t))./ZLBYMeanOrig(:,t);
               if max(abs(TestValue))<100;
                  if t+1<=h;
                     At = MA*At;
                     Xt = [zeros(q,1);Ksit];
                  end;
                  t = t+1;
                  if t==h+1;
                     RunLoop = 0;
                  end;
               else;
                  t = h+1;
%                  break;
               end;
%            else;
%               t = h+1;
%               break;
            end;
         end;
      end;
      ZLBKsiPaths(:,:,it) = ZLBKsiPath;
      AtPaths(:,:,it) = AtPath;
      XtPaths(:,:,it) = XtPath;
      %
      % add measurement error draw (or 0) to the forecast path
      %
      PredictY = PredictY+wTh;
      if AnnualizeData==1;
         PredictY = Annualize([Yinit PredictY],DSGEModel.annual,DSGEModel.annualscale);
      elseif TransData==1;
         YTmp = [Yinitial PredictY];
         [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,YTmp',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
         PredictY = YData(Tt-h+1:Tt,:)';
      end;
      %
      % add the projected path to the variable to be saved
      %
      ZLBYPaths(:,:,it) = PredictY;
      drawnow;
   end;
   %
   % compute the mean values of Xt and At
   %
   AtMean = (1/NumPaths)*sum(AtPaths,3);
   XtMean = (1/NumPaths)*sum(XtPaths,3);
   %
   % close the wait dialog
   %
   set(WaitControls.text,'UserData','done');
   delete(WaitHandle);
   drawnow;
   pause(0.02);
end;
if StopEstimation==1;
   return;
end;
%
% now compute the mean projections using AtMean and XtMean
%
for t=1:h;
   Ksit = Mmats(:,t)+(Pmats(:,:,t)*[XtMean(:,t);AtMean(:,t)]);
   ZLBKsiMean(:,t) = Ksit(1:r);
   if length(size(H))==2;
      ZLBYMean(:,t) = Ax(:,t)+(H'*Ksit);
   else;
      ZLBYMean(:,t) = Ax(:,t)+(H(:,:,DSGEModel.LastPeriod+t)'*Ksit);
   end;
end;
if AnnualizeData==1;
   ZLBYMean = Annualize([Yinit ZLBYMean],DSGEModel.annual,DSGEModel.annualscale);
elseif TransData==1;
   YTmp = [Yinitial ZLBYMean];
   [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,YTmp',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
   ZLBYMean = YData(Tt-h+1:Tt,:)';
end;
%
% update the mean value
%
PredData.ZLBYMean = ZLBYMean;
PredData.ZLBKsiMean = ZLBKsiMean;
if DSGEModel.RunPredictionEvent==1;
   %
   % compute the prediction events results
   %
   PredEventData = CalculatePredictionEvents(ZLBYPaths,DSGEModel.PredictionEvent,1);
end;
if NumPaths>=500;
   [PredData.KernelX,PredData.KernelY] = PredictionKernelDensity(ZLBYPaths);
else;
   PredData.KernelX = [];
   PredData.KernelY = [];
end;
%
% save results to file
%
PredictionEvent = DSGEModel.PredictionEvent;
MatFile = FixFilePath([DSGEModel.OutputDirectory '\predictions\ZLB-Prediction-AdaptiveLearning-' DSGEModel.NameOfModel '-' strrep(NameStr,' ','') '-' TypeStr '-' int2str(DSGEModel.FirstPeriod) '.' int2str(DSGEModel.LastPeriod) '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '.' int2str(DSGEModel.ZLBRuleType) '-' int2str(h) '-' int2str(NumPaths) '.mat']);
save(MatFile,'ZLBYPaths','ZLBKsiPaths','ZLBYMean','ZLBKsiMean','KsitTh','PredData','PredEventData','PredictionEvent');
Status = 1;

%
% end of ZLBAdaptiveLearningDSGEPredictionPathsTheta.m
%
