function [Results,Options,status,kalmanstatus] = DSGERecentPastDecomp(theta,thetaPositions,ModelParameters,IsRecentPast,IsOriginal,EstStr,DSGEModel,CurrINI)
% DSGERecentPastDecomp: Computes the recent & past or the sample-split observed variable decompositions for
%                       fixed parameter values.
%
% USAGE:
%
%       [Results,Options,status,kalmanstatus] = DSGERecentPastDecomp(theta,thetaPositions,ModelParameters,IsRecentPast,IsOriginal,...
%                                                            EstStr,DSGEModel,CurrINI)
%
% REQUIRED INPUT:  theta (vector) with values of the original DSGE model parameters
%
%                  thetaPositions (structure) with fields parameter and distribution. This function
%                                 needs the former field to know which parameter name corresponds to
%                                 position i in theta.
%
%                  ModelParameters (structure) whose fields have names given by the parameter names in the
%                                 DSGE model.
%
%                  IsRecentPast (boolean) that takes the value 1 if recent & past decompositions should be
%                                 computed and 0 if sample-split decompositions have been called for.
%
%                  IsOriginal (boolean) that takes the value 1 if the decomposition concerns the original
%                                 data and 0 if it concerns the annualized data.
%
%                  EstStr (string vector) that takes the value 'Initial Values' or 'Posterior Mode'.
%
%                  DSGEModel (structure) with fields containing all the information about a DSGE Model for YADA.
%
%                  CurrINI (structure) with initialization information.
%
% REQUIRED OUTPUT: Results (structure) with fields "Y", "ObsSS", "ObsIS", "ObsPast", "ObsRecent", "StateIS",
%                                 "StatePast", "StateRecent", "States", and "SampleStrMat". When IsOriginal
%                                 is 0 there are also annualizations for the observed variables in the fields
%                                 "AnnY", "AnnObsSS", "AnnObsIS", "AnnObsPast" and "AnnObsRecent".
%
%                  Options (structure) with fields "RecentPastLag" and "SampleSplitPeriod".
%
% OPTIONAL OUTPUT: status (integer) Indicates whether the solution of the model is unique or not.
%                                 mcode is 1 when unique, 0 if not computed, and various other values when
%                                 there are problems with the solution. Specifically:
%
%                                      mcode = 2:      Roots not correctly computed by real_schur
%                                      mcode = 3:      Too many big roots
%                                      mcode = 35:     Too many big roots, and q(:,right) is singular
%                                      mcode = 4:      Too few big roots
%                                      mcode = 45:     Too few big roots, and q(:,right) is singular
%                                      mcode = 5:      q(:,right) is singular
%                                      mcode = 61:     Too many exact shiftrights
%                                      mcode = 62:     Too many numeric shiftrights
%                                      mcode = 7:      The a matrix has infinite or NaN values. Eigenvalues
%                                                      cannot be calculated.
%                                      mcode = 8:      The function "compute_aim_matrices" returns complex
%                                                      numbers.
%                                      mcode = -1:     No stable solution.
%                                      mcode = -2:     Too many large generalized eigenvalues.
%                                      mcode = -3:     Too few large generalized eigenvalues.
%                                      else            Return code not properly specified
%
%                  kalmanstatus (boolean) indicating the covariance stationarity of the state variables. If all the
%                                 eigenvalues of F are inside the unit circle, then kalmanstatus=0 and the log-likelihood function
%                                 is calculated. If not, then status=1 and initP is set to 0.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2025 European Central Bank.
%
%                       First version: March 3, 2025.
%                        This version: March 17, 2025.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 04-03-2025: Continued the function after the smoother.
%
% * 05-03-2025: Finished the original data decompositions for both "Recent & Past" and "Sample-Split"
%               decompositions.
%
% * 06-03-2025: Finished the annualization for the "Sample-Split case and the annualization of the quarterly part
%               of the "Recent & Past" case.
%
% * 07-03-2025: Finished the anualization for the monthly case for the "Recent & Past" decomposition.
%
% * 12-03-2025: Added the fields "KeepVar" and "ShockNames" to the output structure.
%
% * 13-03-2025: Added an if test when computing the impact of recent shocks under the sample-split case. Corrected
%               the detrimination of T0 when annualized data are used.
%
% * 17-03-2025: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
Results = [];
Options.RecentPastLag = DSGEModel.RecentPastLag;
Options.SampleSplitPeriod = DSGEModel.SampleSplitPeriod;
%
% set up the ModelParameters structure based on theta and thetaPositions
%
if nargout==2;
   %
   % Make sure that the ModelParameter structure takes the correct values.
   % This is not necessary to do when initial values are given to the function.
   % In that case, more substantial checking is required!
   %
   ModelParameters = ThetaToModelParameters(ModelParameters,theta,thetaPositions);
   %
   % copy files to the tmp folder
   %
   [stat,msg] = CopyFile([GetPath(DSGEModel.AIMDataFile) 'compute_aim_matrices.m'],[pwd '\tmp']);
   [stat,msg] = CopyFile(DSGEModel.MeasurementEquationFile,[pwd '\tmp']);
   if FileExist(DSGEModel.UpdateParameterFile)==1;
      [stat,msg] = CopyFile(DSGEModel.UpdateParameterFile,[pwd '\tmp']);
   end;
   if FileExist(DSGEModel.SystemPriorFile)==1;
      [stat,msg] = CopyFile(DSGEModel.SystemPriorFile,[pwd '\tmp']);
   end;
   %
   % check if we should copy model solver file
   %
   if DSGEModel.ModelSolver==4;
      if FileExist(DSGEModel.ModelSolverFile)==1;
         [stat,msg] = CopyFile(DSGEModel.ModelSolverFile,[pwd '\tmp']);
      end;
   end;
   %
   % Update any parameteters that need updating
   %
   if FileExist(DSGEModel.UpdateParameterFile)==1;
      ModelParameters = feval(GetName(DSGEModel.UpdateParameterFile),ModelParameters);
   end;
else;
   status = 1;
   kalmanstatus = 0;
end;
%
% load the AIMData file
%
ErrorStr = '';
if MatlabNumber>=8.0;
   try;
      AIMData = load(DSGEModel.AIMDataFile);
   catch ME;
      ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError(ME)];
   end;
else;
   try;
      AIMData = load(DSGEModel.AIMDataFile);
   catch;
      ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError];
   end;
end;
%
% deal with the sample
%
[FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
SampleStrMat = CreateFullSample(DSGEModel);
%
%TypeStr = 'Original';
%if IsOriginal==0;
%   TypeStr = 'Annualized';
%end;
%
% check the lag for the past and the sample-split
%
NumPeriods = LastPeriod-(FirstPeriod+DSGEModel.KalmanFirstObservation-2);
OriginalNumPeriods = NumPeriods;
if IsOriginal==0;
   NumPeriods = NumPeriods-max(DSGEModel.annual)+1;
   StrMat = SampleStrMat(FirstPeriod+DSGEModel.KalmanFirstObservation+max(DSGEModel.annual)-2:LastPeriod-1,:);
else;
   StrMat = SampleStrMat(FirstPeriod+DSGEModel.KalmanFirstObservation-1:LastPeriod-1,:);
end;
if IsRecentPast==1;
   if DSGEModel.RunDriver==0;
      LagMatrix = '';
      for i=1:NumPeriods;
         LagMatrix = strvcat(LagMatrix,['First past period is lag=' int2str(i)]);
      end;
      [action,SelectedOption] = SelectionDlg(LagMatrix,Options.RecentPastLag,'Select the lag until the first past period:','Lag Selection',['Recent & Past - ' EstStr],'','',CurrINI);
   else;
      SelectedOption = Options.RecentPastLag;
      action = 'OK';
   end;
else;
   SampleSplitPeriod = Options.SampleSplitPeriod;
   if NumPeriods<=Options.SampleSplitPeriod;
      SampleSplitPeriod = 1;
   end;
   if DSGEModel.RunDriver==0;
      [action,SelectedOption] = SelectionDlg(StrMat,NumPeriods-SampleSplitPeriod,'Select the last period before the sample-split:','Lag Selection',['Recent & Past - ' EstStr],'','',CurrINI);
      if IsOriginal==0;
         OriginalSelectedOption = SelectedOption+max(DSGEModel.annual)-1;
      else;
         OriginalSelectedOption = SelectedOption;
      end;
   else;
      SelectedOption = NumPeriods-SampleSplitPeriod;
      action = 'OK';
      if IsOriginal==0;
         OriginalSelectedOption = SelectedOption+max(DSGEModel.annual)-1;
      else;
         OriginalSelectedOption = SelectedOption;
      end;
   end;
end;
%
% check if we should continue
%
if strcmp(lower(action),'cancel')==1;
   return;
end;
%
% solve the model for the posterior mode estimate or the initial values
%
ErrorStr = '';
if MatlabNumber>=8.0;
   try;
      [A,H,R,F,B0,mcode,ErrMsg] = SolveDSGEModel(DSGEModel,theta,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
      if isempty(ErrMsg)==0;
         ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' ErrMsg];
      end;
   catch ME;
      ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' YADALastError(ME)];
   end;
else;
   try;
      [A,H,R,F,B0,mcode,ErrMsg] = SolveDSGEModel(DSGEModel,theta,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
      if isempty(ErrMsg)==0;
         ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' ErrMsg];
      end;
   catch;
      ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' YADALastError];
   end;
end;
if isempty(ErrorStr)==0;
   if nargout>1;
      status = mcode;
   end;
   About(ErrorStr,'information','YADA - Error Message',CurrINI.scrsz(4)-50,600,CurrINI);
   return;
end;
if mcode~=1;
   if mcode==2;
      mcodeStr = 'Roots are not correctly computed by real_schur.'
   elseif mcode==3;
      mcodeStr = 'Too many big roots.';
   elseif mcode==35;
      mcodeStr = 'Too many big roots, and q(:,right) is singular.';
   elseif mcode==4;
      mcodeStr = 'Too few big roots.';
   elseif mcode==45;
      mcodeStr = 'Too few big roots, and q(:,right) is singular.';
   elseif mcode==5;
      mcodeStr = 'q(:,right) is singular.';
   elseif mcode==61;
      mcodeStr = 'Too many exact shiftrights.';
   elseif mcode==62;
      mcodeStr = 'Too many numeric shiftrights.';
   elseif mcode==7;
      mcodeStr = 'Infinite or NaN values detected.';
   elseif mcode==8;
      mcodeStr = 'The function "compute_aim_matrices" returns complex numbers.';
   elseif mcode==-1;
      mcodeStr = 'No stable solution.';
   elseif mcode==-2;
      mcodeStr = 'Too many large generalized eigenvalues.';
   elseif mcode==-3;
      mcodeStr = 'Too few large generalized eigenvalues.';
   else;
      mcodeStr = 'Return code not properly specified.';
   end;
   if DSGEModel.ModelSolver==1;
      SolverStr = 'AiM';
   elseif DSGEModel.ModelSolver==2;
      SolverStr = 'Klein';
   elseif DSGEModel.ModelSolver==3;
      SolverStr = 'Gensys';
   elseif DSGEModel.ModelSolver==4;
      SolverStr = ['Own algorithm (' GetName(DSGEModel.ModelSolverFile) ')'];
   end;
   txt = ['The ' SolverStr ' solver provided the return code: ' num2str(mcode) ', i.e., "' mcodeStr '"'];
   About(txt,'information',['YADA - ' SolverStr ' Solver Error'],500,600,CurrINI);
   if nargout>1;
      status = mcode;
   end;
   return;
end;
if length(size(H))==3;
   if size(H,3)<LastPeriod;
      %
      % too few data points in the 3D H matrix.
      %
      ErrorStr = ['The number of time periods for the time-varying measurement matrix H (' int2str(size(H,3)) ') is lower than the number of observations (T = ' int2str(LastPeriod) '). YADA has therefore aborted from the computation of ' lower(RecentPastStr) ' decomposition distribution.'];
      About(ErrorStr,'error','Error - Measurement Equation',500,600,CurrINI);
      return;
   end;
end;
%
[B0,KeepShocks] = RemoveRedundantColumns(B0);
[r,q] = size(B0);
n = DSGEModel.n;
k = DSGEModel.k;
ShockNames = DSGEModel.StateShockNames(KeepShocks,:);
%
% set the InitialStateVector value
%
if DSGEModel.UseOwnInitialState==1;
   if length(DSGEModel.InitialStateValues)==size(AIMData.endog,1);
      DSGEModel.InitialStateVector = DSGEModel.InitialStateValues(DSGEModel.StateVariablePositions);
   else;
      DSGEModel.InitialStateVector = zeros(length(DSGEModel.StateVariablePositions),1);
   end;
else;
   DSGEModel.InitialStateVector = zeros(length(DSGEModel.StateVariablePositions),1);
end;
%
% We can now try to run the Kalman filter
%
tic;
if MatlabNumber>=8.0;
   try;
      [logLikeValue,stat,lnLt,Yerror,Yhat,SigmaSqRoot,MSEY,InvMSEY,KalmanGain,Ksitt1,Ptt1] = RunKalmanFilter(DSGEModel.Y,DSGEModel.X,FirstPeriod,LastPeriod,A,H,F,B0,R,DSGEModel.InitialStateVector,DSGEModel.UseDoublingAlgorithm,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,1,DSGEModel.AllowUnitRoot,DSGEModel.StateCovConstValue,GetPosition(DSGEModel.StateVariablePositions,DSGEModel.UnitRootStates),DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm);
      KalmanTime = toc;
   catch ME;
      txt = ['YADA caught the following error message from the Kalman filter routine: ' YADALastError(ME)];
      About(txt,'error','Error - Kalman Filter',200,500,CurrINI);
      if nargout>2;
         kalmanstatus = stat;
      end;
      KalmanTime = toc;
      return;
   end;
else;
   try;
      [logLikeValue,stat,lnLt,Yerror,Yhat,SigmaSqRoot,MSEY,InvMSEY,KalmanGain,Ksitt1,Ptt1] = RunKalmanFilter(DSGEModel.Y,DSGEModel.X,FirstPeriod,LastPeriod,A,H,F,B0,R,DSGEModel.InitialStateVector,DSGEModel.UseDoublingAlgorithm,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,1,DSGEModel.AllowUnitRoot,DSGEModel.StateCovConstValue,GetPosition(DSGEModel.StateVariablePositions,DSGEModel.UnitRootStates),DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm);
      KalmanTime = toc;
   catch;
      txt = ['YADA caught the following error message from the Kalman filter routine: ' YADALastError];
      About(txt,'error','Error - Kalman Filter',200,500,CurrINI);
      if nargout>2;
         kalmanstatus = stat;
      end;
      KalmanTime = toc;
      return;
   end;
end;
%
% Run the state smoother
%
if MatlabNumber>=8.0;
   try;
      [Ksitt,Ptt,KsitT,PtT,rttMat,rTMat,NtMat] = RunKalmanSmoother(DSGEModel.Y,FirstPeriod,LastPeriod,Yhat,Ksitt1,Ptt1,H,F,R,B0,Yerror,SigmaSqRoot,InvMSEY,KalmanGain,DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm,DSGEModel.UseDoublingAlgorithm==4);
   catch ME;
      Ksitt = [];
      KsitT = [];
      NtMat = [];
      txt = ['YADA caught the following error message from the Kalman smoother: ' YADALastError(ME)];
      About(txt,'error','Error - Kalman Smoother',200,500,CurrINI);
      return;
   end;
else;
   try;
      [Ksitt,Ptt,KsitT,PtT,rttMat,rTMat,NtMat] = RunKalmanSmoother(DSGEModel.Y,FirstPeriod,LastPeriod,Yhat,Ksitt1,Ptt1,H,F,R,B0,Yerror,SigmaSqRoot,InvMSEY,KalmanGain,DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm,DSGEModel.UseDoublingAlgorithm==4);
   catch;
      Ksitt = [];
      KsitT = [];
      NtMat = [];
      txt = ['YADA caught the following error message from the Kalman smoother: ' YADALastError];
      About(txt,'error','Error - Kalman Smoother',200,500,CurrINI);
      return;
   end;
end;
%
% compute the smooth shocks
%
KsitT = KsitT(:,FirstPeriod+DSGEModel.KalmanFirstObservation-1:LastPeriod);
etatT = B0'*rTMat(:,FirstPeriod+DSGEModel.KalmanFirstObservation-1:LastPeriod);
T = size(etatT,2);
%
% initialize the decompositions
%
ObsSS = zeros(n,T);
ObsIS = zeros(n,T);
StateIS = zeros(r,T);
ObsPast = zeros(n,T,q);
StatePast = zeros(r,T,q);
ObsRecent = zeros(n,T,q);
StateRecent = zeros(r,T,q);
%
% we also need to compute the measurement errors
%
wtT = zeros(n,T);
Y = DSGEModel.Y(:,FirstPeriod+DSGEModel.KalmanFirstObservation-1:LastPeriod);
if k>=1;
   X = DSGEModel.X(:,FirstPeriod+DSGEModel.KalmanFirstObservation-1:LastPeriod);
end;
%
% deal with missing data
%
if DSGEModel.YNaNs==1;
   YNaNs = max(max(isnan(Y)));
   if YNaNs==1;
      ObsPast = NaN*ObsPast;
      ObsRecent = NaN*ObsRecent;
   end;
else;
   YNaNs = 0;
end;
%
if k>0;
   ObsSS = A'*X;
   if YNaNs==1;
      ObsSS(isnan(Y)) = NaN;
   end;
end;
StateIS(:,1) = KsitT(:,1)-(B0*etatT(:,1));
for t=2:T;
   StateIS(:,t) = F*StateIS(:,t-1);
end;
if length(size(H))==2;
   if rank(R)>0;
      if min(size(DSGEModel.X))>=1;
         wtT = Y-(A'*X)-(H'*KsitT);
      else;
         wtT = Y-(H'*KsitT);
      end;
   end;
   ObsIS = H'*StateIS;
   if YNaNs==1;
      ObsIS(isnan(Y)) = NaN;
   end;
else;
   Ht = H(:,:,FirstPeriod+DSGEModel.KalmanFirstObservation-1:LastPeriod);
   if rank(R)>0;
      if k>=1;
         for t=1:T;
            wtT(:,t) = Y(:,t)-(A'*X(:,t))-(Ht(:,:,t)'*KsitT(:,t));
            ObsIS(:,t) = Ht(:,:,t)'*StateIS(:,t);
         end;
      else;
         for t=1:T;
            wtT(:,t) = Y(:,t)-(Ht(:,:,t)'*KsitT(:,t));
            ObsIS(:,t) = Ht(:,:,t)'*StateIS(:,t);
         end;
      end;
   else;
      for t=1:T;
         ObsIS(:,t) = Ht(:,:,t)'*StateIS(:,t);
      end;
   end;
   if YNaNs==1;
      ObsIS(isnan(Y)) = NaN;
   end;
end;
%
% now turn to the recent and past components for the two decomposition types
% "Recent & Past" and "Sample-split"
%
if IsRecentPast==1;
   %
   % the lag until past is relative
   %
   Options.RecentPastLag = SelectedOption;
   tau = SelectedOption;
   Fi = eye(size(F,1));
   for ti=0:tau-1;
      for j=1:q;
         for t=ti+1:T;
            StateRecent(:,t,j) = StateRecent(:,t,j)+(Fi*B0(:,j)*etatT(j,t-ti));
         end;
      end;
      %
      % update Fi. This is the speediest algorithm since F may have high dimension making
      % F^i a costly computation
      %
      Fi = Fi*F;
   end;
   %
   for ti=tau:T;
      for j=1:q;
         for t=ti+1:T;
            StatePast(:,t,j) = StatePast(:,t,j)+(Fi*B0(:,j)*etatT(j,t-ti));
         end;
      end;
      %
      % update Fi. This is the speediest algorithm since F may have high dimension making
      % F^i a costly computation
      %
      Fi = Fi*F;
   end;
   %
   % now turn to the observed
   %
   if length(size(H))==2;
      if YNaNs==0;
         for j=1:q;
            ObsPast(:,:,j) = H'*StatePast(:,:,j);
            ObsRecent(:,:,j) = H'*StateRecent(:,:,j);
         end;
      else;
         for t=1:T;
            pos = (~isnan(Y(:,t)));
            for j=1:q;
               ObsPast(pos,t,j) = H(:,pos)'*StatePast(:,t,j);
               ObsRecent(pos,t,j) = H(:,pos)'*StateRecent(:,t,j);
            end;
         end;
      end;
   else;
      if YNaNs==0;
         for t=1:T;
            for j=1:q;
               ObsPast(:,t,j) = Ht(:,:,t)'*StatePast(:,t,j);
               ObsRecent(:,t,j) = Ht(:,:,t)'*StateRecent(:,t,j);
            end;
         end;
      else;
         for t=1:T;
            pos = (~isnan(Y(:,t)));
            for j=1:q;
               ObsPast(pos,t,j) = Ht(:,pos,t)'*StatePast(:,t,j);
               ObsRecent(pos,t,j) = Ht(:,pos,t)'*StateRecent(:,t,j);
            end;
         end;
      end;
   end;
   %
   % annualization
   %
   if IsOriginal==0;
      Results.AnnY = Annualize(Y,DSGEModel.annual,DSGEModel.annualscale);
      Tann = size(Results.AnnY,2);
      Results.AnnObsSS = Annualize(ObsSS,DSGEModel.annual,DSGEModel.annualscale);
      Results.AnnObsIS = Annualize(ObsIS,DSGEModel.annual,DSGEModel.annualscale);
      Results.AnnObsPast = zeros(n,Tann,q);
      Results.AnnObsRecent = zeros(n,Tann,q);
      Results.AnnMeasErr = Annualize(wtT,DSGEModel.annual,DSGEModel.annualscale);
      %
      % check which type of annualization is used
      %
      if max(DSGEModel.annual)==4;
         %
         % we have quarterly data
         %
         XTmatrix = zeros(n,T,q,3);
         Fi = eye(size(F,1));
         if length(size(H))==2;
            for ti=0:tau-2;
               if ti<=tau-2;
                  for j=1:q;
                     for t=ti+1:T;
                        XTmatrix(:,t,j,1) = XTmatrix(:,t,j,1)+(H'*Fi*B0(:,j)*etatT(j,t-ti));
                     end;
                  end;
               end;
               if ti<=tau-3;
                  for j=1:q;
                     for t=ti+1:T;
                        XTmatrix(:,t,j,2) = XTmatrix(:,t,j,2)+(H'*Fi*B0(:,j)*etatT(j,t-ti));
                     end;
                  end;
               end;
               if ti<=tau-4;
                  for j=1:q;
                     for t=ti+1:T;
                        XTmatrix(:,t,j,3) = XTmatrix(:,t,j,3)+(H'*Fi*B0(:,j)*etatT(j,t-ti));
                     end;
                  end;
               end;
               %
               % update Fi. This is the speediest algorithm since F may have high dimension making
               % F^i a costly computation
               %
               Fi = Fi*F;
            end;
         else;
            for ti=0:tau-2;
               if ti<=tau-2;
                  for j=1:q;
                     for t=ti+1:T;
                        XTmatrix(:,t,j,1) = XTmatrix(:,t,j,1)+(Ht(t-ti)'*Fi*B0(:,j)*etatT(j,t-ti));
                     end;
                  end;
               end;
               if ti<=tau-3;
                  for j=1:q;
                     for t=ti+1:T;
                        XTmatrix(:,t,j,2) = XTmatrix(:,t,j,2)+(Ht(t-ti)'*Fi*B0(:,j)*etatT(j,t-ti));
                     end;
                  end;
               end;
               if ti<=tau-4;
                  for j=1:q;
                     for t=ti+1:T;
                        XTmatrix(:,t,j,3) = XTmatrix(:,t,j,3)+(Ht(t-ti)'*Fi*B0(:,j)*etatT(j,t-ti));
                     end;
                  end;
               end;
               %
               % update Fi. This is the speediest algorithm since F may have high dimension making
               % F^i a costly computation
               %
               Fi = Fi*F;
            end;
         end;
         %
         % we can now annualize ObsPast and ObsRecent
         %
         for t=4:T;
            for j=1:q;
               Results.AnnObsRecent(:,t-3,j) = Annualize([XTmatrix(:,t-3,j,3) XTmatrix(:,t-2,j,2) XTmatrix(:,t-1,j,1) ObsRecent(:,t,j)],DSGEModel.annual,DSGEModel.annualscale);
               Results.AnnObsPast(:,t-3,j) = Annualize([ObsRecent(:,t-3,j)-XTmatrix(:,t-3,j,3)+ObsPast(:,t-3,j) ObsRecent(:,t-2,j)-XTmatrix(:,t-2,j,2)+ObsPast(:,t-2,j) ObsRecent(:,t-1,j)-XTmatrix(:,t-1,j,1)+ObsPast(:,t-1,j) ObsPast(:,t,j)],DSGEModel.annual,DSGEModel.annualscale);
            end;
         end;
         %
      elseif max(DSGEModel.annual)==12;
         %
         % we have monthly data
         %
         XTmatrix = zeros(n,T,q,11);
         if length(size(H))==2;
            for ti=0:tau-2;
               if ti<=tau-2;
                  for j=1:q;
                     for t=ti+1:T;
                        XTmatrix(:,t,j,1) = XTmatrix(:,t,j,1)+(H'*Fi*B0(:,j)*etatT(j,t-ti));
                     end;
                  end;
               end;
               if ti<=tau-3;
                  for j=1:q;
                     for t=ti+1:T;
                        XTmatrix(:,t,j,2) = XTmatrix(:,t,j,2)+(H'*Fi*B0(:,j)*etatT(j,t-ti));
                     end;
                  end;
               end;
               if ti<=tau-4;
                  for j=1:q;
                     for t=ti+1:T;
                        XTmatrix(:,t,j,3) = XTmatrix(:,t,j,3)+(H'*Fi*B0(:,j)*etatT(j,t-ti));
                     end;
                  end;
               end;
               if ti<=tau-5;
                  for j=1:q;
                     for t=ti+1:T;
                        XTmatrix(:,t,j,4) = XTmatrix(:,t,j,4)+(H'*Fi*B0(:,j)*etatT(j,t-ti));
                     end;
                  end;
               end;
               if ti<=tau-6;
                  for j=1:q;
                     for t=ti+1:T;
                        XTmatrix(:,t,j,5) = XTmatrix(:,t,j,5)+(H'*Fi*B0(:,j)*etatT(j,t-ti));
                     end;
                  end;
               end;
               if ti<=tau-7;
                  for j=1:q;
                     for t=ti+1:T;
                        XTmatrix(:,t,j,6) = XTmatrix(:,t,j,6)+(H'*Fi*B0(:,j)*etatT(j,t-ti));
                     end;
                  end;
               end;
               if ti<=tau-8;
                  for j=1:q;
                     for t=ti+1:T;
                        XTmatrix(:,t,j,7) = XTmatrix(:,t,j,7)+(H'*Fi*B0(:,j)*etatT(j,t-ti));
                     end;
                  end;
               end;
               if ti<=tau-9;
                  for j=1:q;
                     for t=ti+1:T;
                        XTmatrix(:,t,j,8) = XTmatrix(:,t,j,8)+(H'*Fi*B0(:,j)*etatT(j,t-ti));
                     end;
                  end;
               end;
               if ti<=tau-10;
                  for j=1:q;
                     for t=ti+1:T;
                        XTmatrix(:,t,j,9) = XTmatrix(:,t,j,9)+(H'*Fi*B0(:,j)*etatT(j,t-ti));
                     end;
                  end;
               end;
               if ti<=tau-11;
                  for j=1:q;
                     for t=ti+1:T;
                        XTmatrix(:,t,j,10) = XTmatrix(:,t,j,10)+(H'*Fi*B0(:,j)*etatT(j,t-ti));
                     end;
                  end;
               end;
               if ti<=tau-12;
                  for j=1:q;
                     for t=ti+1:T;
                        XTmatrix(:,t,j,11) = XTmatrix(:,t,j,11)+(H'*Fi*B0(:,j)*etatT(j,t-ti));
                     end;
                  end;
               end;
               %
               % update Fi. This is the speediest algorithm since F may have high dimension making
               % F^i a costly computation
               %
               Fi = Fi*F;
            end;
         else;
            for ti=0:tau-2;
               if ti<=tau-2;
                  for j=1:q;
                     for t=ti+1:T;
                        XTmatrix(:,t,j,1) = XTmatrix(:,t,j,1)+(Ht(t-ti)'*Fi*B0(:,j)*etatT(j,t-ti));
                     end;
                  end;
               end;
               if ti<=tau-3;
                  for j=1:q;
                     for t=ti+1:T;
                        XTmatrix(:,t,j,2) = XTmatrix(:,t,j,2)+(Ht(t-ti)'*Fi*B0(:,j)*etatT(j,t-ti));
                     end;
                  end;
               end;
               if ti<=tau-4;
                  for j=1:q;
                     for t=ti+1:T;
                        XTmatrix(:,t,j,3) = XTmatrix(:,t,j,3)+(Ht(t-ti)'*Fi*B0(:,j)*etatT(j,t-ti));
                     end;
                  end;
               end;
               if ti<=tau-5;
                  for j=1:q;
                     for t=ti+1:T;
                        XTmatrix(:,t,j,4) = XTmatrix(:,t,j,4)+(Ht(t-ti)'*Fi*B0(:,j)*etatT(j,t-ti));
                     end;
                  end;
               end;
               if ti<=tau-6;
                  for j=1:q;
                     for t=ti+1:T;
                        XTmatrix(:,t,j,5) = XTmatrix(:,t,j,5)+(Ht(t-ti)'*Fi*B0(:,j)*etatT(j,t-ti));
                     end;
                  end;
               end;
               if ti<=tau-7;
                  for j=1:q;
                     for t=ti+1:T;
                        XTmatrix(:,t,j,6) = XTmatrix(:,t,j,6)+(Ht(t-ti)'*Fi*B0(:,j)*etatT(j,t-ti));
                     end;
                  end;
               end;
               if ti<=tau-8;
                  for j=1:q;
                     for t=ti+1:T;
                        XTmatrix(:,t,j,7) = XTmatrix(:,t,j,7)+(Ht(t-ti)'*Fi*B0(:,j)*etatT(j,t-ti));
                     end;
                  end;
               end;
               if ti<=tau-9;
                  for j=1:q;
                     for t=ti+1:T;
                        XTmatrix(:,t,j,8) = XTmatrix(:,t,j,8)+(Ht(t-ti)'*Fi*B0(:,j)*etatT(j,t-ti));
                     end;
                  end;
               end;
               if ti<=tau-10;
                  for j=1:q;
                     for t=ti+1:T;
                        XTmatrix(:,t,j,9) = XTmatrix(:,t,j,9)+(Ht(t-ti)'*Fi*B0(:,j)*etatT(j,t-ti));
                     end;
                  end;
               end;
               if ti<=tau-11;
                  for j=1:q;
                     for t=ti+1:T;
                        XTmatrix(:,t,j,10) = XTmatrix(:,t,j,10)+(Ht(t-ti)'*Fi*B0(:,j)*etatT(j,t-ti));
                     end;
                  end;
               end;
               if ti<=tau-12;
                  for j=1:q;
                     for t=ti+1:T;
                        XTmatrix(:,t,j,11) = XTmatrix(:,t,j,11)+(Ht(t-ti)'*Fi*B0(:,j)*etatT(j,t-ti));
                     end;
                  end;
               end;
               %
               % update Fi. This is the speediest algorithm since F may have high dimension making
               % F^i a costly computation
               %
               Fi = Fi*F;
            end;
         end;
         %
         % we can now annualize ObsPast and ObsRecent
         %
         for t=12:T;
            for j=1:q;
               Results.AnnObsRecent(:,t-11,j) = Annualize([XTmatrix(:,t-11,j,11) XTmatrix(:,t-10,j,10) XTmatrix(:,t-9,j,9) XTmatrix(:,t-8,j,8) XTmatrix(:,t-7,j,7) XTmatrix(:,t-6,j,6) XTmatrix(:,t-5,j,5) XTmatrix(:,t-4,j,4) XTmatrix(:,t-3,j,3) XTmatrix(:,t-2,j,2) XTmatrix(:,t-1,j,1) ObsRecent(:,t,j)],DSGEModel.annual,DSGEModel.annualscale);
               Results.AnnObsPast(:,t-11,j) = Annualize([ObsRecent(:,t-11,j)-XTmatrix(:,t-11,j,11)+ObsPast(:,t-11,j) ObsRecent(:,t-10,j)-XTmatrix(:,t-10,j,10)+ObsPast(:,t-10,j) ObsRecent(:,t-9,j)-XTmatrix(:,t-9,j,9)+ObsPast(:,t-9,j) ObsRecent(:,t-8,j)-XTmatrix(:,t-8,j,8)+ObsPast(:,t-8,j) ObsRecent(:,t-7,j)-XTmatrix(:,t-7,j,7)+ObsPast(:,t-7,j) ObsRecent(:,t-6,j)-XTmatrix(:,t-6,j,6)+ObsPast(:,t-6,j) ObsRecent(:,t-5,j)-XTmatrix(:,t-5,j,5)+ObsPast(:,t-5,j) ObsRecent(:,t-4,j)-XTmatrix(:,t-4,j,4)+ObsPast(:,t-4,j) ObsRecent(:,t-3,j)-XTmatrix(:,t-3,j,3)+ObsPast(:,t-3,j) ObsRecent(:,t-2,j)-XTmatrix(:,t-2,j,2)+ObsPast(:,t-2,j) ObsRecent(:,t-1,j)-XTmatrix(:,t-1,j,1)+ObsPast(:,t-1,j) ObsPast(:,t,j)],DSGEModel.annual,DSGEModel.annualscale);
            end;
         end;
      else;
         %
         % annual data, nothing to do
         %
         Results.AnnY = Y;
         Results.AnnObsSS = ObsSS;
         Results.AnnObsIS = ObsIS;
         Results.AnnObsPast = ObsPast;
         Results.AnnObsRecent = ObsRecent;
         Results.AnnMeasErr = wtT;
      end;
   end;
else;
   %
   % we have the sample-split decomposition
   %
   Options.SampleSplitPeriod = NumPeriods-SelectedOption;
   T0 = SelectedOption;
   if IsOriginal==0;
      T0 = T0+max(DSGEModel.annual)-1;
   end;
   Fi = eye(size(F,1));
   for ti=0:T-1;
      %
      % select a shock
      %
      for j=1:q;
         for t=ti+1:T0;
            StatePast(:,t,j) = StatePast(:,t,j)+(Fi*B0(:,j)*etatT(j,t-ti));
         end;
         for t=max(T0,ti)+1:T;
            %
            % check if the shock is dated after T0
            %
            if t-ti>T0;
               StateRecent(:,t,j) = StateRecent(:,t,j)+(Fi*B0(:,j)*etatT(j,t-ti));
            else;
               StatePast(:,t,j) = StatePast(:,t,j)+(Fi*B0(:,j)*etatT(j,t-ti));
            end;
         end;
      end;
      %
      % update Fi. This is the speediest algorithm since F may have high dimension making
      % F^i a costly computation
      %
      Fi = Fi*F;
   end;
   %
   % now turn to the observed
   %
   if length(size(H))==2;
      if YNaNs==0;
         for j=1:q;
            ObsPast(:,:,j) = H'*StatePast(:,:,j);
            ObsRecent(:,:,j) = H'*StateRecent(:,:,j);
         end;
      else;
         for t=1:T;
            pos = (~isnan(Y(:,t)));
            if t<=T0;
               for j=1:q;
                  ObsPast(pos,t,j) = H(:,pos)'*StatePast(:,t,j);
               end;
            else;
               for j=1:q;
                  ObsRecent(pos,t,j) = H(:,pos)'*StateRecent(:,t,j);
               end;
            end;
         end;
      end;
   else;
      if YNaNs==0;
         for t=1:T;
            for j=1:q;
               ObsPast(:,t,j) = Ht(:,:,t)'*StatePast(:,t,j);
               ObsRecent(:,t,j) = Ht(:,:,t)'*StateRecent(:,t,j);
            end;
         end;
      else;
         for t=1:T;
            pos = (~isnan(Y(:,t)));
            if t<=T0;
               for j=1:q;
                  ObsPast(pos,t,j) = Ht(:,pos,t)'*StatePast(:,t,j);
               end;
            else;
               for j=1:q;
                  ObsRecent(pos,t,j) = Ht(:,pos,t)'*StateRecent(:,t,j);
               end;
            end;
         end;
      end;
   end;
   %
   % annualization
   %
   if IsOriginal==0;
      Results.AnnY = Annualize(Y,DSGEModel.annual,DSGEModel.annualscale);
      Tann = size(Results.AnnY,2);
      Results.AnnObsSS = Annualize(ObsSS,DSGEModel.annual,DSGEModel.annualscale);
      Results.AnnObsIS = Annualize(ObsIS,DSGEModel.annual,DSGEModel.annualscale);
      Results.AnnObsPast = zeros(n,Tann,q);
      Results.AnnObsRecent = zeros(n,Tann,q);
      Results.AnnMeasErr = Annualize(wtT,DSGEModel.annual,DSGEModel.annualscale);
      for j=1:q;
         Results.AnnObsRecent(:,:,j) = Annualize(ObsRecent(:,:,j),DSGEModel.annual,DSGEModel.annualscale);
         Results.AnnObsPast(:,:,j) = Annualize(ObsPast(:,:,j),DSGEModel.annual,DSGEModel.annualscale);
      end;
   end;
end;
%
Results.Y = Y;
Results.ObsSS = ObsSS;
Results.StateIS = StateIS;
Results.ObsIS = ObsIS;
Results.StatePast = StatePast;
Results.ObsPast = ObsPast;
Results.StateRecent = StateRecent;
Results.ObsRecent = ObsRecent;
Results.MeasErr = wtT;
Results.States = KsitT;
Results.tauT0 = SelectedOption;
Results.KeepVar = KeepShocks;
Results.ShockNames = ShockNames;
Results.SampleStrMat = SampleStrMat(FirstPeriod+DSGEModel.KalmanFirstObservation-1:LastPeriod,:);

%
% end of DSGERecentPastDecomp.m
%
