function [MeanPrediction,CovPrediction] = ZLBDSGEPredictiveMoments(MatFile,ModelStr,CondStr,ShockMethod,DSGEModel,CurrINI,h,NumDraws,NumPredPaths,CurrChain,IsPosterior,Weights)
% ZLBDSGEPredictiveMoments: Computes the predictive moments from the predictive paths based on the ZLB.
%
% USAGE:
%
%       [MeanPrediction,CovPrediction] = ZLBDSGEPredictiveMoments(MatFile,ModelStr,CondStr,ShockMethod,DSGEModel,CurrINI,X,h,NumDraws,NumPredPaths,CurrChain,IsPosterior,Weights)
%
% REQUIRED INPUT:  ModelStr (string vector) that is either "DSGE" or "AdaptiveLearningDSGE-x".
%
%                  CondStr (string vector) that is either "Cond" or "".
%
%                  ShockMethod (string vector) that is either "Direct-", "WaggonerZha-", "Subset-" or "".
%
%                  DSGEModel (structure) with fields containing all the information about a DSGE Model for YADA.
%
%                  CurrINI (structure), with initialization information.
%
%                  h (integer) with the length of the prediction horizon.
%
%                  NumDraws (integer) with the total number of parameter draws.
%
%                  NumPredPaths (integer) with tht total number of prediction paths per parameter draw.
%
%                  CurrChain (integer) with the number for the current Markov chain.
%
%                  IsPosterior (boolean) that is 1 if draws from the posterior distribution are used and 0
%                              if draws from the prior are supplied.
%
%                  Weights (vector) of dimension NumDraws x 1 with the importance weights from the SMC with
%                              likelihood tempering posterior sampler.
%
% REQUIRED OUTPUT: MeanPrediction (matrix) of dimension nxh with the mean predictions.
%
%                  CovPrediction (matrix) of dimension nxnxMaxPH with the sample covariance matrix for the predictions.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2025 European Central Bank.
%
%                       First version: February 7, 2025.
%                        This version: February 12, 2025.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 10-02-2025: Updated the documentation.
%
% * 12-02-2025: Added the PITMoments calculations.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialze
%
if isempty(Weights)==1;
   Weights = ones(NumDraws,1);
end;
n = DSGEModel.n;
MeanPrediction = zeros(n,h);
CovPrediction = zeros(n,n,h);
PITMoments = [];
%
TotalPaths = NumPredPaths*NumDraws;
for ParamIndex=1:NumDraws;
   if IsPosterior==1;
      file = FixFilePath([DSGEModel.OutputDirectory '\predictions\' ModelStr '-ZLB-' CondStr 'Prediction-Original-0-' ShockMethod DSGEModel.NameOfModel '-' int2str(CurrChain) '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '.' int2str(DSGEModel.ZLBRuleType) '-' int2str(h) '-' int2str(ParamIndex) '.' int2str(NumDraws) '.' int2str(NumPredPaths) '.mat']);
   else;
      file = FixFilePath([DSGEModel.OutputDirectory '\predictions\' ModelStr '-ZLB-' CondStr 'Prediction-Original-Prior-0-' ShockMethod DSGEModel.NameOfModel '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '.' int2str(DSGEModel.ZLBRuleType) '-' int2str(h) '-' int2str(ParamIndex) '.' int2str(NumDraws) '.' int2str(NumPredPaths) '.mat']);
   end;
   PredData = load(file);
   PredPaths = PredData.PredPaths;
   MeanPrediction = MeanPrediction+((1/TotalPaths)*sum(PredPaths*Weights(ParamIndex),3));
   MeanY = (1/NumPredPaths)*sum(PredPaths*Weights(ParamIndex),3);
   PITMoments(ParamIndex).PredictY = MeanY;
   PITMoments(ParamIndex).PredictYCov = zeros(n,n,h);
   for j=1:h;
      PP = shiftdim(PredPaths(:,j,:),2)';
      CovPrediction(:,:,j) = CovPrediction(:,:,j)+((Weights(ParamIndex)/TotalPaths)*PP*PP');
      Cov = ((Weights(ParamIndex)/NumPredPaths)*PP*PP')-(MeanY(:,j)*MeanY(:,j)');
      PITMoments(ParamIndex).PredictYCov(:,:,j) = diag(abs(diag(Cov)))+tril(Cov,-1)+tril(Cov,-1)';
   end;
end;
%
% compute the central second moment from the non-central
%
for j=1:h;
   CovPrediction(:,:,j) = CovPrediction(:,:,j)-(MeanPrediction(:,j)*MeanPrediction(:,j)');
   CovPrediction(:,:,j) = diag(abs(diag(CovPrediction(:,:,j))))+tril(CovPrediction(:,:,j),-1)+tril(CovPrediction(:,:,j),-1)';
end;
%
% save to file
%
save(MatFile,'MeanPrediction','CovPrediction','PITMoments');

%
% end of ZLBDSGEPredictiveMoments.m
%
