function [DoneCalc,PredEventData,ObsVarMean] = ZLBAdaptiveLearningDSGECondPredictionPathsMixed(thetaPostSample,thetaMode,thetaPositions,ModelParameters,DSGEModel,CurrINI,X,Z,U,h,NumPaths,AnnualizeData,TransData,CurrChain,IsPosterior,Weights)
% ZLBAdaptiveLearningDSGECondPredictionPathsMixed: Calculates a certain number of possible paths that the endogenous variables
%                                                  can take out-of-sample for a given sample of draws from the posterior of the
%                                                  parameters in the DSGE model subject to adaptive learning and when the monetary
%                                                  policy rate is subject to a zero lower bound. The conditioning assumptions are
%                                                  imposed with the mixed approach.
%
% USAGE:
%
%       [DoneCalc,PredEventData,ObsVarMean] = ZLBAdaptiveLearningDSGECondPredictionPathsMixed(thetaPostSample,thetaMode,thetaPositions,...
%                                                         ModelParameters,DSGEModel,CurrINI,X,Z,U,h,NumPaths,AnnualizeData,TransData,...
%                                                         CurrChain,IsPosterior,Weights)
%
% REQUIRED INPUT:  thetaPostSample (matrix) of dimension NumDrawsxNumParam with draws from the posterior or prior
%                                  distribution of the original DSGE model parameters.
%
%                  thetaMode (vector) with the posterior mode or initial value of the original DSGE model parameters.
%
%                  thetaPositions (structure) with fields parameter and distribution. This function
%                                  needs the former field to know which parameter name corresponds to
%                                  position i in theta.
%
%                  ModelParameters (structure) whose fields have names given by the parameter names in the
%                                  DSGE model.
%
%                  DSGEModel (structure) with fields containing all the information about a DSGE Model for YADA.
%
%                  CurrINI (structure), with initialization information.
%
%                  X (matrix) of dimension kxMaxPH with the observations of the exogenous variables for
%                                  the prediction paths.
%
%                  Z (matrix) of dimension mxg with the out-of-sample data we shall condition the
%                                  predictions on.
%
%                  U (matrix) of dimension mxg with initial values for the mapping between the observed
%                                  data and the conditioning data.
%
%                  h (integer) with the prediction horizon.
%
%                  NumPaths (integer) the number of paths to draw.
%
%                  AnnualizeData (boolean) that is 1 if the prediction paths should be annualized and 0 otherwise.
%
%                  TransData (boolean) that is 1 if the prediction paths should be transformed and 0 otherwise.
%
%                  CurrChain (integer) with the number for the current Markov chain.
%
%                  IsPosterior (boolean) that is 1 if draws from the posterior distribution are used and 0
%                                  if draws from the prior are supplied.
%
%                  Weights (vector) of dimension NumDraws x 1 with the importance weights from the SMC with
%                                  likelihood tempering posterior sampler.
%
% REQUIRED OUTPUT: DoneCalc (boolean) that is 1 if the calculations were completed and 0 otherwise.
%
%                  PredEventData (n x 8) matrix where the first column holds the number of times
%                                  the PredPaths data satisfies the event, the second hold the number
%                                  of times the PredPaths data is greater than the event, the third
%                                  the number of time the data is above the upper bound, while the
%                                  fourth column holds the number of times the event has been tested.
%                                  The fifth and sixth column measure the average deviation from the
%                                  lower bound when below the lower bound and the square of this, while
%                                  the seventh and eighth columns hold similar data for the upper bound
%                                  of the event when the variables are above the upper bound.
%
%                  ObsVarMean (n x 1) vector with the steady-state values of the observed variables.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2025 European Central Bank.
%
%                       First version: December 11, 2024.
%                        This version: January 7, 2025.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 12-12-2024: Added the parallel code and backup values if a certain parameter values doesn't work
%               in the main loop.
%
% * 13-12-2024: Made sure that ModelParameters is properly updated.
%
% * 06-01-2025: Updated the documentation.
%
% * 07-01-2025: Fixed a bug where the "mode" values of Xbar and DpKInvKpDDpk need to be setup.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
DoneCalc = 0;
PredEventData = [];
ObsVarMean = [];
[NumDraws,NumParam] = size(thetaPostSample);
k = DSGEModel.k;
if isempty(Weights)==1;
   Weights = ones(NumDraws,1);
end;
n = DSGEModel.n;
if (AnnualizeData==1)|(TransData==1);
   if TransData==0;
      TypeStr = 'Annualized';
   else;
      TypeStr = 'Transformed';
   end;
else;
   TypeStr = 'Original';
end;
%
PriorHeader = '';
if IsPosterior==0;
   PriorHeader = ' For Prior';
end;
%
% copy files to the tmp folder
%
[stat,msg] = CopyFile([GetPath(DSGEModel.AIMDataFile) 'compute_aim_matrices.m'],[pwd '\tmp']);
[stat,msg] = CopyFile(DSGEModel.MeasurementEquationFile,[pwd '\tmp']);
if FileExist(DSGEModel.UpdateParameterFile)==1;
   [stat,msg] = CopyFile(DSGEModel.UpdateParameterFile,[pwd '\tmp']);
end;
if FileExist(DSGEModel.SystemPriorFile)==1;
   [stat,msg] = CopyFile(DSGEModel.SystemPriorFile,[pwd '\tmp']);
end;
%
% check if we should copy the belief system file
%
if DSGEModel.BeliefSystem==2;
   [stat,msg] = CopyFile(DSGEModel.BeliefSystemFile,[pwd '\tmp']);
end;
%
% check if we should copy model solver file
%
if DSGEModel.ModelSolver==4;
   if FileExist(DSGEModel.ModelSolverFile)==1;
      [stat,msg] = CopyFile(DSGEModel.ModelSolverFile,[pwd '\tmp']);
   end;
   if FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==1;
      [stat,msg] = CopyFile(DSGEModel.AdaptiveLearningModelSolverFile,[pwd '\tmp']);
   end;
end;
%
% load the AIMData file
%
ErrorStr = '';
if MatlabNumber>=8.0;
   try;
      AIMData = load(DSGEModel.AIMDataFile);
   catch ME;
      ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError(ME)];
   end;
else;
   try;
      AIMData = load(DSGEModel.AIMDataFile);
   catch;
      ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError];
   end;
end;
if isempty(ErrorStr)==0;
   About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
   return;
end;
%
% create the output directory
%
stat = MakeDir(DSGEModel.OutputDirectory,'predictions');
if stat~=1;
   if ispc==1;
      txt = ['YADA was for some reason unable to create the directory "' DSGEModel.OutputDirectory '\predictions". The computation of predictive distributions has therefore been aborted.'];
   else;
      txt = ['YADA was for some reason unable to create the directory "' DSGEModel.OutputDirectory '/predictions". The computation of predictive distributions has therefore been aborted.'];
   end;
   About(txt,'information','YADA - Directory Creation Problem',180,600,CurrINI);
   return;
end;
%
% set the InitialStateVector value
%
if DSGEModel.UseOwnInitialState==1;
   if length(DSGEModel.InitialStateValues)==size(AIMData.endog,1);
      DSGEModel.InitialStateVector = DSGEModel.InitialStateValues(DSGEModel.StateVariablePositions);
   else;
      DSGEModel.InitialStateVector = zeros(length(DSGEModel.StateVariablePositions),1);
   end;
else;
   DSGEModel.InitialStateVector = zeros(length(DSGEModel.StateVariablePositions),1);
end;
%
% Get the sub-sample
%
[DSGEModel.FirstPeriod,DSGEModel.LastPeriod] = CreateSubSample(DSGEModel);
FirstPeriod = DSGEModel.FirstPeriod;
LastPeriod = DSGEModel.LastPeriod;
%
% try to solve the model for the posterior mode estimates
%
ErrorStr = '';
if MatlabNumber>=8.0;
   try;
      [A,H,R,F,B0,mcode,ErrMsg,ModelParameters] = SolveDSGEModel(DSGEModel,thetaMode,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
      if isempty(ErrMsg)==0;
         ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' ErrMsg];
      end;
   catch ME;
      ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' YADALastError(ME)];
   end;
else;
   try;
      [A,H,R,F,B0,mcode,ErrMsg,ModelParameters] = SolveDSGEModel(DSGEModel,thetaMode,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
      if isempty(ErrMsg)==0;
         ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' ErrMsg];
      end;
   catch;
      ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' YADALastError];
   end;
end;
if isempty(ErrorStr)==0;
   About(ErrorStr,'information','YADA - Error Message',CurrINI.scrsz(4)-50,600,CurrINI);
   return;
end;
if mcode~=1;
   if mcode==2;
      mcodeStr = 'Roots are not correctly computed by real_schur.'
   elseif mcode==3;
      mcodeStr = 'Too many big roots.';
   elseif mcode==35;
      mcodeStr = 'Too many big roots, and q(:,right) is singular.';
   elseif mcode==4;
      mcodeStr = 'Too few big roots.';
   elseif mcode==45;
      mcodeStr = 'Too few big roots, and q(:,right) is singular.';
   elseif mcode==5;
      mcodeStr = 'q(:,right) is singular.';
   elseif mcode==61;
      mcodeStr = 'Too many exact shiftrights.';
   elseif mcode==62;
      mcodeStr = 'Too many numeric shiftrights.';
   elseif mcode==7;
      mcodeStr = 'Infinite or NaN values detected.';
   elseif mcode==8;
      mcodeStr = 'The function "compute_aim_matrices" returns complex numbers.';
   elseif mcode==-1;
      mcodeStr = 'No stable solution.';
   elseif mcode==-2;
      mcodeStr = 'Too many large generalized eigenvalues.';
   elseif mcode==-3;
      mcodeStr = 'Too few large generalized eigenvalues.';
   else;
      mcodeStr = 'Return code not properly specified.';
   end;
   if DSGEModel.ModelSolver==1;
      SolverStr = 'AiM';
   elseif DSGEModel.ModelSolver==2;
      SolverStr = 'Klein';
   elseif DSGEModel.ModelSolver==3;
      SolverStr = 'Gensys';
   elseif DSGEModel.ModelSolver==4;
      SolverStr = ['Own algorithm (' GetName(DSGEModel.ModelSolverFile) ')'];
   end;
   txt = ['The ' SolverStr ' solver provided the return code: ' num2str(mcode) ', i.e., "' mcodeStr '"'];
   About(txt,'information',['YADA - ' SolverStr ' Solver Error'],500,600,CurrINI);
   return;
end;
if length(size(H))==3;
   if size(H,3)<DSGEModel.LastPeriod+h;
      %
      % too few data points in the 3D H matrix.
      %
      ErrorStr = ['The number of time periods for the time-varying measurement matrix H (' int2str(size(H,3)) ') is lower than the number of observations plus forecasts (T+h = ' int2str(DSGEModel.LastPeriod+h) '). YADA has therefore aborted from the unconditional predictions.'];
      About(ErrorStr,'error','Error - Measurement Equation',500,600,CurrINI);
      return;
   end;
end;
[S,KeepShocks] = RemoveRedundantColumns(B0);
if isempty(S)==1;
   txt = 'The model doesn''t have any structural shocks. YADA has aborted from the lower bound computations.';
   About(txt,'information',['ZLB Predictive Distributions' PriorHeader],200,600,CurrINI);
   return;
end;
[r,q] = size(B0);
%
% determine the steady-state interest rate path.
%
if k>0;
   XPred = X;
   if size(DSGEModel.X,2)>DSGEModel.LastPeriod;
      X = DSGEModel.X(:,DSGEModel.LastPeriod+1:size(DSGEModel.X,2));
      LastValue = X(:,size(X,2));
   else;
      X = [];
      LastValue = DSGEModel.X(:,DSGEModel.LastPeriod);
   end;
   if isempty(DSGEModel.PredictedX)==0;
      X = [X DSGEModel.PredictedX];
   end;
   Tx = size(X,2);
   Xold = X;
   if Tx>DSGEModel.ZLBSampleT+1;
      X = X(:,1:DSGEModel.ZLBSampleT+1);
   elseif Tx<DSGEModel.ZLBSampleT+1;
      X = [X LastValue*ones(1,DSGEModel.ZLBSampleT+1-Tx)];
   end;
   Rbar = A(:,DSGEModel.ZLBPosition)'*X;
else;
   Rbar = zeros(1,DSGEModel.ZLBSampleT+1);
   X = zeros(1,DSGEModel.ZLBSampleT+1)
end;
if size(X,2)<h;
   X = [X XPred(:,DSGEModel.ZLBSampleT+2:h)];
end;
%
% setup the zero lower bound path
%
Tzlb = length(DSGEModel.ZLBData);
if Tzlb==1;
   Rzlb = DSGEModel.ZLBData*ones(h,DSGEModel.ZLBSampleT+1);
else;
   if Tzlb>DSGEModel.LastPeriod;
      Rzlb = zeros(h,Tzlb-DSGEModel.LastPeriod);
      for i=1:h;
         Rzlb(i,1:Tzlb-DSGEModel.LastPeriod-i+2) = DSGEModel.ZLBData(DSGEModel.LastPeriod+i-1:Tzlb);
      end;
      TZLB = size(Rzlb,2);
      if TZLB>=DSGEModel.ZLBSampleT+1;
         RzlbLast = Rzlb(1,size(Rzlb,2));
         % make the matrix have the correct dimension
         Rzlb = Rzlb(:,1:DSGEModel.ZLBSampleT+1);
         % replace zeros with the last value of the lower bound
         % provided the locations appear after the last nonzero
         for i=1:h;
            for j=2:DSGEModel.ZLBSampleT+1;
               if (Rzlb(i,j)==0)&(Rzlb(i,j-1)~=0);
                  Rzlb(i,j) = RzlbLast;
               end;
            end;
         end;
         %Rzlb((Rzlb==0)) = RzlbLast;
      else;
         %
         % we should add the last value in each row to fill the matrix
         %
         RzlbLast = Rzlb(1,size(Rzlb,2));
         Rzlb((Rzlb==0)) = RzlbLast;
         Rzlb = [Rzlb RzlbLast*ones(h,DSGEModel.ZLBSampleT+1-TZLB)];
      end;
   else;
      Rzlb = DSGEModel.ZLBData(Tzlb)*ones(h,DSGEModel.ZLBSampleT+1);
   end;
end;
%
% since H may involve estimated parameters we save a copy of Rzlb for later use
%
RzlbMat = Rzlb;
%
% Transform the steady-state interest rate and the lower bound to the same units
% as the unrestricted monetary policy rate
%
if length(size(H))==2;
   Hvalue = H(DSGEModel.RtildePosition,DSGEModel.ZLBPosition);
   Rzlb = (1/Hvalue)*Rzlb;
   Rbar = (1/Hvalue)*Rbar;
else;
   Hvalue = shiftdim(H(DSGEModel.RtildePosition,DSGEModel.ZLBPosition,DSGEModel.LastPeriod:Hlength),2)';
   Th = length(Hvalue);
   if Th>DSGEModel.ZLBSampleT;
      Hvalue = Hvalue(1,1:DSGEModel.ZLBSampleT);
   elseif Th<DSGEModel.ZLBSampleT;
      Hvalue = [Hvalue Hvalue(1,Th)*ones(1,DSGEModel.ZLBSampleT-Th)];
   end;
   Rzlb = Rzlb./Hvalue;
   Rbar = Rbar./Hvalue;
end;
%
% setup the parameters that determine the projected restricted policy rate
%
e_pr = zeros(r+1,1);
e_pr(DSGEModel.RtildePosition) = 1;
%
% Determine the belief system matrices
%
if DSGEModel.BeliefSystem==1;
   BeliefSys = YADADefaultBeliefSystem(DSGEModel.X,DSGEModel.XVariableNames,DSGEModel.StateVariableNames,DSGEModel.ForwardVariableNames);
else;
   BeliefSys = feval(GetName(DSGEModel.BeliefSystemFile),DSGEModel.XVariableNames,DSGEModel.StateVariableNames,DSGEModel.ForwardVariableNames);
end;
%
% Compute the transformation of the structural from that we use when solving the model
%
r = length(DSGEModel.StateVariablePositions);
f = length(DSGEModel.ForwardVariablePositions);
m = BeliefSys.m;
q = size(B0,2);
%
% get the structural form matrices
%
[Hlag,H0,Hexp,D] = DSGEStructuralForm(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions);
%
% reshuffle the equations to separate the restricted from the unrestricted policy rate
%
[Hlag,H0,Hexp,D,DA,MA] = ZLBDSGEStructuralForm(Hlag,H0,Hexp,D,DSGEModel.ZLBSampleT,DSGEModel.RtildePosition,DSGEModel.REqPosition,DSGEModel.ZLBRuleType);
%
% solve the model with the new matrices
%
[F,B0,BA,mcode] = SolveExtendedDSGEModel(Hlag,H0,Hexp,D,DA,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
%
% rewrite the G-matrices for the belief equations that match the position of the unrestricted policy rate being last and the restricted policy rate
% being in position "DSGEModel.RtildePosition". The latter variable should have zeros in the belief equations, while the former should have the
% coefficients from "DSGEModel.RtildePosition"
%
G = ZLBBeliefSystem(BeliefSys.G,m,k,r,DSGEModel.RtildePosition,DSGEModel.ZLBRuleType);
%
SMatrix = eye(r+1);
Sperp = SMatrix(:,setdiff((1:r+1),DSGEModel.ForwardVariablePositions));
SMatrix = SMatrix(:,DSGEModel.ForwardVariablePositions);
%
% Determine the first observation to use when computing log-likelihood
%
KalmanFirstObservation = DSGEModel.KalmanFirstObservation;
%
% Setup all the basic data matrix for the deterministic variables
% used for initializing Sigmax
%
if k>0
   x = DSGEModel.X(:,FirstPeriod+KalmanFirstObservation-1:LastPeriod);
else;
   x = [];
end;
T = LastPeriod-(FirstPeriod+KalmanFirstObservation-1)+1;
%
% Compute the initial values of betavec, Sigmau, R10 and Sigmaeps
%
[betavec,Sigmau,R10,Sigmaeps,Gtilde,Kgf] = AdaptiveLearningInitialValuesDSGE(F,B0,ModelParameters.sigmarAL,ModelParameters.sigmaeAL,SMatrix,G,m,x,f,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue);
%
% call the transformation function
%
[TransStatus,Hstarlag,Hstar0,Hstar1f] = AdaptiveLearningTransformDSGEModel(Hlag,H0,Hexp,SMatrix,Sperp);
%
% compute initial values for the solution matrices
%
if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
   [Mt1,Ft1,Bt1,InvH0H1star] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betavec);
else;
   [Mt1,Ft1,Bt1,InvH0H1star] = feval(GetName(DSGEModel.AdaptiveLearningModelSolverFile),Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betavec,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
end;
%
% compute the solution matrix BAt1 on the anticipated shocks
%
BAt1 = InvH0H1star*DA;
%
ConstPos = 0;
if k>0;
   ConstPos = LocateStr(DSGEModel.XVariableNames,'const');
   if ConstPos==0;
      %
      % we will now check if X has a unit vector
      %
      Xhist = DSGEModel.X(:,FirstPeriod:LastPeriod);
      T = size(Xhist,2);
      for j=1:k;
         ConstTest = (Xhist==ones(1,T));
         if sum(ConstTest)==T;
            ConstPos = j;
            break;
         end;
      end;
   end;
   xbar = zeros(k,1);
   xbar(ConstPos) = 1;
else;
   xbar = [];
end;
%
% we need to fix the H-matrix when using the extended form of the model
% by adding zeros to 
%
HOld = H;
if length(size(H))==2;
   H = [H;zeros(1,n)];
else;
   HT = zeros(size(H));
   for t=1:size(H,3);
      HT(:,:,t) = [H(:,:,t);zeros(1,n)];
   end;
   H = HT;
end;
%
% check if the adaptive learning parameters allows for updating of the beliefs
%
IsALUpdate = (CheckAdaptiveLearningFixedBeliefs(ModelParameters)==0);
%
% run the Kalman filter
%
if IsALUpdate==1;
   if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
      [lnL,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningKalmanFilter(m,SMatrix,betavec,Sigmau,R10,Sigmaeps,ModelParameters.rhoAL,Hstar0,Hstarlag,Hstar1f,D,Mt1,Ft1,Bt1,Gtilde,Kgf,DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,KalmanFirstObservation,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs);
   else;
      [lnL,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningUserSolverKalmanFilter(GetName(DSGEModel.AdaptiveLearningModelSolverFile),m,SMatrix,betavec,Sigmau,R10,Sigmaeps,ModelParameters.rhoAL,Hstar0,Hstarlag,Hstar1f,D,Mt1,Ft1,Bt1,Gtilde,Kgf,DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,KalmanFirstObservation,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
   end;
else;
   [lnL,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningSingleKalmanFilter(m,betavec,Mt1,Ft1,Bt1,DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,KalmanFirstObservation,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs);
   LastSolution.InvMatT1 = InvH0H1star;
   LastSolution.InvMatT = InvH0H1star;
end;
%
% get some parameter values
%
T = size(Ksitt1,2);
[qm,g] = size(Z);
q_r = length(DSGEModel.MixedConditionalShocks);
%
% create the mapping matrices for conditioning variables
%
K1 = DSGEModel.K1(:,DSGEModel.ConditionalVariables);
if length(size(DSGEModel.K2))==2;
   K2 = DSGEModel.K2(:,DSGEModel.ConditionalVariables);
   K2length = 2;
else;
   K2 = DSGEModel.K2(:,DSGEModel.ConditionalVariables,:);
   K2length = 3;
end;
SCond = (isequal(U,zeros(size(U)))&isequal(K2,zeros(size(K2))));
if (SCond==1)&(K2length==3);
   K2 = K2(:,:,1);
   K2length = 2;
end;
%
% check if the user wants to take the conditioning infortmation into account
% when determining the mean and covariance matrix for the states at T.
%
if (DSGEModel.KsiUseCondData==1)&(SCond==1);
   if (k>0);
      XPred = X(:,1:g);
   else;
      XPred = [];
   end;
   if length(size(H))==2;
      [KsiTTz,PTTz] = AdaptiveLearningCondPredictionSmoother(Ksitt1(:,T),Ptt1(:,:,T),Yhat(:,T),DSGEModel.Y(:,LastPeriod),XPred,Z,K1,A,H,R,Mtilde(:,:,T),Ftilde(:,:,T),Btilde(:,:,T),betavec,LastSolution.betaTT,LastSolution.M,Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,DSGEModel);
   else;
      [KsiTTz,PTTz] = AdaptiveLearningCondPredictionSmoother(Ksitt1(:,T),Ptt1(:,:,T),Yhat(:,T),DSGEModel.Y(:,LastPeriod),XPred,Z,K1,A,H(:,:,LastPeriod+1:LastPeriod+g),R,Mtilde(:,:,T),Ftilde(:,:,T),Btilde(:,:,T),betavec,LastSolution.betaTT,LastSolution.M,Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,DSGEModel);
   end;
%   KsiTTz = Ksitt(:,T);
%   PTTz = Ptt(:,:,T);
elseif (DSGEModel.KsiUseCondData==1)&(T>=g);
   %
   % we need to run the Kalman smoother
   %
   [KsitT,PtT,etatt,etatT,wtt,wtT] = AdaptiveLearningKalmanSmoother(Ksitt1,Ptt1,Yerror,InvMSEY,H,R,Ftilde,Btilde,FirstPeriod,LastPeriod,KalmanFirstObservation,DSGEModel.YNaNs);
   PtT = PtT(:,:,T-g+1:T);
   KsitT = KsitT(:,T-g+1:T);
   if isempty(wtT)==1;
      wtT = zeros(n,g);
   else;
      wtT = wtT(:,T-g+1:T);
   end;
   if length(size(H))==2;
      if k>0;
         if g>1;
            Xtra = DSGEModel.X(:,LastPeriod-g+2:LastPeriod);
         else;
            Xtra = [];
         end;
         [KsiTTz,PTTz] = AdaptiveLearningCondPredictionKalmanSmoother(KsitT,PtT,wtT,Z,U,[Xtra X(:,1:g)],K1,K2,A,H,R,Mtilde(:,:,T),Ftilde(:,:,T),Btilde(:,:,T),betavec,LastSolution.betaTT,LastSolution.M,Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,DSGEModel);
      else;
         [KsiTTz,PTTz] = AdaptiveLearningCondPredictionKalmanSmoother(KsitT,PtT,wtT,Z,U,[],K1,K2,A,H,R,Mtilde(:,:,T),Ftilde(:,:,T),Btilde(:,:,T),betavec,LastSolution.betaTT,LastSolution.M,Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,DSGEModel);
      end;
   else;
      if k>0;
         if g>1;
            Xtra = DSGEModel.X(:,LastPeriod-g+2:LastPeriod);
         else;
            Xtra = [];
         end;
         [KsiTTz,PTTz] = AdaptiveLearningCondPredictionKalmanSmoother(KsitT,PtT,wtT,Z,U,[Xtra X(:,1:g)],K1,K2,A,H(:,:,LastPeriod-g+2:LastPeriod+g),R,Mtilde(:,:,T),Ftilde(:,:,T),Btilde(:,:,T),betavec,LastSolution.betaTT,LastSolution.M,Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,DSGEModel);
      else;
         [KsiTTz,PTTz] = AdaptiveLearningCondPredictionKalmanSmoother(KsitT,PtT,wtT,Z,U,[],K1,K2,A,H(:,:,LastPeriod-g+2:LastPeriod+g),R,Mtilde(:,:,T),Ftilde(:,:,T),Btilde(:,:,T),betavec,LastSolution.betaTT,LastSolution.M,Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,DSGEModel);
      end;
   end;
else;
   KsiTTz = Ksitt(:,T);
   PTTz = Ptt(:,:,T);
end;
PTTz = diag(abs(diag(PTTz)))+tril(PTTz,-1)+tril(PTTz,-1)';
%
% store solution matrices
%
if DSGEModel.FixedBeliefsForProj==1;
   MT1 = Mtilde(:,:,T);
   FT1 = Ftilde(:,:,T);
   BT1 = Btilde(:,:,T);
else;
   MT1 = zeros(r+1,k,h);
   FT1 = zeros(r+1,r,h);
   BT1 = zeros(r+1,q,h);
   MT1(:,:,1) = Mtilde(:,:,T);
   FT1(:,:,1) = Ftilde(:,:,T);
   BT1(:,:,1) = Btilde(:,:,T);
   InvMatT1 = zeros(r+1,r+1,h);
   InvMatT1(:,:,1) = LastSolution.InvMatT1;
end;
%
% write down the solution matrices for projections using ZLB format, with Mt, Gt and C1
%
[Mt,Gt,C1,Mtilde,Pt] = ZLBProjectionMatrices(Mtilde(:,:,T),Ftilde(:,:,T),Btilde(:,:,T),LastSolution.InvMatT1*DA,MA);
%
% deal with deterministic variables
%
if k>0;
   Ax = A'*X;
else;
   Ax = zeros(n,h);
   X = zeros(r,max(DSGEModel.ZLBSampleT+1,h))
end;
%
% create the matrices for selecting shocks
%
N = eye(q);
M = N(:,DSGEModel.MixedConditionalShocks);
if length(size(H))==2;
   etaMat = K1'*H'*Btilde(:,:,T)*M;
   if rank(etaMat)<qm;
      txt = ['The shocks you have selected for conditioning cannot be used due to a singularity in the solution algorithm. Estimation of the conditional predictive distribution' lower(NameStr) ' has therfore been aborted.'];
      About(txt,'error','Error - Conditional Shocks',150,600,CurrINI);
      PredPaths = [];
      return;
   end;
else;
   BadEtaMat = 0;
   for i=1:g;
      etaMat = K1'*H(:,:,LastPeriod+i)'*Btilde(:,:,T)*M;
      if rank(etaMat)<qm;
         txt = ['The shocks you have selected for conditioning cannot be used due to a singularity in the solution algorithm. Estimation of the conditional predictive distribution' lower(NameStr) ' has therfore been aborted.'];
         About(txt,'error','Error - Conditional Shocks',150,600,CurrINI);
         PredPaths = [];
         BadEtaMat = 1;
         break;
      end;
   end;
   if BadEtaMat==1;
      return;
   end;
end;
%
% stack the model to determine the shock distribution
%
Xbar = zeros(n*g,1);
XTi = zeros(r+1,1);
Gmat = zeros(n*g,r+1);
WTg = zeros(n*g,1);
K = zeros(n*g,qm*g);
DT1 = zeros(n*g,q*g);
ZTg = zeros(qm*g,1);
UT = zeros(qm*g,1);
Fexp = eye(r+1);
if length(size(H))==2;
   if DSGEModel.FixedBeliefsForProj==1;
      for i=1:g;
         Fexp = FT1*Fexp;
         Ind = (1+((g-i)*n):(g-i+1)*n);
         Ind1 = (1+((g-i)*q):(g-i+1)*q);
         Ind2 = (1+((g-i)*qm):(g-i+1)*qm);
         if k>0;
            XTi = (MT1*X(:,i))+(FT1*XTi);
            Xbar(Ind,1) = (A'*X(:,i))+(H'*XTi);
         end;
         Gmat(Ind,:) = H'*Fexp;
         DT1(Ind,Ind1) = H'*BT1;
         K(Ind,Ind2) = K1;
         Fi = FT1;
         for j=1:i-1;
            Ind1 = Ind1+q;
            Ind3 = Ind+(j*n);
            DT1(Ind,Ind1) = H'*Fi*BT1;
            if K2length==2;
               K(Ind3,Ind2) = K2;
            else;
               if j<=size(K2,3);
                  K(Ind3,Ind2) = K2(:,:,j);
               end;
            end;
            Fi = Fi*FT1;
         end;
         ZTg(Ind2,1) = Z(:,i);
         UT(Ind2,1) = U(:,g);
      end;
   else;
      %
      % we have updated beliefs
      %
      for i=1:g;
         Fexp = FT1(:,:,i)*Fexp;
         Ind = (1+((g-i)*n):(g-i+1)*n);
         Ind1 = (1+((g-i)*q):(g-i+1)*q);
         Ind2 = (1+((g-i)*qm):(g-i+1)*qm);
         if k>0;
            XTi = (MT1(:,:,i)*X(:,i))+(FT1(:,:,i)*XTi);
            Xbar(Ind,1) = (A'*X(:,i))+(H'*XTi);
         end;
         Gmat(Ind,:) = H'*Fexp;
         DT1(Ind,Ind1) = H'*BT1(:,:,i);
         K(Ind,Ind2) = K1;
         Fi = FT1(:,:,i);
         for j=1:i-1;
            Ind1 = Ind1+q;
            Ind3 = Ind+(j*n);
            DT1(Ind,Ind1) = H'*Fi*BT1(:,:,i);
            if K2length==2;
               K(Ind3,Ind2) = K2;
            else;
               if j<=size(K2,3);
                  K(Ind3,Ind2) = K2(:,:,j);
               end;
            end;
            Fi = Fi*FT1(:,:,i-j);
         end;
         ZTg(Ind2,1) = Z(:,i);
         UT(Ind2,1) = U(:,g);
         %
         % require new solution matrices
         %
         if i==1;
            MT1(:,:,i+1) = LastSolution.MTT;
            FT1(:,:,i+1) = LastSolution.FTT;
            BT1(:,:,i+1) = LastSolution.BTT;
            betaTiT = LastSolution.betaTT;
            InvMatT1(:,:,i) = LastSolution.InvMatT;
         elseif (i>1)&(i<g);
            %
            % we first need to project betaTiT
            %
            betaTiT = betavec+(LastSolution.M*(betaTiT-betavec));
            if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
               [MTiT,FTiT,BTiT] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betaTiT);
            else;
               [MTiT,FTiT,BTiT] = feval(GetName(DSGEModel.AdaptiveLearningModelSolverFile),Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betaTiT,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
            end;
            MT1(:,:,i+1) = MTiT;
            FT1(:,:,i+1) = FTiT;
            BT1(:,:,i+1) = BTiT;
            InvMatT1(:,:,i+1) = InvMatT;
         end;
      end;
   end;
else;
   if DSGEModel.FixedBeliefsForProj==1;
      for i=1:g;
         Fexp = FT1*Fexp;
         Ind = (1+((g-i)*n):(g-i+1)*n);
         Ind1 = (1+((g-i)*q):(g-i+1)*q);
         Ind2 = (1+((g-i)*qm):(g-i+1)*qm);
         if k>0;
            XTi = (MT1*X(:,i))+(FT1*XTi);
            Xbar(Ind,1) = (A'*X(:,i))+(H(:,:,LastPeriod+i)'*XTi);
         end;
         Gmat(Ind,:) = H(:,:,LastPeriod+i)'*Fexp;
         DT1(Ind,Ind1) = H(:,:,LastPeriod+i)'*BT1;
         K(Ind,Ind2) = K1;
         Fi = FT1;
         for j=1:i-1;
            Ind1 = Ind1+q;
            Ind3 = Ind+(j*n);
            DT1(Ind,Ind1) = H(:,:,LastPeriod+i)'*Fi*BT1;
            if K2length==2;
               K(Ind3,Ind2) = K2;
            else;
               if j<=size(K2,3);
                  K(Ind3,Ind2) = K2(:,:,j);
               end;
            end;
            Fi = Fi*FT1;
         end;
         ZTg(Ind2,1) = Z(:,i);
         UT(Ind2,1) = U(:,g);
      end;
   else;
      %
      % we have updated beliefs
      %
      for i=1:g;
         Fexp = FT1(:,:,i)*Fexp;
         Ind = (1+((g-i)*n):(g-i+1)*n);
         Ind1 = (1+((g-i)*q):(g-i+1)*q);
         Ind2 = (1+((g-i)*qm):(g-i+1)*qm);
         if k>0;
            XTi = (MT1(:,:,i)*X(:,i))+(FT1(:,:,i)*XTi);
            Xbar(Ind,1) = (A'*X(:,i))+(H(:,:,LastPeriod+i)'*XTi);
         end;
         Gmat(Ind,:) = H(:,:,LastPeriod+i)'*Fexp;
         DT1(Ind,Ind1) = H(:,:,LastPeriod+i)'*BT1(:,:,i);
         K(Ind,Ind2) = K1;
         Fi = FT1(:,:,i);
         for j=1:i-1;
            Ind1 = Ind1+q;
            Ind3 = Ind+(j*n);
            DT1(Ind,Ind1) = H(:,:,LastPeriod+i)'*Fi*BT1(:,:,i);
            if K2length==2;
               K(Ind3,Ind2) = K2;
            else;
               if j<=size(K2,3);
                  K(Ind3,Ind2) = K2(:,:,j);
               end;
            end;
            Fi = Fi*FT1(:,:,i-j);
         end;
         ZTg(Ind2,1) = Z(:,i);
         UT(Ind2,1) = U(:,g);
         %
         % require new solution matrices
         %
         if i==1;
            MT1(:,:,i+1) = LastSolution.MTT;
            FT1(:,:,i+1) = LastSolution.FTT;
            BT1(:,:,i+1) = LastSolution.BTT;
            betaTiT = LastSolution.betaTT;
            InvMatT1(:,:,i+1) = LastSolution.InvMatT;
         elseif (i>1)&(i<g);
            %
            % we first need to project betaTiT
            %
            betaTiT = betavec+(LastSolution.M*(betaTiT-betavec));
            if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
               [MTiT,FTiT,BTiT] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betaTiT);
            else;
               [MTiT,FTiT,BTiT] = feval(GetName(DSGEModel.AdaptiveLearningModelSolverFile),Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betaTiT,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
            end;
            MT1(:,:,i+1) = MTiT;
            FT1(:,:,i+1) = FTiT;
            BT1(:,:,i+1) = BTiT;
            InvMatT1(:,:,i+1) = InvMatT;
         end;
      end;
   end;
end;
%
% check which conditioning assumptions to use
%
PosVec = (1:qm*g)' .* ~isnan(ZTg);
PosVec = PosVec(PosVec>0);
ZTg = ZTg(PosVec);
UT = UT(PosVec);
K = K(:,PosVec);
Morth = N(:,setdiff((1:q),DSGEModel.MixedConditionalShocks));
IgM = kron(eye(g),M);
%
IgMorth = kron(eye(g),Morth);
%
% speed-up code suggested by Ingvar Strid
%
KpD = K'*DT1*IgM;
KpDDpK = KpD*KpD';
DpKInvKpDDpK = KpD'/KpDDpK;
%
% compute the conditioning relation when the free shocks, state vector, and measurement errors are at mean
%
kbarTg = ZTg-UT-(K'*(Xbar+(Gmat*KsiTTz)));
%
% compute the mean of the shocks when the free shocks, state vector, and measurement errors are at mean
%
mubarTg = DpKInvKpDDpK*kbarTg;
%
% compute the idempotent covariance matrix for the shocks
%
SigmaTg = eye(q_r*g)-(DpKInvKpDDpK*KpD);
SigmaTg = diag(abs(diag(SigmaTg)))+tril(SigmaTg,-1)+tril(SigmaTg,-1)';
[SE,LE] = schur(SigmaTg);
SigmaTgStd = SE*sqrt(abs(LE));
%
% setup projections matrices for the restricted policy rate
%
% each row in ePG is the row vector multiplied by V(t) for tau=0,1,...,DSGEModel.ZLBSampleT
% each element in ePGx is the deterministic term of the projection
%
ePG = zeros(DSGEModel.ZLBSampleT+1,size(Gt,2));
ePGx = zeros(DSGEModel.ZLBSampleT+1,1);
Pmats = zeros(r+1,q+r+2+DSGEModel.ZLBSampleT,max(DSGEModel.ZLBSampleT+1,h));
Mmats = zeros(r+1,h);
if DSGEModel.FixedBeliefsForProj==1;
   ePG(1,:) = e_pr'*Pt(1:r+1,:);
   xbart = zeros(size(Mt,1),1);
   ePGx(1) = e_pr'*Mtilde*X(:,1);
   Mmats(:,1) = Mtilde*X(:,1);
   Pmats(:,:,1) = Pt;
   for tau=1:DSGEModel.ZLBSampleT;
      ePG(tau+1,:) = ePG(tau,:)*Gt;
      xbart = (Mt*X(:,tau))+Gt*xbart;
      ePGx(tau+1) = e_pr'*((Mtilde*X(:,tau+1))+(Pt*xbart));
      Mmats(:,tau+1) = Mtilde*X(:,tau+1);
      Pmats(:,:,tau+1) = Pt;
   end;
   if h>DSGEModel.ZLBSampleT+1;
      for tau=DSGEModel.ZLBSampleT+1:h-1;
         Pmats(:,:,tau+1) = Pt;
         Mmats(:,tau+1) = Mtilde*X(:,tau+1);
      end;
   end;
else;
   %
   % beliefs are now updated, take the values of the
   % solution matrices from period T that have been updated for T+1
   %
   MTiT = LastSolution.MTT;
   FTiT = LastSolution.FTT;
   BTiT = LastSolution.BTT;
   betaTiT = LastSolution.betaTT;
   InvMatT = LastSolution.InvMatT;
   %
   % write down the solution matrices for projections using ZLB format, with Mt, Gt and C1
   %
   [Mt,Gt,C1,Mtilde,Pt] = ZLBProjectionMatrices(MTiT,FTiT,BTiT,InvMatT*DA,MA);
   ePG(1,:) = e_pr'*Pt(1:r+1,:);
   xbart = zeros(size(Mt,1),1);
   ePGx(1) = e_pr'*Mtilde*X(:,1);
   Mmats(:,1) = Mtilde*X(:,1);
   Pmats(:,:,1) = Pt;
   for tau=1:DSGEModel.ZLBSampleT;
      %
      % update betaTiT
      %
      betaTiT = betavec+(LastSolution.M*(betaTiT-betavec));
      %
      % update the solution matrices
      %
      if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
         [MTiT,FTiT,BTiT,InvMatT] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betaTiT);
      else;
         [MTiT,FTiT,BTiT,InvMatT] = feval(GetName(DSGEModel.AdaptiveLearningModelSolverFile),Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betaTiT,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
      end;
      %
      % obtain the new projection matrices
      %
      [Mt,Gt,C1,Mtilde,Pt] = ZLBProjectionMatrices(MTiT,FTiT,BTiT,InvMatT*DA,MA);
      ePG(tau+1,:) = ePG(tau,:)*Gt;
      xbart = (Mt*X(:,tau))+Gt*xbart;
      ePGx(tau+1) = e_pr'*((Mtilde*X(:,tau+1))+(Pt*xbart));
      Mmats(:,tau+1) = Mtilde*X(:,tau+1);
      Pmats(:,:,tau+1) = Pt;
      if (tau>=g)&(tau+2<=h);
         MT1(:,:,tau+2) = MTiT;
         FT1(:,:,tau+2) = FTiT;
         BT1(:,:,tau+2) = BTiT;
         InvMatT1(:,:,tau+2) = InvMatT;
      end;
   end;
   if h>DSGEModel.ZLBSampleT+1;
      for tau=DSGEModel.ZLBSampleT+1:h-1;
         %
         % update betaTiT
         %
         betaTiT = betavec+(LastSolution.M*(betaTiT-betavec));
         %
         % update the solution matrices
         %
         if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
            [MTiT,FTiT,BTiT,InvMatT] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betaTiT);
         else;
            [MTiT,FTiT,BTiT,InvMatT] = feval(GetName(DSGEModel.AdaptiveLearningModelSolverFile),Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betaTiT,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
         end;
         %
         % obtain the new projection matrices
         %
         [Mt,Gt,C1,Mtilde,Pt] = ZLBProjectionMatrices(MTiT,FTiT,BTiT,InvMatT*DA,MA);
         Pmats(:,:,tau+1) = Pt;
         Mmats(:,tau+1) = Mtilde*X(:,tau+1);
         if (tau>=g)&(tau+2<=h);
            MT1(:,:,tau+2) = MTiT;
            FT1(:,:,tau+2) = FTiT;
            BT1(:,:,tau+2) = BTiT;
            InvMatT1(:,:,tau+2) = InvMatT;
         end;
      end;
   end;
end;
%
% compute the mean shock values
%
etaShocks = zeros(q,h);
etaShocks(DSGEModel.MixedConditionalShocks,1:g) = fliplr(reshape(mubarTg,q_r,g));
KsiT = KsiTTz;
PredMean = zeros(n,h);
xiStates = zeros(r+1,h);
K2sum = zeros(qm,1);
ZPos = ~isnan(Z);
if length(size(H))==2;
   if DSGEModel.FixedBeliefsForProj==1;
      %
      % we have fixed belief coefficients
      %
      for i=1:g;
         %
         % update the states
         %
         if k>0;
            KsiT = (MT1*X(:,i))+(FT1*KsiT)+(BT1*etaShocks(:,i));
         else;
            KsiT = (FT1*KsiT)+(BT1*etaShocks(:,i));
         end;
         xiStates(:,i) = KsiT;
         %
         % compute the conditional mean predictions
         %
         if k>0;
            PredMean(:,i) = (A'*X(:,i))+(H'*KsiT);
         else;
            PredMean(:,i) = H'*KsiT;
         end;
         %
         % update the K2sum measure
         %
         if K2length==2;
            K2sum = K2sum+(K2'*PredMean(:,i));
         else;
            K2sum = zeros(qm,1);
            for j=1:i;
               if j<=size(K2,3);
                  K2sum = K2sum+(K2(:,:,j)'*PredMean(:,i+1-j));
               end;
            end;
         end;
      end;
      for i=g+1:h;
         %
         % update the states
         %
         if k>0;
            KsiT = (MT1*X(:,i))+(FT1*KsiT);
         else;
            KsiT = FT1*KsiT;
         end;
         xiStates(:,i) = KsiT;
         %
         % compute the conditional mean predictions
         %
         if k>0;
            PredMean(:,i) = (A'*X(:,i))+(H'*KsiT);
         else;
            PredMean(:,i) = H'*KsiT;
         end;
      end;
   else;
      %
      % belief coefficients are projected forward
      %
      for i=1:g;
         %
         % update the states
         %
         if k>0;
            KsiT = (MT1(:,:,i)*X(:,i))+(FT1(:,:,i)*KsiT)+(BT1(:,:,i)*etaShocks(:,i));
         else;
            KsiT = (FT1(:,:,i)*KsiT)+(BT1(:,:,i)*etaShocks(:,i));
         end;
         xiStates(:,i) = KsiT;
         %
         % compute the conditional mean predictions
         %
         if k>0;
            PredMean(:,i) = (A'*X(:,i))+(H'*KsiT);
         else;
            PredMean(:,i) = H'*KsiT;
         end;
         %
         % update the K2sum measure
         %
         if K2length==2;
            K2sum = K2sum+(K2'*PredMean(:,i));
         else;
            K2sum = zeros(qm,1);
            for j=1:i;
               if j<=size(K2,3);
                  K2sum = K2sum+(K2(:,:,j)'*PredMean(:,i+1-j));
               end;
            end;
         end;
      end;
      for i=g+1:h;
         %
         % update the states
         %
         if k>0;
            KsiT = (MT1(:,:,i)*X(:,i))+(FT1(:,:,i)*KsiT);
         else;
            KsiT = FT1(:,:,i)*KsiT;
         end;
         xiStates(:,i) = KsiT;
         %
         % compute the conditional mean predictions
         %
         if k>0;
            PredMean(:,i) = (A'*X(:,i))+(H'*KsiT);
         else;
            PredMean(:,i) = H'*KsiT;
         end;
      end;
   end;
else;
   if DSGEModel.FixedBeliefsForProj==1;
      %
      % we have fixed belief coefficients
      %
      for i=1:g;
         %
         % update the states
         %
         if k>0;
            KsiT = (MT1*X(:,i))+(FT1*KsiT)+(BT1*etaShocks(:,i));
         else;
            KsiT = (FT1*KsiT)+(BT1*etaShocks(:,i));
         end;
         xiStates(:,i) = KsiT;
         %
         % compute the conditional mean predictions
         %
         if k>0;
            PredMean(:,i) = (A'*X(:,i))+(H(:,:,LastPeriod+i)'*KsiT);
         else;
            PredMean(:,i) = H(:,:,LastPeriod+i)'*KsiT;
         end;
         %
         % update the K2sum measure
         %
         if K2length==2;
            K2sum = K2sum+(K2'*PredMean(:,i));
         else;
            K2sum = zeros(qm,1);
            for j=1:i;
               if j<=size(K2,3);
                  K2sum = K2sum+(K2(:,:,j)'*PredMean(:,i+1-j));
               end;
            end;
         end;
      end;
      for i=g+1:h;
         %
         % update the states
         %
         if k>0;
            KsiT = (MT1*X(:,i))+(FT1*KsiT);
         else;
            KsiT = FT1*KsiT;
         end;
         xiStates(:,i) = KsiT;
         %
         % compute the conditional mean predictions
         %
         if k>0;
            PredMean(:,i) = (A'*X(:,i))+(H(:,:,LastPeriod+i)'*KsiT);
         else;
            PredMean(:,i) = H(:,:,LastPeriod+i)'*KsiT;
         end;
      end;
   else;
      for i=1:g
         %
         % update the states
         %
         if k>0;
            KsiT = (MT1(:,:,i)*X(:,i))+(FT1(:,:,i)*KsiT)+(BT1(:,:,i)*etaShocks(:,i));
         else;
            KsiT = (FT1(:,:,i)*KsiT)+(BT1(:,:,i)*etaShocks(:,i));
         end;
         xiStates(:,i) = KsiT;
         %
         % compute the conditional mean predictions
         %
         if k>0;
            PredMean(:,i) = (A'*X(:,i))+(H(:,:,LastPeriod+i)'*KsiT);
         else;
            PredMean(:,i) = H(:,:,LastPeriod+i)'*KsiT;
         end;
         %
         % update the K2sum measure
         %
         if K2length==2;
            K2sum = K2sum+(K2'*PredMean(:,i));
         else;
            K2sum = zeros(qm,1);
            for j=1:i;
               if j<=size(K2,3);
                  K2sum = K2sum+(K2(:,:,j)'*PredMean(:,i+1-j));
               end;
            end;
         end;
      end;
      for i=g+1:h;
         %
         % update the states
         %
         if k>0;
            KsiT = (MT1(:,:,i)*X(:,i))+(FT1(:,:,i)*KsiT);
         else;
            KsiT = FT1(:,:,i)*KsiT;
         end;
         xiStates(:,i) = KsiT;
         %
         % compute the conditional mean predictions
         %
         if k>0;
            PredMean(:,i) = (A'*X(:,i))+(H(:,:,LastPeriod+i)'*KsiT);
         else;
            PredMean(:,i) = H(:,:,LastPeriod+i)'*KsiT;
         end;
      end;
   end;
end;
%
% compute the mean predictions
%
ZLBYMean = zeros(n,h);
etaMean = etaShocks;
KsiTMean = KsiTTz;
ZLBKsiMean = zeros(r,h);
%
At = zeros(DSGEModel.ZLBSampleT+1,1);
Xt = [etaMean(:,1);KsiTMean];
for t=1:h;
   [At,status,AlgorithmInformation] = ForwardBackShootingAlgorithmAdaptiveLearning(Xt,At,ePG,ePGx,Rzlb(t,:),Rbar,DSGEModel.AIMTolerance);
   if status==1;
      Ksit = Mmats(:,t)+(Pmats(:,:,t)*[Xt;At]);
      ZLBKsiMean(:,t) = Ksit(1:r);
      if length(size(H))==2;
         ZLBYMean(:,t) = Ax(:,t)+(H'*Ksit);
      else;
         ZLBYMean(:,t) = Ax(:,t)+(H(:,:,LastPeriod+t)'*Ksit);
      end;
      if t+1<=h;
         At = MA*At;
         Xt = [etaMean(:,t+1);Ksit];
      end;
   else;
      break;
   end;
end;
%
% check if the mean shocks and state failed to find the anticipated shocks.
%
if status==0;
   if AlgorithmInformation(1)==1;
      AddTxt = 'YADA has recorded an invertibility problem when attempting to solve for the anticipated shocks.';
   elseif AlgorithmInformation(2)==1;
      AddTxt = 'YADA has recorded that a solution cannot be found, i.e., Step 3 (ii) holds (see the YADA Manual).';
   elseif AlgorithmInformation(3)==1;
      AddTxt = 'YADA has recorded that the anticipated shocks are nonnegative, that the restricted policy rate satisfies the zero lower bound, but the complementary slackness condition is violated.';
   else;
      AddTxt = ['YADA has recorded that the maximum number of attempts (' int2str(AlgorithmInformation(5)) ') to find the anticipated shocks was reached.'];
   end;
   txt = ['The forward-back shooting algorithm failed to solve for the anticipated shocks for period ' int2str(t) ' using the population means of the structural shocks and the state variables in the last historical period. ' AddTxt ' Are you sure you want to continue?'];
   answer = Query(txt,'question',300,['Zero Lower Bound Forecasts - ' NameStr],600,'no',CurrINI);
   if strcmp(lower(answer),'no')==1;
      return;
   end;
end;
%
% check if we should annualize or transform
%
ZLBYMeanOrig = ZLBYMean;
if AnnualizeData==1;
   ZLBYMean = Annualize([Yinit ZLBYMean],DSGEModel.annual,DSGEModel.annualscale);
elseif TransData==1;
   YTmp = [Yinitial ZLBYMean];
   [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,YTmp',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
   ZLBYMean = YData(Tt-h+1:Tt,:)';
end;
%
% set up the data matrices
%
if isempty(DSGEModel.X)==0;
   Xdata = DSGEModel.X(:,FirstPeriod+KalmanFirstObservation-1:LastPeriod);
else;
   Xdata = [];
end;
if strcmp(DSGEModel.DataFrequency,'q')==1;
   Yinit = DSGEModel.Y(:,LastPeriod-2:LastPeriod);
elseif strcmp(DSGEModel.DataFrequency,'m')==1;
   Yinit = DSGEModel.Y(:,LastPeriod-10:LastPeriod);
else;
   Yinit = [];
end;
maxYinit = max(max(isnan(Yinit)));
if DSGEModel.RunPredictionEvent==1;
   PredEventData = zeros(n,8);
end;
%
if TransData==1;
   Yinitial = DSGEModel.Y(:,1:DSGEModel.LastPeriod);
   %
   % try to covert: h
   %
   YTmp = [Yinitial DSGEModel.Y(:,DSGEModel.LastPeriod)*ones(1,h)];
   [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,YTmp',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
   Tt = size(YData,1);
   maxYinitial = max(max(isnan(Yinitial)));
else;
   maxYinitial = [];
   Tt = [];
end;
if k==1;
   ObsVarMean = zeros(n,1);
end;
MeanPredictY = zeros(n,h);
%
PriorHeader = '';
if IsPosterior==0;
   PriorHeader = ' For Prior';
end;
%
% mode results to use in case the forward back shooting algorithm runs into a problem with the parameter draaw
%
[S,L] = SchurDecomposition(PTTz);
KsiTTzmode = KsiTTz;
Smode = S;
Lmode = L;
ePGmode = ePG;
ePGxmode = ePGx;
Mmatsmode = Mmats;
Pmatsmode = Pmats;
Rzlbmode = Rzlb;
Rbarmode = Rbar;
Axmode = Ax;
Amode = A;
Hmode = H;
if rank(R)>0;
   R(:,DSGEModel.ZLBPosition) = 0;
   R(DSGEModel.ZLBPosition,:) = 0;
   [Rm,Lm] = SchurDecomposition(R);
   Rmeasure = Rm*sqrt(Lm);
   NumMeasure = size(Lm,1);
else;
   Rmeasure = [];
   NumMeasure = 0;
end;
Rmeasuremode = Rmeasure;
NumMeasuremode = NumMeasure;
MT1mode = MT1;
FT1mode = FT1;
BT1mode = BT1;
MAmode = MA;
Xbarmode = Xbar;
Gmatmode = Gmat;
DpKInvKpDDpKmode = DpKInvKpDDpK;
DT1mode = DT1;
SigmaTgStdmode = SigmaTgStd;
%
% check if we should use parallel computations or not
%
if CurrINI.NumWorkers<2;
   %
   % check if we should setup a wait handle
   %
   if DSGEModel.ShowProgress==1;
      ProgressStructure.title = ['Progress for ' num2str(NumDraws*NumPaths,'%0.0f') ' Prediction Paths'];
      ProgressStructure.facecolor = CurrINI.progress_facecolor;
      ProgressStructure.startfacecolor = CurrINI.progress_startfacecolor;
      ProgressStructure.edgecolor = CurrINI.progress_edgecolor;
      ProgressStructure.bgcolor = CurrINI.progress_bgcolor;
      ProgressStructure.stop = 0;
      ProgressStructure.clock = DSGEModel.ShowProgressClock;
      ProgressStructure.label = 'Path Number:';
      %
      ProgressStructure.name = ['ZLB Cond. Predictive Distributions' PriorHeader];
      ProgressStructure.CurrINI = CurrINI;
      WaitHandle = ProgressDLG(0,ProgressStructure);
      set(WaitHandle,'Color',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'));
      drawnow;
   else;
      txt = ['Please wait while YADA computes predictive distributions under adaptive learning and subject to the ZLB for the ' lower(TypeStr) ' data. Computations started at: ' StringTrim(datestr(now,14))];
      WaitHandle = WaitDLG(txt,'information',['ZLB Predictive Distributions ' PriorHeader ' - ' TypeStr ' Data'],600,150,CurrINI,0);
      WaitControls = get(WaitHandle,'UserData');
   end;
   %
   % begin the loop
   %
   abort = '';
   StopEstimation = 0;
   if length(size(H))==2;
      for ParamIndex=1:NumDraws;
         %
         % Solve the model for the current parameter vector
         %
         theta = thetaPostSample(ParamIndex,:)';
         [A,H,R,F,B0,status,ErrMsg,ModelParameters] = SolveDSGEModel(DSGEModel,theta,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
         if status~=1;
            %
            % use posterior mode or initial values as a backup
            %
            [A,H,R,F,B0,status,ErrMsg,ModelParameters] = SolveDSGEModel(DSGEModel,thetaMode,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
         end;
         %
         % check if we can use these values.
         %
         etaMat = K1'*H'*B0*M;
         if rank(etaMat)<qm;
            %
            % solve the mode for initial parameter values or for the posterior mode values
            %
            [A,H,R,F,B0,status,ErrMsg,ModelParameters] = SolveDSGEModel(DSGEModel,thetaMode,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
         end;
         %
         % Try to solve the model subject to the anticipated shocks
         %
         [Gtmp,C1tmp,Ptmp,mcode] = ZLBKleinSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.ZLBSampleT,DSGEModel.RtildePosition,DSGEModel.REqPosition,DSGEModel.ZLBRuleType,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
         if mcode~=1;
            %
            % solve the mode for initial parameter values or for the posterior mode values
            %
            [A,H,R,F,B0,status,ErrMsg,ModelParameters] = SolveDSGEModel(DSGEModel,thetaMode,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
         end;
         %
         % get the structural form matrices
         %
         [Hlag,H0,Hexp,D] = DSGEStructuralForm(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions);
         %
         % reshuffle the equations to separate the restricted from the unrestricted policy rate
         %
         [Hlag,H0,Hexp,D,DA,MA] = ZLBDSGEStructuralForm(Hlag,H0,Hexp,D,DSGEModel.ZLBSampleT,DSGEModel.RtildePosition,DSGEModel.REqPosition,DSGEModel.ZLBRuleType);
         %
         % solve the model with the new matrices
         %
         [F,B0,BA,mcode] = SolveExtendedDSGEModel(Hlag,H0,Hexp,D,DA,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
         if mcode~=1;
            %
            % use the posterior mode estimates of the parameters
            %
            [A,H,R,F,B0,status,ErrMsg,ModelParameters] = SolveDSGEModel(DSGEModel,thetaMode,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
            [Hlag,H0,Hexp,D] = DSGEStructuralForm(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions);
            [Hlag,H0,Hexp,D,DA,MA] = ZLBDSGEStructuralForm(Hlag,H0,Hexp,D,DSGEModel.ZLBSampleT,DSGEModel.RtildePosition,DSGEModel.REqPosition,DSGEModel.ZLBRuleType);
            [F,B0,BA,mcode] = SolveExtendedDSGEModel(Hlag,H0,Hexp,D,DA,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
         end;
         %
         % Compute the initial values of betavec, Sigmau, R10 and Sigmaeps
         %
         [betavec,Sigmau,R10,Sigmaeps,Gtilde,Kgf] = AdaptiveLearningInitialValuesDSGE(F,B0,ModelParameters.sigmarAL,ModelParameters.sigmaeAL,SMatrix,G,m,x,f,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,Gtilde,Kgf);
         %
         % call the transformation function
         %
         [TransStatus,Hstarlag,Hstar0,Hstar1f] = AdaptiveLearningTransformDSGEModel(Hlag,H0,Hexp,SMatrix,Sperp);
         %
         % compute initial values for the solution matrices
         %
         if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
            [Mt1,Ft1,Bt1,InvH0H1star] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betavec);
         else;
            [Mt1,Ft1,Bt1,InvH0H1star] = feval(GetName(DSGEModel.AdaptiveLearningModelSolverFile),Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betavec,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
         end;
         %
         % compute the solution matrix BAt1 on the anticipated shocks
         %
         BAt1 = InvH0H1star*DA;
         %
         % make sure that we take into account that Rbar depends on the parameters
         %
         if k>0;
            Rbar = A(:,DSGEModel.ZLBPosition)'*X;
         end;
         Rbar = Rbar(:,1:DSGEModel.ZLBSampleT+1);
         %
         % Transform the steady-state interest rate and the lower bound to the same units
         % as the unrestricted monetary policy rate
         %
         Hvalue = H(DSGEModel.RtildePosition,DSGEModel.ZLBPosition);
         Rzlb = (1/Hvalue)*RzlbMat;
         Rbar = (1/Hvalue)*Rbar;
         %
         % we need to fix the H-matrix when using the extended form of the model
         % by adding zeros to 
         %
         HOld = H;
         %
         % we need to fix the H-matrix when using the extended form of the model
         % by adding zeros to 
         %
         H = [H;zeros(1,n)];
         %
         % run the Kalman filter
         %
         if IsALUpdate==1;
            if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
               [lnL,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningKalmanFilter(m,SMatrix,betavec,Sigmau,R10,Sigmaeps,ModelParameters.rhoAL,Hstar0,Hstarlag,Hstar1f,D,Mt1,Ft1,Bt1,Gtilde,Kgf,DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,KalmanFirstObservation,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs);
            else;
               [lnL,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningUserSolverKalmanFilter(GetName(DSGEModel.AdaptiveLearningModelSolverFile),m,SMatrix,betavec,Sigmau,R10,Sigmaeps,ModelParameters.rhoAL,Hstar0,Hstarlag,Hstar1f,D,Mt1,Ft1,Bt1,Gtilde,Kgf,DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,KalmanFirstObservation,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
            end;
         else;
            [lnL,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningSingleKalmanFilter(m,betavec,Mt1,Ft1,Bt1,DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,KalmanFirstObservation,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs);
            LastSolution.InvMatT1 = InvH0H1star;
            LastSolution.InvMatT = InvH0H1star;
         end;
         %
         % check if the user wants to take the conditioning infortmation into account
         % when determining the mean and covariance matrix for the states at T.
         %
         if (DSGEModel.KsiUseCondData==1)&(SCond==1);
            if (k>0);
               XPred = X(:,1:g);
            else;
               XPred = [];
            end;
            [KsiTTz,PTTz] = AdaptiveLearningCondPredictionSmoother(Ksitt1(:,T),Ptt1(:,:,T),Yhat(:,T),DSGEModel.Y(:,LastPeriod),XPred,Z,K1,A,H,R,Mtilde(:,:,T),Ftilde(:,:,T),Btilde(:,:,T),betavec,LastSolution.betaTT,LastSolution.M,Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,DSGEModel);
%            KsiTTz = Ksitt(:,T);
%            PTTz = Ptt(:,:,T);
         elseif (DSGEModel.KsiUseCondData==1)&(T>=g);
            %
            % we need to run the Kalman smoother
            %
            [KsitT,PtT,etatt,etatT,wtt,wtT] = AdaptiveLearningKalmanSmoother(Ksitt1,Ptt1,Yerror,InvMSEY,H,R,Ftilde,Btilde,FirstPeriod,LastPeriod,KalmanFirstObservation,DSGEModel.YNaNs);
            PtT = PtT(:,:,T-g+1:T);
            KsitT = KsitT(:,T-g+1:T);
            if isempty(wtT)==1;
               wtT = zeros(n,g);
            else;
               wtT = wtT(:,T-g+1:T);
            end;
            if k>0;
               if g>1;
                  Xtra = DSGEModel.X(:,LastPeriod-g+2:LastPeriod);
               else;
                  Xtra = [];
               end;
               [KsiTTz,PTTz] = AdaptiveLearningCondPredictionKalmanSmoother(KsitT,PtT,wtT,Z,U,[Xtra X(:,1:g)],K1,K2,A,H,R,Mtilde(:,:,T),Ftilde(:,:,T),Btilde(:,:,T),betavec,LastSolution.betaTT,LastSolution.M,Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,DSGEModel);
            else;
               [KsiTTz,PTTz] = AdaptiveLearningCondPredictionKalmanSmoother(KsitT,PtT,wtT,Z,U,[],K1,K2,A,H,R,Mtilde(:,:,T),Ftilde(:,:,T),Btilde(:,:,T),betavec,LastSolution.betaTT,LastSolution.M,Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,DSGEModel);
            end;
         else;
            KsiTTz = Ksitt(:,T);
            PTTz = Ptt(:,:,T);
         end;
         PTTz = diag(abs(diag(PTTz)))+tril(PTTz,-1)+tril(PTTz,-1)';
         %
         % use the Schur decomposition for the reduced rank matrix PTTz instead of
         % the Choleski decomposition
         %
         [S,L] = SchurDecomposition(PTTz);
         KsiTT = KsiTTz;
         %
         % deal with possible missing observations in the historical sample if we
         % should annualize or transform the predictions
         %
         if AnnualizeData==1;
            if maxYinit==1;
               %
               % take into account that there are missing obs for the
               % historical data which can affect annualization
               %
               YSmooth = DSGEModel.Y(:,FirstPeriod+KalmanFirstObservation-1:LastPeriod);
               nanpos = isnan(YSmooth);
               %
               % compute the Kalman smoother
               %
               [KsitT,PtT,etatt,etatT,wtt,wtT] = AdaptiveLearningKalmanSmoother(Ksitt1,Ptt1,Yerror,InvMSEY,H,R,Ftilde,Btilde,FirstPeriod,LastPeriod,KalmanFirstObservation,DSGEModel.YNaNs);
               %
               if k>0;
                  YHistPred = (A'*Xdata)+(H'*KsitT);
               else;
                  YHistPred = H'*KsitT;
               end;
               YSmooth(nanpos) = YHistPred(nanpos);
               if FirstPeriod+KalmanFirstObservation-1>1;
                  YSmooth = [DSGEModel.Y(:,1:FirstPeriod+KalmanFirstObservation-2) YSmooth];
               end;
               if strcmp(DSGEModel.DataFrequency,'q')==1;
                  Yinit = YSmooth(:,LastPeriod-2:LastPeriod);
               elseif strcmp(DSGEModel.DataFrequency,'m')==1;
                  Yinit = YSmooth(:,LastPeriod-10:LastPeriod);
               else;
                  Yinit = [];
               end;
            end;
         elseif TransData==1;
            if maxYinitial==1;
               %
               % take into account that there are missing obs for the
               % historical data which can affect general transformations
               %
               YSmooth = DSGEModel.Y(:,FirstPeriod+KalmanFirstObservation-1:LastPeriod);
               nanpos = isnan(YSmooth);
               %
               % compute the Kalman smoother
               %
               [KsitT,PtT,etatt,etatT,wtt,wtT] = AdaptiveLearningKalmanSmoother(Ksitt1,Ptt1,Yerror,InvMSEY,H,R,Ftilde,Btilde,FirstPeriod,LastPeriod,KalmanFirstObservation,DSGEModel.YNaNs);
               %
               if k>0;
                  YHistPred = (A'*Xdata)+(H'*KsitT);
               else;
                  YHistPred = H'*KsitT;
               end;
               YSmooth(nanpos) = YHistPred(nanpos);
               if FirstPeriod+KalmanFirstObservation-1>1;
                  YSmooth = [DSGEModel.Y(:,1:FirstPeriod+KalmanFirstObservation-2) YSmooth];
               end;
               Yinitial = YSmooth(:,1:LastPeriod);
            end;
         end;
         %
         % store solution matrices
         %
         if DSGEModel.FixedBeliefsForProj==1;
            MT1 = Mtilde(:,:,T);
            FT1 = Ftilde(:,:,T);
            BT1 = Btilde(:,:,T);
         else;
            MT1 = zeros(r+1,k,h);
            FT1 = zeros(r+1,r,h);
            BT1 = zeros(r+1,q,h);
            MT1(:,:,1) = Mtilde(:,:,T);
            FT1(:,:,1) = Ftilde(:,:,T);
            BT1(:,:,1) = Btilde(:,:,T);
            InvMatT1 = zeros(r+1,r+1,h);
            InvMatT1(:,:,1) = LastSolution.InvMatT1;
         end;
         %
         % write down the solution matrices for projections using ZLB format, with Mt, Gt and C1
         %
         [Mt,Gt,C1,Mtilde,Pt] = ZLBProjectionMatrices(Mtilde(:,:,T),Ftilde(:,:,T),Btilde(:,:,T),LastSolution.InvMatT1*DA,MA);
         %
         % deal with deterministic variables
         %
         if k>0;
            Ax = A'*X;
         else;
            Ax = zeros(n,h);
            X = zeros(r,max(DSGEModel.ZLBSampleT+1,h))
         end;
         %
         % setup projections matrices for the restricted policy rate
         %
         % each row in ePG is the row vector multiplied by V(t) for tau=0,1,...,DSGEModel.ZLBSampleT
         % each element in ePGx is the deterministic term of the projection
         %
         ePG = zeros(DSGEModel.ZLBSampleT+1,size(Gt,2));
         ePGx = zeros(DSGEModel.ZLBSampleT+1,1);
         Pmats = zeros(r+1,q+r+2+DSGEModel.ZLBSampleT,max(DSGEModel.ZLBSampleT+1,h));
         Mmats = zeros(r+1,h);
         if DSGEModel.FixedBeliefsForProj==1;
            ePG(1,:) = e_pr'*Pt(1:r+1,:);
            xbart = zeros(size(Mt,1),1);
            ePGx(1) = e_pr'*Mtilde*X(:,1);
            Mmats(:,1) = Mtilde*X(:,1);
            Pmats(:,:,1) = Pt;
            for tau=1:DSGEModel.ZLBSampleT;
               ePG(tau+1,:) = ePG(tau,:)*Gt;
               xbart = (Mt*X(:,tau))+Gt*xbart;
               ePGx(tau+1) = e_pr'*((Mtilde*X(:,tau+1))+(Pt*xbart));
               Mmats(:,tau+1) = Mtilde*X(:,tau+1);
               Pmats(:,:,tau+1) = Pt;
            end;
            if h>DSGEModel.ZLBSampleT+1;
               for tau=DSGEModel.ZLBSampleT+1:h-1;
                  Pmats(:,:,tau+1) = Pt;
                  Mmats(:,tau+1) = Mtilde*X(:,tau+1);
               end;
            end;
         else;
            %
            % beliefs are now updated, take the values of the
            % solution matrices from period T that have been updated for T+1
            %
            MTiT = LastSolution.MTT;
            FTiT = LastSolution.FTT;
            BTiT = LastSolution.BTT;
            betaTiT = LastSolution.betaTT;
            InvMatT = LastSolution.InvMatT;
            MT1(:,:,2) = LastSolution.MTT;
            FT1(:,:,2) = LastSolution.FTT;
            BT1(:,:,2) = LastSolution.BTT;
            %
            % write down the solution matrices for projections using ZLB format, with Mt, Gt and C1
            %
            [Mt,Gt,C1,Mtilde,Pt] = ZLBProjectionMatrices(MTiT,FTiT,BTiT,InvMatT*DA,MA);
            ePG(1,:) = e_pr'*Pt(1:r+1,:);
            xbart = zeros(size(Mt,1),1);
            ePGx(1) = e_pr'*Mtilde*X(:,1);
            Mmats(:,1) = Mtilde*X(:,1);
            Pmats(:,:,1) = Pt;
            for tau=1:DSGEModel.ZLBSampleT;
               %
               % update betaTiT
               %
               betaTiT = betavec+(LastSolution.M*(betaTiT-betavec));
               %
               % update the solution matrices
               %
               if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
                  [MTiT,FTiT,BTiT,InvMatT] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betaTiT);
               else;
                  [MTiT,FTiT,BTiT,InvMatT] = feval(GetName(DSGEModel.AdaptiveLearningModelSolverFile),Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betaTiT,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
               end;
               %
               % obtain the new projection matrices
               %
               [Mt,Gt,C1,Mtilde,Pt] = ZLBProjectionMatrices(MTiT,FTiT,BTiT,InvMatT*DA,MA);
               ePG(tau+1,:) = ePG(tau,:)*Gt;
               xbart = (Mt*X(:,tau))+Gt*xbart;
               ePGx(tau+1) = e_pr'*((Mtilde*X(:,tau+1))+(Pt*xbart));
               Mmats(:,tau+1) = Mtilde*X(:,tau+1);
               Pmats(:,:,tau+1) = Pt;
               if tau+2<=h;
                  MT1(:,:,tau+2) = MTiT;
                  FT1(:,:,tau+2) = FTiT;
                  BT1(:,:,tau+2) = BTiT;
                  InvMatT1(:,:,tau+2) = InvMatT;
               end;
            end;
            if h>DSGEModel.ZLBSampleT+1;
               for tau=DSGEModel.ZLBSampleT+1:h-1;
                  %
                  % update betaTiT
                  %
                  betaTiT = betavec+(LastSolution.M*(betaTiT-betavec));
                  %
                  % update the solution matrices
                  %
                  if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
                     [MTiT,FTiT,BTiT,InvMatT] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betaTiT);
                  else;
                     [MTiT,FTiT,BTiT,InvMatT] = feval(GetName(DSGEModel.AdaptiveLearningModelSolverFile),Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betaTiT,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
                  end;
                  %
                  % obtain the new projection matrices
                  %
                  [Mt,Gt,C1,Mtilde,Pt] = ZLBProjectionMatrices(MTiT,FTiT,BTiT,InvMatT*DA,MA);
                  Pmats(:,:,tau+1) = Pt;
                  Mmats(:,tau+1) = Mtilde*X(:,tau+1);
                  if tau+2<=h;
                     MT1(:,:,tau+2) = MTiT;
                     FT1(:,:,tau+2) = FTiT;
                     BT1(:,:,tau+2) = BTiT;
                     InvMatT1(:,:,tau+2) = InvMatT;
                  end;
               end;
            end;
         end;
         %
         % stack the model to determine the shock distribution
         %
         Xbar = zeros(n*g,1);
         XTi = zeros(r+1,1);
         Gmat = zeros(n*g,r+1);
         DT1 = zeros(n*g,q*g);
         Fexp = eye(r+1);
         if DSGEModel.FixedBeliefsForProj==1;
            for i=1:g;
               Fexp = FT1*Fexp;
               Ind = (1+((g-i)*n):(g-i+1)*n);
               Ind1 = (1+((g-i)*q):(g-i+1)*q);
               Ind2 = (1+((g-i)*qm):(g-i+1)*qm);
               if k>0;
                  XTi = (MT1*X(:,i))+(FT1*XTi);
                  Xbar(Ind,1) = (A'*X(:,i))+(H'*XTi);
               end;
               Gmat(Ind,:) = H'*Fexp;
               DT1(Ind,Ind1) = H'*BT1;
               Fi = FT1;
               for j=1:i-1;
                  Ind1 = Ind1+q;
                  Ind3 = Ind+(j*n);
                  DT1(Ind,Ind1) = H'*Fi*BT1;
                  Fi = Fi*FT1;
               end;
            end;
         else;
            %
            % we have updated beliefs
            %
            for i=1:g;
               Fexp = FT1(:,:,i)*Fexp;
               Ind = (1+((g-i)*n):(g-i+1)*n);
               Ind1 = (1+((g-i)*q):(g-i+1)*q);
               Ind2 = (1+((g-i)*qm):(g-i+1)*qm);
               if k>0;
                  XTi = (MT1(:,:,i)*X(:,i))+(FT1(:,:,i)*XTi);
                  Xbar(Ind,1) = (A'*X(:,i))+(H'*XTi);
               end;
               Gmat(Ind,:) = H'*Fexp;
               DT1(Ind,Ind1) = H'*BT1(:,:,i);
               Fi = FT1(:,:,i);
               for j=1:i-1;
                  Ind1 = Ind1+q;
                  Ind3 = Ind+(j*n);
                  DT1(Ind,Ind1) = H'*Fi*BT1(:,:,i);
                  Fi = Fi*FT1(:,:,i-j);
               end;
            end;
         end;
         %
         % speed-up code suggested by Ingvar Strid
         %
         KpD = K'*DT1*IgM;
         KpDDpK = KpD*KpD';
         DpKInvKpDDpK = KpD'/KpDDpK;
         %
         % compute the idempotent covariance matrix for the shocks
         %
         SigmaTg = eye(q_r*g)-(DpKInvKpDDpK*KpD);
         SigmaTg = diag(abs(diag(SigmaTg)))+tril(SigmaTg,-1)+tril(SigmaTg,-1)';
         [SE,LE] = schur(SigmaTg);
         SigmaTgStd = SE*sqrt(abs(LE));
         %
         % check if we have measurement errors and if so compute a Schur decomposition of the R matrix
         %
         if rank(R)>0;
            R(:,DSGEModel.ZLBPosition) = 0;
            R(DSGEModel.ZLBPosition,:) = 0;
            [Rm,Lm] = SchurDecomposition(R);
            Rmeasure = Rm*sqrt(Lm);
            NumMeasure = size(Lm,1);
         else;
            Rmeasure = [];
            wTh = zeros(n,h);
            NumMeasure = 0;
         end;
         %
         % check if we should compute the steady-state mean
         % of the observed variables
         %
         if k==1;
            ObsVarMeanTmp = (1/NumDraws)*A';
         end;
         %
         % we are now ready to compute possible paths at theta
         %
         PredPaths = zeros(n,h,NumPaths);
         AtMean = zeros(DSGEModel.ZLBSampleT+1,h);
         XtMean = zeros(q+r+1,h);
         ShockMatrix = zeros(q,h);
         NumZLBTimes = zeros(1,h);
         NumBadValues = 0;
         NumBindingValues = 0;
         %
         for it=1:NumPaths;
            CurrPath = it+(NumPaths*(ParamIndex-1));
            %
            % check if we should update the progress dialog
            %
            if DSGEModel.ShowProgress==1;
               abort = get(WaitHandle,'UserData');
               if strcmp(abort,'cancel')==1;
                  StopEstimation = 1;
                  break;
               else;
                  ProgressDLG([CurrPath/(NumDraws*NumPaths) CurrPath]);
               end;
            end;
            %
            % set up a loop to allow for new draws of the state vector
            % and if that fails to use the thetaMode parameter values
            %
            RunLoop = 1;
            TotalRunLoop = 1;
            while RunLoop==1;
               %
               % Draw a sequence of measurement errors
               %
               if isempty(Rmeasure)==0;
                  wTh = Rmeasure*randn(NumMeasure,h);
                  wTh(DSGEModel.ZLBPosition,:) = 0;
               else;
                  wTh = zeros(n,h);
               end;
               WTg = vec(fliplr(wTh(:,1:g)));
               %
               % Draw a state vector for time T using the N(KsiTTz,PTTz) distribution.
               %
               KsiT = KsiTT+(S*sqrt(L)*randn(size(L,2),1));
               %
               % make sure that the initial values of the unconstrained and constrained policy rate are identical
               %
               KsiT(r+1) = KsiT(DSGEModel.RtildePosition);
               %
               etaTh = zeros(q,h);
               etaTh(:,1:g) = randn(q,g);
               %
               % retrieve the q-q_r shocks eta(t,q-q_r)
               %
               NTgFree = vec(fliplr(Morth'*etaTh(:,1:g)));
               %
               % setut the mean for the shock distribution
               %
               kTg = ZTg-UT-(K'*(Xbar+(Gmat*KsiT)+WTg+(DT1*IgMorth*NTgFree)));
               muTg = DpKInvKpDDpK*kTg;
               PredictY = zeros(n,h);
               At = zeros(DSGEModel.ZLBSampleT+1,1);
               Xt = [zeros(q,1);KsiT];
               ZLBKsiPath = zeros(r,h);
               AtPath = zeros(DSGEModel.ZLBSampleT+1,h);
               XtPath = zeros(q+r+1,h);
               %
               % draw economic shocks from N(muTg,SigmaTg) for period 1 to g
               % and from N(0,I) for period g+1 to h.
               %
               ShockDraw = randn(q_r*g,1);
               etaTg = muTg+(SigmaTgStd*ShockDraw);
               %
               % overlay with restricted shocks
               %
               etaTh(DSGEModel.MixedConditionalShocks,1:g) = fliplr(reshape(etaTg,q_r,g));
               %
               gGood = 0;
               t = 1;
               K2sum = zeros(qm,1);
               BadDraw = 0;
               while t<=g;
                  ZPosi = ZPos(:,t);
                  m1 = sum(ZPosi);
                  %
                  % add shocks to Xt
                  %
                  Xt(1:q) = etaTh(:,t);
                  [AtNew,status,AlgorithmInformation] = ForwardBackShootingAlgorithmAdaptiveLearning(Xt,At,ePG,ePGx,Rzlb(t,:),Rbar,DSGEModel.AIMTolerance);
                  if status==1;
                     %
                     % check if we have non-zero anticipated shocks
                     %
                     Ksit = Mmats(:,t)+(Pmats(:,:,t)*[Xt;AtNew]);
                     if (sum(AtNew)>0)&(m1>0);
                        NumBindingValues = NumBindingValues+1;
                        %
                        % check the constraint
                        %
                        if k>0;
                           CA = Z(ZPosi,t) - (K1(:,ZPosi)'*A'*X(:,t)) - (K1(:,ZPosi)'*wTh(:,t)) - (K1(:,ZPosi)'*H'*Ksit) - K2sum(ZPosi) -U(ZPosi,g);
                        else;
                           CA = Z(ZPosi,t) - (K1(:,ZPosi)'*wTh(:,t)) - (K1(:,ZPosi)'*H'*Ksit) - K2sum(ZPosi) -U(ZPosi,g);
                        end;
                        if abs(CA)>=1.0e-06;
                           NumBadValues = NumBadValues+1;
                           status = 0;
                        end;
                     end;
                     if status==1;
                        At = AtNew;
                        AtPath(:,t) = At;
                        XtPath(:,t) = Xt;
                        if max(At)>0;
                           %
                           % add the shocks when ZLB is binding and count how many times for given horizon
                           %
                           ShockMatrix(:,t) = ShockMatrix(:,t)+Xt(1:q,1);
                           NumZLBTimes(1,t) = NumZLBTimes(1,t)+1;
                        end;
                        ZLBKsiPath(:,t) = Ksit(1:r);
                        KsiT = Ksit;
                        PredictY(:,t) = Ax(:,t)+(H'*Ksit)+wTh(:,t);
                        %
                        % test if the prediction is "reasonable"
                        %
                        TestValue = (PredictY(:,t)-ZLBYMeanOrig(:,t))./ZLBYMeanOrig(:,t);
                        if max(abs(TestValue))<100;
                           if t+1<=h;
                              At = MA*At;
                              Xt = [zeros(q,1);Ksit];
                           end;
                           %
                           % update the sum of K2 times lagged values of the observed variables
                           %
                           if K2length==2;
                              K2sum = K2sum+(K2'*PredictY(:,t));
                           else;
                              K2sum = zeros(qm,1);
                              for j=1:t;
                                 if j<=size(K2,3);
                                    K2sum = K2sum+(K2(:,:,j)'*PredictY(:,t+1-j));
                                 end;
                              end;
                           end;
                           t = t+1;
                           if t==g+1;
                              gGood = 1;
                           end;
                        else;
                           t = g+1;
                        end;
                     else;
                        t = g+1;
                     end;
                  else;
                     BadDraw = BadDraw+1;
                     if BadDraw>=100;
                        t = g+1;
                     end;
                  end;
               end;
               %
               % check if we should continue
               %
               if gGood==1;
                  if t==h+1;
                     RunLoop = 0;
                  end;
                  while t<=h;
                     Xt(1:q) = randn(q,1);
                     [AtNew,status,AlgorithmInformation] = ForwardBackShootingAlgorithmAdaptiveLearning(Xt,At,ePG,ePGx,Rzlb(t,:),Rbar,DSGEModel.AIMTolerance);
                     if status==1;
                        At = AtNew;
                        AtPath(:,t) = At;
                        XtPath(:,t) = Xt;
                        if max(At)>0;
                           %
                           % add the shocks when ZLB is binding and count how many times for given horizon
                           %
                           ShockMatrix(:,t) = ShockMatrix(:,t)+Xt(1:q,1);
                           NumZLBTimes(1,t) = NumZLBTimes(1,t)+1;
                        end;
                        Ksit = Mmats(:,t)+(Pmats(:,:,t)*[Xt;At]);
                        ZLBKsiPath(:,t) = Ksit(1:r);
                        %
                        PredictY(:,t) = Ax(:,t)+(H'*Ksit)+wTh(:,t);
                        %
                        % test if the prediction is "reasonable"
                        %
                        TestValue = (PredictY(:,t)-ZLBYMeanOrig(:,t))./ZLBYMeanOrig(:,t);
                        if max(abs(TestValue))<100;
                           if t+1<=h;
                              At = MA*At;
                              Xt = [zeros(q,1);Ksit];
                           end;
                           t = t+1;
                           if t==h+1;
                              RunLoop = 0;
                           end;
                        else;
                           t = h+1;
                        end;
                     end;
                  end;
               end;
               TotalRunLoop = TotalRunLoop+1;
               if TotalRunLoop>100;
                  %
                  % use the original parameters instead of the current vector as it doesn't work
                  %
                  KsiTT = KsiTTzmode;
                  S = Smode;
                  L = Lmode;
                  ePG = ePGmode;
                  ePGx = ePGxmode;
                  Mmats = Mmatsmode;
                  Pmats = Pmatsmode;
                  Rzlb = Rzlbmode;
                  Rbar = Rbarmode;
                  Ax = Axmode;
                  A = Amode;
                  H = Hmode;
                  Rmeasure = Rmeasuremode;
                  NumMeasure = NumMeasuremode;
                  MT1 = MT1mode;
                  FT1 = FT1mode;
                  BT1 = BT1mode;
                  MA = MAmode;
                  Xbar = Xbarmode;
                  Gmat = Gmatmode;
                  DpKInvKpDDpK = DpKInvKpDDpKmode;
                  DT1 = DT1mode;
                  SigmaTgStd = SigmaTgStdmode;
                  TotalRunLoop = 1;
               end;
            end;
            %
            % check if we should annualize or transform
            %
            if AnnualizeData==1;
               PredictY = Annualize([Yinit PredictY],DSGEModel.annual,DSGEModel.annualscale);
            elseif TransData==1;
               YTmp = [Yinitial PredictY];
               [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,YTmp',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
               PredictY = YData(Tt-h+1:Tt,:)';
            end;
            %
            % update the mean predictions for all paths and parameter draws
            %
            MeanPredictY = MeanPredictY+((Weights(ParamIndex)/(NumPaths*NumDraws))*PredictY);
            %
            % add the projected path to the variable to be saved
            %
            PredPaths(:,:,it) = PredictY;
            drawnow;
         end;
         %
         if StopEstimation==1;
            break;
         end;
         %
         % save the calculations to disk
         %
         if IsPosterior==1;
            if AnnualizeData==0;
               if TransData==0;
                  file = FixFilePath([DSGEModel.OutputDirectory '\predictions\AdaptiveLearningDSGE-' int2str(DSGEModel.FixedBeliefsForProj) '-ZLB-CondPrediction-Original-0-Subset-' DSGEModel.NameOfModel '-' num2str(CurrChain,'%0.0f') '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '.' int2str(DSGEModel.ZLBRuleType) '-' num2str(h,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPaths,'%0.0f') '.mat']);
               else;
                  file = FixFilePath([DSGEModel.OutputDirectory '\predictions\AdaptiveLearningDSGE-' int2str(DSGEModel.FixedBeliefsForProj) '-ZLB-CondPrediction-Transformed-0-Subset-' DSGEModel.NameOfModel '-' num2str(CurrChain,'%0.0f') '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '.' int2str(DSGEModel.ZLBRuleType) '-' num2str(h,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPaths,'%0.0f') '.mat']);
               end;
            else;
               file = FixFilePath([DSGEModel.OutputDirectory '\predictions\AdaptiveLearningDSGE-' int2str(DSGEModel.FixedBeliefsForProj) '-ZLB-CondPrediction-Annual-0-Subset-' DSGEModel.NameOfModel '-' num2str(CurrChain,'%0.0f') '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '.' int2str(DSGEModel.ZLBRuleType) '-' num2str(h,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPaths,'%0.0f') '.mat']);
            end;
         else;
            if AnnualizeData==0;
               if TransData==0;
                  file = FixFilePath([DSGEModel.OutputDirectory '\predictions\AdaptiveLearningDSGE-' int2str(DSGEModel.FixedBeliefsForProj) '-ZLB-CondPrediction-Original-Prior-0-Subset-' DSGEModel.NameOfModel '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '.' int2str(DSGEModel.ZLBRuleType) '-' num2str(h,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPaths,'%0.0f') '.mat']);
               else;
                  file = FixFilePath([DSGEModel.OutputDirectory '\predictions\AdaptiveLearningDSGE-' int2str(DSGEModel.FixedBeliefsForProj) '-ZLB-CondPrediction-Transformed-Prior-0-Subset-' DSGEModel.NameOfModel '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '.' int2str(DSGEModel.ZLBRuleType) '-' num2str(h,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPaths,'%0.0f') '.mat']);
               end;
            else;
               file = FixFilePath([DSGEModel.OutputDirectory '\predictions\AdaptiveLearningDSGE-' int2str(DSGEModel.FixedBeliefsForProj) '-ZLB-CondPrediction-Annual-Prior-0-Subset-' DSGEModel.NameOfModel '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '.' int2str(DSGEModel.ZLBRuleType) '-' num2str(h,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPaths,'%0.0f') '.mat']);
            end;
         end;
         if k==1;
            ObsVarMean = ObsVarMean+ObsVarMeanTmp;
         end;
         save(file,'PredPaths','ShockMatrix','NumZLBTimes');
         if DSGEModel.RunPredictionEvent==1;
            %
            % compute the prediction events results
            %
            PredEventData = PredEventData+CalculatePredictionEvents(PredPaths,DSGEModel.PredictionEvent,Weights(ParamIndex));
         end;
      end;
   else;
      %
      % we have a time-varying H-matrix
      %
      for ParamIndex=1:NumDraws;
         %
         % Solve the model for the current parameter vector
         %
         theta = thetaPostSample(ParamIndex,:)';
         [A,H,R,F,B0,status,ErrMsg,ModelParameters] = SolveDSGEModel(DSGEModel,theta,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
         if status~=1;
            %
            % use posterior mode or initial values as a backup
            %
            [A,H,R,F,B0,status,ErrMsg,ModelParameters] = SolveDSGEModel(DSGEModel,thetaMode,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
         end;
         %
         % check if we can use these values.
         %
         etaMat = K1'*H(:,:,LastPeriod+1)'*B0*Mmatrix;
         if rank(etaMat)<qm;
            %
            % solve the mode for initial parameter values or for the posterior mode values
            %
            [A,H,R,F,B0,status,ErrMsg,ModelParameters] = SolveDSGEModel(DSGEModel,thetaMode,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
         end;
         %
         % Try to solve the model subject to the anticipated shocks
         %
         [Gtmp,C1tmp,Ptmp,mcode] = ZLBKleinSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.ZLBSampleT,DSGEModel.RtildePosition,DSGEModel.REqPosition,DSGEModel.ZLBRuleType,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
         if mcode~=1;
            %
            % solve the mode for initial parameter values or for the posterior mode values
            %
            [A,H,R,F,B0,status,ErrMsg,ModelParameters] = SolveDSGEModel(DSGEModel,thetaMode,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
         end;
         %
         % get the structural form matrices
         %
         [Hlag,H0,Hexp,D] = DSGEStructuralForm(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions);
         %
         % reshuffle the equations to separate the restricted from the unrestricted policy rate
         %
         [Hlag,H0,Hexp,D,DA,MA] = ZLBDSGEStructuralForm(Hlag,H0,Hexp,D,DSGEModel.ZLBSampleT,DSGEModel.RtildePosition,DSGEModel.REqPosition,DSGEModel.ZLBRuleType);
         %
         % solve the model with the new matrices
         %
         [F,B0,BA,mcode] = SolveExtendedDSGEModel(Hlag,H0,Hexp,D,DA,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
         if mcode~=1;
            %
            % use the posterior mode estimates of the parameters
            %
            [A,H,R,F,B0,status,ErrMsg,ModelParameters] = SolveDSGEModel(DSGEModel,thetaMode,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
            [Hlag,H0,Hexp,D] = DSGEStructuralForm(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions);
            [Hlag,H0,Hexp,D,DA,MA] = ZLBDSGEStructuralForm(Hlag,H0,Hexp,D,DSGEModel.ZLBSampleT,DSGEModel.RtildePosition,DSGEModel.REqPosition,DSGEModel.ZLBRuleType);
            [F,B0,BA,mcode] = SolveExtendedDSGEModel(Hlag,H0,Hexp,D,DA,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
         end;
         %
         % Compute the initial values of betavec, Sigmau, R10 and Sigmaeps
         %
         [betavec,Sigmau,R10,Sigmaeps,Gtilde,Kgf] = AdaptiveLearningInitialValuesDSGE(F,B0,ModelParameters.sigmarAL,ModelParameters.sigmaeAL,SMatrix,G,m,x,f,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,Gtilde,Kgf);
         %
         % call the transformation function
         %
         [TransStatus,Hstarlag,Hstar0,Hstar1f] = AdaptiveLearningTransformDSGEModel(Hlag,H0,Hexp,SMatrix,Sperp);
         %
         % compute initial values for the solution matrices
         %
         if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
            [Mt1,Ft1,Bt1,InvH0H1star] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betavec);
         else;
            [Mt1,Ft1,Bt1,InvH0H1star] = feval(GetName(DSGEModel.AdaptiveLearningModelSolverFile),Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betavec,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
         end;
         %
         % compute the solution matrix BAt1 on the anticipated shocks
         %
         BAt1 = InvH0H1star*DA;
         %
         % make sure that we take into account that Rbar depends on the parameters
         %
         if k>0;
            Rbar = A(:,DSGEModel.ZLBPosition)'*X;
         end;
         Rbar = Rbar(:,1:DSGEModel.ZLBSampleT+1);
         %
         % Transform the steady-state interest rate and the lower bound to the same units
         % as the unrestricted monetary policy rate
         %
         Hvalue = shiftdim(H(DSGEModel.RtildePosition,DSGEModel.ZLBPosition,DSGEModel.LastPeriod:Hlength),2)';
         Th = length(Hvalue);
         if Th>DSGEModel.ZLBSampleT;
            Hvalue = Hvalue(1,1:DSGEModel.ZLBSampleT);
         elseif Th<DSGEModel.ZLBSampleT;
            Hvalue = [Hvalue Hvalue(1,Th)*ones(1,DSGEModel.ZLBSampleT-Th)];
         end;
         Rzlb = RzlbMat./Hvalue;
         Rbar = Rbar./Hvalue;
         %
         % we need to fix the H-matrix when using the extended form of the model
         % by adding zeros to 
         %
         HOld = H;
         %
         % we need to fix the H-matrix when using the extended form of the model
         % by adding zeros to 
         %
         HT = zeros(size(H));
         for t=1:size(H,3);
            HT(:,:,t) = [H(:,:,t);zeros(1,n)];
         end;
         H = HT;
         %
         % run the Kalman filter
         %
         if IsALUpdate==1;
            if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
               [lnL,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningKalmanFilter(m,SMatrix,betavec,Sigmau,R10,Sigmaeps,ModelParameters.rhoAL,Hstar0,Hstarlag,Hstar1f,D,Mt1,Ft1,Bt1,Gtilde,Kgf,DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,KalmanFirstObservation,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs);
            else;
               [lnL,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningUserSolverKalmanFilter(GetName(DSGEModel.AdaptiveLearningModelSolverFile),m,SMatrix,betavec,Sigmau,R10,Sigmaeps,ModelParameters.rhoAL,Hstar0,Hstarlag,Hstar1f,D,Mt1,Ft1,Bt1,Gtilde,Kgf,DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,KalmanFirstObservation,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
            end;
         else;
            [lnL,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningSingleKalmanFilter(m,betavec,Mt1,Ft1,Bt1,DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,KalmanFirstObservation,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs);
            LastSolution.InvMatT1 = InvH0H1star;
            LastSolution.InvMatT = InvH0H1star;
         end;
         %
         % check if the user wants to take the conditioning infortmation into account
         % when determining the mean and covariance matrix for the states at T.
         %
         if (DSGEModel.KsiUseCondData==1)&(SCond==1);
            if (k>0);
               XPred = X(:,1:g);
            else;
               XPred = [];
            end;
            [KsiTTz,PTTz] = AdaptiveLearningCondPredictionSmoother(Ksitt1(:,T),Ptt1(:,:,T),Yhat(:,T),DSGEModel.Y(:,LastPeriod),XPred,Z,K1,A,H(:,:,LastPeriod+1:LastPeriod+g),R,Mtilde(:,:,T),Ftilde(:,:,T),Btilde(:,:,T),betavec,LastSolution.betaTT,LastSolution.M,Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,DSGEModel);
%            KsiTTz = Ksitt(:,T);
%            PTTz = Ptt(:,:,T);
         elseif (DSGEModel.KsiUseCondData==1)&(T>=g);
            %
            % we need to run the Kalman smoother
            %
            [KsitT,PtT,etatt,etatT,wtt,wtT] = AdaptiveLearningKalmanSmoother(Ksitt1,Ptt1,Yerror,InvMSEY,H,R,Ftilde,Btilde,FirstPeriod,LastPeriod,KalmanFirstObservation,DSGEModel.YNaNs);
            PtT = PtT(:,:,T-g+1:T);
            KsitT = KsitT(:,T-g+1:T);
            if isempty(wtT)==1;
               wtT = zeros(n,g);
            else;
               wtT = wtT(:,T-g+1:T);
            end;
            %
            if k>0;
               if g>1;
                  Xtra = DSGEModel.X(:,LastPeriod-g+2:LastPeriod);
               else;
                  Xtra = [];
               end;
               [KsiTTz,PTTz] = AdaptiveLearningCondPredictionKalmanSmoother(KsitT,PtT,wtT,Z,U,[Xtra X(:,1:g)],K1,K2,A,H(:,:,LastPeriod-g+2:LastPeriod+g),R,Mtilde(:,:,T),Ftilde(:,:,T),Btilde(:,:,T),betavec,LastSolution.betaTT,LastSolution.M,Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,DSGEModel);
            else;
               [KsiTTz,PTTz] = AdaptiveLearningCondPredictionKalmanSmoother(KsitT,PtT,wtT,Z,U,[],K1,K2,A,H(:,:,LastPeriod-g+2:LastPeriod+g),R,Mtilde(:,:,T),Ftilde(:,:,T),Btilde(:,:,T),betavec,LastSolution.betaTT,LastSolution.M,Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,DSGEModel);
            end;
         else;
            KsiTTz = Ksitt(:,T);
            PTTz = Ptt(:,:,T);
         end;
         PTTz = diag(abs(diag(PTTz)))+tril(PTTz,-1)+tril(PTTz,-1)';
         %
         % use the Schur decomposition for the reduced rank matrix PTTz instead of
         % the Choleski decomposition
         %
         [S,L] = SchurDecomposition(PTTz);
         KsiTT = KsiTTz;
         %
         % deal with possible missing observations in the historical sample if we
         % should annualize or transform the predictions
         %
         if AnnualizeData==1;
            if maxYinit==1;
               %
               % take into account that there are missing obs for the
               % historical data which can affect annualization
               %
               YSmooth = DSGEModel.Y(:,FirstPeriod+KalmanFirstObservation-1:LastPeriod);
               nanpos = isnan(YSmooth);
               %
               % compute the Kalman smoother
               %
               [KsitT,PtT,etatt,etatT,wtt,wtT] = AdaptiveLearningKalmanSmoother(Ksitt1,Ptt1,Yerror,InvMSEY,H,R,Ftilde,Btilde,FirstPeriod,LastPeriod,KalmanFirstObservation,DSGEModel.YNaNs);
               %
               if k>0;
                  for t=1:T;
                     YHistPred = (A'*Xdata(:,t))+(H(:,:,FirstPeriod+KalmanFirstObservation-2+t)'*KsitT(:,t));
                  end;
               else;
                  for t=1:T;
                     YHistPred = H(:,:,FirstPeriod+KalmanFirstObservation-2+t)'*KsitT(:,t);
                  end;
               end;
               YSmooth(nanpos) = YHistPred(nanpos);
               if FirstPeriod+KalmanFirstObservation-1>1;
                  YSmooth = [DSGEModel.Y(:,1:FirstPeriod+KalmanFirstObservation-2) YSmooth];
               end;
               if strcmp(DSGEModel.DataFrequency,'q')==1;
                  Yinit = YSmooth(:,LastPeriod-2:LastPeriod);
               elseif strcmp(DSGEModel.DataFrequency,'m')==1;
                  Yinit = YSmooth(:,LastPeriod-10:LastPeriod);
               else;
                  Yinit = [];
               end;
            end;
         elseif TransData==1;
            if maxYinitial==1;
               %
               % take into account that there are missing obs for the
               % historical data which can affect general transformations
               %
               YSmooth = DSGEModel.Y(:,FirstPeriod+KalmanFirstObservation-1:LastPeriod);
               nanpos = isnan(YSmooth);
               %
               % compute the Kalman smoother
               %
               [KsitT,PtT,etatt,etatT,wtt,wtT] = AdaptiveLearningKalmanSmoother(Ksitt1,Ptt1,Yerror,InvMSEY,H,R,Ftilde,Btilde,FirstPeriod,LastPeriod,KalmanFirstObservation,DSGEModel.YNaNs);
               %
               if k>0;
                  for t=1:T;
                     YHistPred = (A'*Xdata(:,t))+(H(:,:,FirstPeriod+KalmanFirstObservation-2+t)'*KsitT(:,t));
                  end;
               else;
                  for t=1:T;
                     YHistPred = H(:,:,FirstPeriod+KalmanFirstObservation-2+t)'*KsitT(:,t);
                  end;
               end;
               YSmooth(nanpos) = YHistPred(nanpos);
               if FirstPeriod+KalmanFirstObservation-1>1;
                  YSmooth = [DSGEModel.Y(:,1:FirstPeriod+KalmanFirstObservation-2) YSmooth];
               end;
               Yinitial = YSmooth(:,1:LastPeriod);
            end;
         end;
         %
         % store solution matrices
         %
         if DSGEModel.FixedBeliefsForProj==1;
            MT1 = Mtilde(:,:,T);
            FT1 = Ftilde(:,:,T);
            BT1 = Btilde(:,:,T);
         else;
            MT1 = zeros(r+1,k,h);
            FT1 = zeros(r+1,r,h);
            BT1 = zeros(r+1,q,h);
            MT1(:,:,1) = Mtilde(:,:,T);
            FT1(:,:,1) = Ftilde(:,:,T);
            BT1(:,:,1) = Btilde(:,:,T);
            InvMatT1 = zeros(r+1,r+1,h);
            InvMatT1(:,:,1) = LastSolution.InvMatT1;
         end;
         %
         % write down the solution matrices for projections using ZLB format, with Mt, Gt and C1
         %
         [Mt,Gt,C1,Mtilde,Pt] = ZLBProjectionMatrices(Mtilde(:,:,T),Ftilde(:,:,T),Btilde(:,:,T),LastSolution.InvMatT1*DA,MA);
         %
         % deal with deterministic variables
         %
         if k>0;
            Ax = A'*X;
         else;
            Ax = zeros(n,h);
            X = zeros(r,max(DSGEModel.ZLBSampleT+1,h))
         end;
         %
         % setup projections matrices for the restricted policy rate
         %
         % each row in ePG is the row vector multiplied by V(t) for tau=0,1,...,DSGEModel.ZLBSampleT
         % each element in ePGx is the deterministic term of the projection
         %
         ePG = zeros(DSGEModel.ZLBSampleT+1,size(Gt,2));
         ePGx = zeros(DSGEModel.ZLBSampleT+1,1);
         Pmats = zeros(r+1,q+r+2+DSGEModel.ZLBSampleT,max(DSGEModel.ZLBSampleT+1,h));
         Mmats = zeros(r+1,h);
         if DSGEModel.FixedBeliefsForProj==1;
            ePG(1,:) = e_pr'*Pt(1:r+1,:);
            xbart = zeros(size(Mt,1),1);
            ePGx(1) = e_pr'*Mtilde*X(:,1);
            Mmats(:,1) = Mtilde*X(:,1);
            Pmats(:,:,1) = Pt;
            for tau=1:DSGEModel.ZLBSampleT;
               ePG(tau+1,:) = ePG(tau,:)*Gt;
               xbart = (Mt*X(:,tau))+Gt*xbart;
               ePGx(tau+1) = e_pr'*((Mtilde*X(:,tau+1))+(Pt*xbart));
               Mmats(:,tau+1) = Mtilde*X(:,tau+1);
               Pmats(:,:,tau+1) = Pt;
            end;
            if h>DSGEModel.ZLBSampleT+1;
               for tau=DSGEModel.ZLBSampleT+1:h-1;
                  Pmats(:,:,tau+1) = Pt;
                  Mmats(:,tau+1) = Mtilde*X(:,tau+1);
               end;
            end;
         else;
            %
            % beliefs are now updated, take the values of the
            % solution matrices from period T that have been updated for T+1
            %
            MTiT = LastSolution.MTT;
            FTiT = LastSolution.FTT;
            BTiT = LastSolution.BTT;
            betaTiT = LastSolution.betaTT;
            InvMatT = LastSolution.InvMatT;
            MT1(:,:,2) = LastSolution.MTT;
            FT1(:,:,2) = LastSolution.FTT;
            BT1(:,:,2) = LastSolution.BTT;
            InvMatT1(:,:,2) = LastSolution.InvMatT;
            %
            % write down the solution matrices for projections using ZLB format, with Mt, Gt and C1
            %
            [Mt,Gt,C1,Mtilde,Pt] = ZLBProjectionMatrices(MTiT,FTiT,BTiT,InvMatT*DA,MA);
            ePG(1,:) = e_pr'*Pt(1:r+1,:);
            xbart = zeros(size(Mt,1),1);
            ePGx(1) = e_pr'*Mtilde*X(:,1);
            Mmats(:,1) = Mtilde*X(:,1);
            Pmats(:,:,1) = Pt;
            for tau=1:DSGEModel.ZLBSampleT;
               %
               % update betaTiT
               %
               betaTiT = betavec+(LastSolution.M*(betaTiT-betavec));
               %
               % update the solution matrices
               %
               if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
                  [MTiT,FTiT,BTiT,InvMatT] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betaTiT);
               else;
                  [MTiT,FTiT,BTiT,InvMatT] = feval(GetName(DSGEModel.AdaptiveLearningModelSolverFile),Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betaTiT,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
               end;
               %
               % obtain the new projection matrices
               %
               [Mt,Gt,C1,Mtilde,Pt] = ZLBProjectionMatrices(MTiT,FTiT,BTiT,InvMatT*DA,MA);
               ePG(tau+1,:) = ePG(tau,:)*Gt;
               xbart = (Mt*X(:,tau))+Gt*xbart;
               ePGx(tau+1) = e_pr'*((Mtilde*X(:,tau+1))+(Pt*xbart));
               Mmats(:,tau+1) = Mtilde*X(:,tau+1);
               Pmats(:,:,tau+1) = Pt;
               if tau+2<=h;
                  MT1(:,:,tau+2) = MTiT;
                  FT1(:,:,tau+2) = FTiT;
                  BT1(:,:,tau+2) = BTiT;
                  InvMatT1(:,:,tau+2) = InvMatT;
               end;
            end;
            if h>DSGEModel.ZLBSampleT+1;
               for tau=DSGEModel.ZLBSampleT+1:h-1;
                  %
                  % update betaTiT
                  %
                  betaTiT = betavec+(LastSolution.M*(betaTiT-betavec));
                  %
                  % update the solution matrices
                  %
                  if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
                     [MTiT,FTiT,BTiT,InvMatT] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betaTiT);
                  else;
                     [MTiT,FTiT,BTiT,InvMatT] = feval(GetName(DSGEModel.AdaptiveLearningModelSolverFile),Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betaTiT,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
                  end;
                  %
                  % obtain the new projection matrices
                  %
                  [Mt,Gt,C1,Mtilde,Pt] = ZLBProjectionMatrices(MTiT,FTiT,BTiT,InvMatT*DA,MA);
                  Pmats(:,:,tau+1) = Pt;
                  Mmats(:,tau+1) = Mtilde*X(:,tau+1);
                  if tau+2<=h;
                     MT1(:,:,tau+2) = MTiT;
                     FT1(:,:,tau+2) = FTiT;
                     BT1(:,:,tau+2) = BTiT;
                     InvMatT1(:,:,tau+2) = InvMatT;
                  end;
               end;
            end;
         end;
         %
         % stack the model to determine the shock distribution
         %
         Xbar = zeros(n*g,1);
         XTi = zeros(r+1,1);
         Gmat = zeros(n*g,r+1);
         DT1 = zeros(n*g,q*g);
         Fexp = eye(r+1);
         if DSGEModel.FixedBeliefsForProj==1;
            for i=1:g;
               Fexp = FT1*Fexp;
               Ind = (1+((g-i)*n):(g-i+1)*n);
               Ind1 = (1+((g-i)*q):(g-i+1)*q);
               Ind2 = (1+((g-i)*qm):(g-i+1)*qm);
               if k>0;
                  XTi = (MT1*X(:,i))+(FT1*XTi);
                  Xbar(Ind,1) = (A'*X(:,i))+(H(:,:,LastPeriod+i)'*XTi);
               end;
               Gmat(Ind,:) = H(:,:,LastPeriod+i)'*Fexp;
               DT1(Ind,Ind1) = H(:,:,LastPeriod+i)'*BT1;
               Fi = FT1;
               for j=1:i-1;
                  Ind1 = Ind1+q;
                  Ind3 = Ind+(j*n);
                  DT1(Ind,Ind1) = H(:,:,LastPeriod+i)'*Fi*BT1;
                  Fi = Fi*FT1;
               end;
            end;
         else;
            %
            % we have updated beliefs
            %
            for i=1:g;
               Fexp = FT1(:,:,i)*Fexp;
               Ind = (1+((g-i)*n):(g-i+1)*n);
               Ind1 = (1+((g-i)*q):(g-i+1)*q);
               Ind2 = (1+((g-i)*qm):(g-i+1)*qm);
               if k>0;
                  XTi = (MT1(:,:,i)*X(:,i))+(FT1(:,:,i)*XTi);
                  Xbar(Ind,1) = (A'*X(:,i))+(H(:,:,LastPeriod+i)'*XTi);
               end;
               Gmat(Ind,:) = H(:,:,LastPeriod+i)'*Fexp;
               DT1(Ind,Ind1) = H(:,:,LastPeriod+i)'*BT1(:,:,i);
               Fi = FT1(:,:,i);
               for j=1:i-1;
                  Ind1 = Ind1+q;
                  Ind3 = Ind+(j*n);
                  DT1(Ind,Ind1) = H(:,:,LastPeriod+i)'*Fi*BT1(:,:,i);
                  Fi = Fi*FT1(:,:,i-j);
               end;
            end;
         end;
         %
         % speed-up code suggested by Ingvar Strid
         %
         KpD = K'*DT1*IgM;
         KpDDpK = KpD*KpD';
         DpKInvKpDDpK = KpD'/KpDDpK;
         %
         % compute the idempotent covariance matrix for the shocks
         %
         SigmaTg = eye(q_r*g)-(DpKInvKpDDpK*KpD);
         SigmaTg = diag(abs(diag(SigmaTg)))+tril(SigmaTg,-1)+tril(SigmaTg,-1)';
         [SE,LE] = schur(SigmaTg);
         SigmaTgStd = SE*sqrt(abs(LE));
         %
         % check if we have measurement errors and if so compute a Schur decomposition of the R matrix
         %
         if rank(R)>0;
            R(:,DSGEModel.ZLBPosition) = 0;
            R(DSGEModel.ZLBPosition,:) = 0;
            [Rm,Lm] = SchurDecomposition(R);
            Rmeasure = Rm*sqrt(Lm);
            NumMeasure = size(Lm,1);
         else;
            Rmeasure = [];
            wTh = zeros(n,h);
            NumMeasure = 0;
         end;
         %
         % check if we should compute the steady-state mean
         % of the observed variables
         %
         if k==1;
            ObsVarMeanTmp = (1/NumDraws)*A';
         end;
         %
         % we are now ready to compute possible paths at theta
         %
         PredPaths = zeros(n,h,NumPaths);
         AtMean = zeros(DSGEModel.ZLBSampleT+1,h);
         XtMean = zeros(q+r+1,h);
         ShockMatrix = zeros(q,h);
         NumZLBTimes = zeros(1,h);
         NumBadValues = 0;
         NumBindingValues = 0;
         %
         for it=1:NumPaths;
            CurrPath = it+(NumPaths*(ParamIndex-1));
            %
            % check if we should update the progress dialog
            %
            if DSGEModel.ShowProgress==1;
               abort = get(WaitHandle,'UserData');
               if strcmp(abort,'cancel')==1;
                  StopEstimation = 1;
                  break;
               else;
                  ProgressDLG([CurrPath/(NumDraws*NumPaths) CurrPath]);
               end;
            end;
            %
            % set up a loop to allow for new draws of the state vector
            % and if that fails to use the thetaMode parameter values
            %
            RunLoop = 1;
            TotalRunLoop = 1;
            while RunLoop==1;
               %
               % Draw a sequence of measurement errors
               %
               if isempty(Rmeasure)==0;
                  wTh = Rmeasure*randn(NumMeasure,h);
                  wTh(DSGEModel.ZLBPosition,:) = 0;
               else;
                  wTh = zeros(n,h);
               end;
               WTg = vec(fliplr(wTh(:,1:g)));
               %
               % Draw a state vector for time T using the N(KsiTTz,PTTz) distribution.
               %
               KsiT = KsiTT+(S*sqrt(L)*randn(size(L,2),1));
               %
               % make sure that the initial values of the unconstrained and constrained policy rate are identical
               %
               KsiT(r+1) = KsiT(DSGEModel.RtildePosition);
               %
               etaTh = zeros(q,h);
               etaTh(:,1:g) = randn(q,g);
               %
               % retrieve the q-q_r shocks eta(t,q-q_r)
               %
               NTgFree = vec(fliplr(Morth'*etaTh(:,1:g)));
               %
               % setut the mean for the shock distribution
               %
               kTg = ZTg-UT-(K'*(Xbar+(Gmat*KsiT)+WTg+(DT1*IgMorth*NTgFree)));
               muTg = DpKInvKpDDpK*kTg;
               PredictY = zeros(n,h);
               At = zeros(DSGEModel.ZLBSampleT+1,1);
               Xt = [zeros(q,1);KsiT];
               ZLBKsiPath = zeros(r,h);
               AtPath = zeros(DSGEModel.ZLBSampleT+1,h);
               XtPath = zeros(q+r+1,h);
               %
               % draw economic shocks from N(muTg,SigmaTg) for period 1 to g
               % and from N(0,I) for period g+1 to h.
               %
               ShockDraw = randn(q_r*g,1);
               etaTg = muTg+(SigmaTgStd*ShockDraw);
               %
               % overlay with restricted shocks
               %
               etaTh(DSGEModel.MixedConditionalShocks,1:g) = fliplr(reshape(etaTg,q_r,g));
               %
               gGood = 0;
               t = 1;
               K2sum = zeros(qm,1);
               BadDraw = 0;
               while t<=g;
                  ZPosi = ZPos(:,t);
                  m1 = sum(ZPosi);
                  %
                  % add shocks to Xt
                  %
                  Xt(1:q) = etaTh(:,t);
                  [AtNew,status,AlgorithmInformation] = ForwardBackShootingAlgorithmAdaptiveLearning(Xt,At,ePG,ePGx,Rzlb(t,:),Rbar,DSGEModel.AIMTolerance);
                  if status==1;
                     %
                     % check if we have non-zero anticipated shocks
                     %
                     Ksit = Mmats(:,t)+(Pmats(:,:,t)*[Xt;AtNew]);
                     if (sum(AtNew)>0)&(m1>0);
                        NumBindingValues = NumBindingValues+1;
                        %
                        % check the constraint
                        %
                        if k>0;
                           CA = Z(ZPosi,t) - (K1(:,ZPosi)'*A'*X(:,t)) - (K1(:,ZPosi)'*wTh(:,t)) - (K1(:,ZPosi)'*H(:,:,LastPeriod+t)'*Ksit) - K2sum(ZPosi) -U(ZPosi,g);
                        else;
                           CA = Z(ZPosi,t) - (K1(:,ZPosi)'*wTh(:,t)) - (K1(:,ZPosi)'*H(:,:,LastPeriod+t)'*Ksit) - K2sum(ZPosi) -U(ZPosi,g);
                        end;
                        if abs(CA)>=1.0e-06;
                           NumBadValues = NumBadValues+1;
                           status = 0;
                        end;
                     end;
                     if status==1;
                        At = AtNew;
                        AtPath(:,t) = At;
                        XtPath(:,t) = Xt;
                        if max(At)>0;
                           %
                           % add the shocks when ZLB is binding and count how many times for given horizon
                           %
                           ShockMatrix(:,t) = ShockMatrix(:,t)+Xt(1:q,1);
                           NumZLBTimes(1,t) = NumZLBTimes(1,t)+1;
                        end;
                        ZLBKsiPath(:,t) = Ksit(1:r);
                        KsiT = Ksit;
                        PredictY(:,t) = Ax(:,t)+(H(:,:,LastPeriod+t)'*Ksit)+wTh(:,t);
                        %
                        % test if the prediction is "reasonable"
                        %
                        TestValue = (PredictY(:,t)-ZLBYMeanOrig(:,t))./ZLBYMeanOrig(:,t);
                        if max(abs(TestValue))<100;
                           if t+1<=h;
                              At = MA*At;
                              Xt = [zeros(q,1);Ksit];
                           end;
                           %
                           % update the sum of K2 times lagged values of the observed variables
                           %
                           if K2length==2;
                              K2sum = K2sum+(K2'*PredictY(:,t));
                           else;
                              K2sum = zeros(qm,1);
                              for j=1:t;
                                 if j<=size(K2,3);
                                    K2sum = K2sum+(K2(:,:,j)'*PredictY(:,t+1-j));
                                 end;
                              end;
                           end;
                           t = t+1;
                           if t==g+1;
                              gGood = 1;
                           end;
                        else;
                           t = g+1;
                        end;
                     else;
                        t = g+1;
                     end;
                  else;
                     BadDraw = BadDraw+1;
                     if BadDraw>=100;
                        t = g+1;
                     end;
                  end;
               end;
               %
               % check if we should continue
               %
               if gGood==1;
                  if t==h+1;
                     RunLoop = 0;
                  end;
                  while t<=h;
                     Xt(1:q) = randn(q,1);
                     [AtNew,status,AlgorithmInformation] = ForwardBackShootingAlgorithmAdaptiveLearning(Xt,At,ePG,ePGx,Rzlb(t,:),Rbar,DSGEModel.AIMTolerance);
                     if status==1;
                        At = AtNew;
                        AtPath(:,t) = At;
                        XtPath(:,t) = Xt;
                        if max(At)>0;
                           %
                           % add the shocks when ZLB is binding and count how many times for given horizon
                           %
                           ShockMatrix(:,t) = ShockMatrix(:,t)+Xt(1:q,1);
                           NumZLBTimes(1,t) = NumZLBTimes(1,t)+1;
                        end;
                        Ksit = Mmats(:,t)+(Pmats(:,:,t)*[Xt;At]);
                        ZLBKsiPath(:,t) = Ksit(1:r);
                        %
                        PredictY(:,t) = Ax(:,t)+(H(:,:,LastPeriod+t)'*Ksit)+wTh(:,t);
                        %
                        % test if the prediction is "reasonable"
                        %
                        TestValue = (PredictY(:,t)-ZLBYMeanOrig(:,t))./ZLBYMeanOrig(:,t);
                        if max(abs(TestValue))<100;
                           if t+1<=h;
                              At = MA*At;
                              Xt = [zeros(q,1);Ksit];
                           end;
                           t = t+1;
                           if t==h+1;
                              RunLoop = 0;
                           end;
                        else;
                           t = h+1;
                        end;
                     end;
                  end;
               end;
               TotalRunLoop = TotalRunLoop+1;
               if TotalRunLoop>100;
                  %
                  % use the original parameters instead of the current vector as it doesn't work
                  %
                  KsiTT = KsiTTzmode;
                  S = Smode;
                  L = Lmode;
                  ePG = ePGmode;
                  ePGx = ePGxmode;
                  Mmats = Mmatsmode;
                  Pmats = Pmatsmode;
                  Rzlb = Rzlbmode;
                  Rbar = Rbarmode;
                  Ax = Axmode;
                  A = Amode;
                  H = Hmode;
                  Rmeasure = Rmeasuremode;
                  NumMeasure = NumMeasuremode;
                  MT1 = MT1mode;
                  FT1 = FT1mode;
                  BT1 = BT1mode;
                  MA = MAmode;
                  Xbar = Xbarmode;
                  Gmat = Gmatmode;
                  DpKInvKpDDpK = DpKInvKpDDpKmode;
                  DT1 = DT1mode;
                  SigmaTgStd = SigmaTgStdmode;
                  TotalRunLoop = 1;
               end;
            end;
            %
            % check if we should annualize or transform
            %
            if AnnualizeData==1;
               PredictY = Annualize([Yinit PredictY],DSGEModel.annual,DSGEModel.annualscale);
            elseif TransData==1;
               YTmp = [Yinitial PredictY];
               [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,YTmp',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
               PredictY = YData(Tt-h+1:Tt,:)';
            end;
            %
            % update the mean predictions for all paths and parameter draws
            %
            MeanPredictY = MeanPredictY+((Weights(ParamIndex)/(NumPaths*NumDraws))*PredictY);
            %
            % add the projected path to the variable to be saved
            %
            PredPaths(:,:,it) = PredictY;
            drawnow;
         end;
         %
         if StopEstimation==1;
            break;
         end;
         %
         % save the calculations to disk
         %
         if IsPosterior==1;
            if AnnualizeData==0;
               if TransData==0;
                  file = FixFilePath([DSGEModel.OutputDirectory '\predictions\AdaptiveLearningDSGE-' int2str(DSGEModel.FixedBeliefsForProj) '-ZLB-CondPrediction-Original-0-Subset-' DSGEModel.NameOfModel '-' num2str(CurrChain,'%0.0f') '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '.' int2str(DSGEModel.ZLBRuleType) '-' num2str(h,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPaths,'%0.0f') '.mat']);
               else;
                  file = FixFilePath([DSGEModel.OutputDirectory '\predictions\AdaptiveLearningDSGE-' int2str(DSGEModel.FixedBeliefsForProj) '-ZLB-CondPrediction-Transformed-0-Subset-' DSGEModel.NameOfModel '-' num2str(CurrChain,'%0.0f') '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '.' int2str(DSGEModel.ZLBRuleType) '-' num2str(h,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPaths,'%0.0f') '.mat']);
               end;
            else;
               file = FixFilePath([DSGEModel.OutputDirectory '\predictions\AdaptiveLearningDSGE-' int2str(DSGEModel.FixedBeliefsForProj) '-ZLB-CondPrediction-Annual-0-Subset-' DSGEModel.NameOfModel '-' num2str(CurrChain,'%0.0f') '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '.' int2str(DSGEModel.ZLBRuleType) '-' num2str(h,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPaths,'%0.0f') '.mat']);
            end;
         else;
            if AnnualizeData==0;
               if TransData==0;
                  file = FixFilePath([DSGEModel.OutputDirectory '\predictions\AdaptiveLearningDSGE-' int2str(DSGEModel.FixedBeliefsForProj) '-ZLB-CondPrediction-Original-Prior-0-Subset-' DSGEModel.NameOfModel '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '.' int2str(DSGEModel.ZLBRuleType) '-' num2str(h,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPaths,'%0.0f') '.mat']);
               else;
                  file = FixFilePath([DSGEModel.OutputDirectory '\predictions\AdaptiveLearningDSGE-' int2str(DSGEModel.FixedBeliefsForProj) '-ZLB-CondPrediction-Transformed-Prior-0-Subset-' DSGEModel.NameOfModel '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '.' int2str(DSGEModel.ZLBRuleType) '-' num2str(h,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPaths,'%0.0f') '.mat']);
               end;
            else;
               file = FixFilePath([DSGEModel.OutputDirectory '\predictions\AdaptiveLearningDSGE-' int2str(DSGEModel.FixedBeliefsForProj) '-ZLB-CondPrediction-Annual-Prior-0-Subset-' DSGEModel.NameOfModel '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '.' int2str(DSGEModel.ZLBRuleType) '-' num2str(h,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPaths,'%0.0f') '.mat']);
            end;
         end;
         if k==1;
            ObsVarMean = ObsVarMean+ObsVarMeanTmp;
         end;
         save(file,'PredPaths','ShockMatrix','NumZLBTimes');
         if DSGEModel.RunPredictionEvent==1;
            %
            % compute the prediction events results
            %
            PredEventData = PredEventData+CalculatePredictionEvents(PredPaths,DSGEModel.PredictionEvent,Weights(ParamIndex));
         end;
      end;
   end;
   %
   % close the wait dialog
   %
   if DSGEModel.ShowProgress==1;
      if IsHandle(WaitHandle)==1;
         set(WaitHandle,'UserData','done');
         close(WaitHandle);
         drawnow;
         pause(0.02);
      end;
   else;
      set(WaitControls.text,'UserData','done');
      delete(WaitHandle);
      drawnow;
      pause(0.02);
   end;
else;
   %
   % we can do parallel computations over the parameter draws
   %
   txt = ['Please wait while YADA computes predictive distributions subject to the zero (effective) lower bound for the ' lower(TypeStr) ' data using parallel computations with ' int2str(CurrINI.NumWorkers) ' workers. Computations started at: ' StringTrim(datestr(now,14))];
   WaitHandle = WaitDLG(txt,'information',['ZLB Predictive Distributions ' PriorHeader ' - ' TypeStr ' Data'],600,150,CurrINI,0);
   WaitControls = get(WaitHandle,'UserData');
   %
   if DSGEModel.RandomNumberValue==1;
      %
      % without eval statements the spmd-code gives an error in older matlab systems
      %
      eval('spmd; rng(0,''combRecursive''); end;');
   else;
      eval('spmd; rng(''default''); end;');
   end;
   %
   % begin the loop
   %
   abort = '';
   StopEstimation = 0;
   MP = ModelParameters;
   Gtildetmp = Gtilde;
   Kgftmp = Kgf;
   Xtmp = X;
   if length(size(H))==2;
      parfor ParamIndex=1:NumDraws;
         %
         % Solve the model for the current parameter vector
         %
         theta = thetaPostSample(ParamIndex,:)';
         ModelParameters = MP;
         Gtilde = Gtildetmp;
         Kgf = Kgftmp;
         X = Xtmp;
         [A,H,R,F,B0,status,ErrMsg,ModelParameters] = SolveDSGEModel(DSGEModel,theta,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
         if status~=1;
            %
            % use posterior mode or initial values as a backup
            %
            [A,H,R,F,B0,status,ErrMsg,ModelParameters] = SolveDSGEModel(DSGEModel,thetaMode,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
         end;
         %
         % check if we can use these values.
         %
         etaMat = K1'*H'*B0*M;
         if rank(etaMat)<qm;
            %
            % solve the mode for initial parameter values or for the posterior mode values
            %
            [A,H,R,F,B0,status,ErrMsg,ModelParameters] = SolveDSGEModel(DSGEModel,thetaMode,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
         end;
         %
         % Try to solve the model subject to the anticipated shocks
         %
         [Gtmp,C1tmp,Ptmp,mcode] = ZLBKleinSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.ZLBSampleT,DSGEModel.RtildePosition,DSGEModel.REqPosition,DSGEModel.ZLBRuleType,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
         if mcode~=1;
            %
            % solve the mode for initial parameter values or for the posterior mode values
            %
            [A,H,R,F,B0,status,ErrMsg,ModelParameters] = SolveDSGEModel(DSGEModel,thetaMode,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
         end;
         %
         % get the structural form matrices
         %
         [Hlag,H0,Hexp,D] = DSGEStructuralForm(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions);
         %
         % reshuffle the equations to separate the restricted from the unrestricted policy rate
         %
         [Hlag,H0,Hexp,D,DA,MA] = ZLBDSGEStructuralForm(Hlag,H0,Hexp,D,DSGEModel.ZLBSampleT,DSGEModel.RtildePosition,DSGEModel.REqPosition,DSGEModel.ZLBRuleType);
         %
         % solve the model with the new matrices
         %
         [F,B0,BA,mcode] = SolveExtendedDSGEModel(Hlag,H0,Hexp,D,DA,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
         if mcode~=1;
            %
            % use the posterior mode estimates of the parameters
            %
            [A,H,R,F,B0,status,ErrMsg,ModelParameters] = SolveDSGEModel(DSGEModel,thetaMode,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
            [Hlag,H0,Hexp,D] = DSGEStructuralForm(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions);
            [Hlag,H0,Hexp,D,DA,MA] = ZLBDSGEStructuralForm(Hlag,H0,Hexp,D,DSGEModel.ZLBSampleT,DSGEModel.RtildePosition,DSGEModel.REqPosition,DSGEModel.ZLBRuleType);
            [F,B0,BA,mcode] = SolveExtendedDSGEModel(Hlag,H0,Hexp,D,DA,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
         end;
         %
         % Compute the initial values of betavec, Sigmau, R10 and Sigmaeps
         %
         [betavec,Sigmau,R10,Sigmaeps,Gtilde,Kgf] = AdaptiveLearningInitialValuesDSGE(F,B0,ModelParameters.sigmarAL,ModelParameters.sigmaeAL,SMatrix,G,m,x,f,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,Gtilde,Kgf);
         %
         % call the transformation function
         %
         [TransStatus,Hstarlag,Hstar0,Hstar1f] = AdaptiveLearningTransformDSGEModel(Hlag,H0,Hexp,SMatrix,Sperp);
         %
         % compute initial values for the solution matrices
         %
         if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
            [Mt1,Ft1,Bt1,InvH0H1star] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betavec);
         else;
            [Mt1,Ft1,Bt1,InvH0H1star] = feval(GetName(DSGEModel.AdaptiveLearningModelSolverFile),Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betavec,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
         end;
         %
         % compute the solution matrix BAt1 on the anticipated shocks
         %
         BAt1 = InvH0H1star*DA;
         %
         % make sure that we take into account that Rbar depends on the parameters
         %
         if k>0;
            Rbar = A(:,DSGEModel.ZLBPosition)'*X;
         end;
         Rbar = Rbar(:,1:DSGEModel.ZLBSampleT+1);
         %
         % Transform the steady-state interest rate and the lower bound to the same units
         % as the unrestricted monetary policy rate
         %
         Hvalue = H(DSGEModel.RtildePosition,DSGEModel.ZLBPosition);
         Rzlb = (1/Hvalue)*RzlbMat;
         Rbar = (1/Hvalue)*Rbar;
         %
         % we need to fix the H-matrix when using the extended form of the model
         % by adding zeros to 
         %
         HOld = H;
         %
         % we need to fix the H-matrix when using the extended form of the model
         % by adding zeros to 
         %
         H = [H;zeros(1,n)];
         %
         % run the Kalman filter
         %
         if IsALUpdate==1;
            if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
               [lnL,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningKalmanFilter(m,SMatrix,betavec,Sigmau,R10,Sigmaeps,ModelParameters.rhoAL,Hstar0,Hstarlag,Hstar1f,D,Mt1,Ft1,Bt1,Gtilde,Kgf,DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,KalmanFirstObservation,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs);
            else;
               [lnL,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningUserSolverKalmanFilter(GetName(DSGEModel.AdaptiveLearningModelSolverFile),m,SMatrix,betavec,Sigmau,R10,Sigmaeps,ModelParameters.rhoAL,Hstar0,Hstarlag,Hstar1f,D,Mt1,Ft1,Bt1,Gtilde,Kgf,DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,KalmanFirstObservation,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
            end;
         else;
            [lnL,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningSingleKalmanFilter(m,betavec,Mt1,Ft1,Bt1,DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,KalmanFirstObservation,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs);
            LastSolution.InvMatT1 = InvH0H1star;
            LastSolution.InvMatT = InvH0H1star;
         end;
         %
         % check if the user wants to take the conditioning infortmation into account
         % when determining the mean and covariance matrix for the states at T.
         %
         if (DSGEModel.KsiUseCondData==1)&(SCond==1);
            if (k>0);
               XPred = X(:,1:g);
            else;
               XPred = [];
            end;
            [KsiTTz,PTTz] = AdaptiveLearningCondPredictionSmoother(Ksitt1(:,T),Ptt1(:,:,T),Yhat(:,T),DSGEModel.Y(:,LastPeriod),XPred,Z,K1,A,H,R,Mtilde(:,:,T),Ftilde(:,:,T),Btilde(:,:,T),betavec,LastSolution.betaTT,LastSolution.M,Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,DSGEModel);
%            KsiTTz = Ksitt(:,T);
%            PTTz = Ptt(:,:,T);
         elseif (DSGEModel.KsiUseCondData==1)&(T>=g);
            %
            % we need to run the Kalman smoother
            %
            [KsitT,PtT,etatt,etatT,wtt,wtT] = AdaptiveLearningKalmanSmoother(Ksitt1,Ptt1,Yerror,InvMSEY,H,R,Ftilde,Btilde,FirstPeriod,LastPeriod,KalmanFirstObservation,DSGEModel.YNaNs);
            PtT = PtT(:,:,T-g+1:T);
            KsitT = KsitT(:,T-g+1:T);
            if isempty(wtT)==1;
               wtT = zeros(n,g);
            else;
               wtT = wtT(:,T-g+1:T);
            end;
            if k>0;
               if g>1;
                  Xtra = DSGEModel.X(:,LastPeriod-g+2:LastPeriod);
               else;
                  Xtra = [];
               end;
               [KsiTTz,PTTz] = AdaptiveLearningCondPredictionKalmanSmoother(KsitT,PtT,wtT,Z,U,[Xtra X(:,1:g)],K1,K2,A,H,R,Mtilde(:,:,T),Ftilde(:,:,T),Btilde(:,:,T),betavec,LastSolution.betaTT,LastSolution.M,Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,DSGEModel);
            else;
               [KsiTTz,PTTz] = AdaptiveLearningCondPredictionKalmanSmoother(KsitT,PtT,wtT,Z,U,[],K1,K2,A,H,R,Mtilde(:,:,T),Ftilde(:,:,T),Btilde(:,:,T),betavec,LastSolution.betaTT,LastSolution.M,Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,DSGEModel);
            end;
         else;
            KsiTTz = Ksitt(:,T);
            PTTz = Ptt(:,:,T);
         end;
         PTTz = diag(abs(diag(PTTz)))+tril(PTTz,-1)+tril(PTTz,-1)';
         %
         % use the Schur decomposition for the reduced rank matrix PTTz instead of
         % the Choleski decomposition
         %
         [S,L] = SchurDecomposition(PTTz);
         KsiTT = KsiTTz;
         %
         % deal with possible missing observations in the historical sample if we
         % should annualize or transform the predictions
         %
         if AnnualizeData==1;
            if maxYinit==1;
               %
               % take into account that there are missing obs for the
               % historical data which can affect annualization
               %
               YSmooth = DSGEModel.Y(:,FirstPeriod+KalmanFirstObservation-1:LastPeriod);
               nanpos = isnan(YSmooth);
               %
               % compute the Kalman smoother
               %
               [KsitT,PtT,etatt,etatT,wtt,wtT] = AdaptiveLearningKalmanSmoother(Ksitt1,Ptt1,Yerror,InvMSEY,H,R,Ftilde,Btilde,FirstPeriod,LastPeriod,KalmanFirstObservation,DSGEModel.YNaNs);
               %
               if k>0;
                  YHistPred = (A'*Xdata)+(H'*KsitT);
               else;
                  YHistPred = H'*KsitT;
               end;
               YSmooth(nanpos) = YHistPred(nanpos);
               if FirstPeriod+KalmanFirstObservation-1>1;
                  YSmooth = [DSGEModel.Y(:,1:FirstPeriod+KalmanFirstObservation-2) YSmooth];
               end;
               if strcmp(DSGEModel.DataFrequency,'q')==1;
                  Yinit = YSmooth(:,LastPeriod-2:LastPeriod);
               elseif strcmp(DSGEModel.DataFrequency,'m')==1;
                  Yinit = YSmooth(:,LastPeriod-10:LastPeriod);
               else;
                  Yinit = [];
               end;
            end;
         elseif TransData==1;
            if maxYinitial==1;
               %
               % take into account that there are missing obs for the
               % historical data which can affect general transformations
               %
               YSmooth = DSGEModel.Y(:,FirstPeriod+KalmanFirstObservation-1:LastPeriod);
               nanpos = isnan(YSmooth);
               %
               % compute the Kalman smoother
               %
               [KsitT,PtT,etatt,etatT,wtt,wtT] = AdaptiveLearningKalmanSmoother(Ksitt1,Ptt1,Yerror,InvMSEY,H,R,Ftilde,Btilde,FirstPeriod,LastPeriod,KalmanFirstObservation,DSGEModel.YNaNs);
               %
               if k>0;
                  YHistPred = (A'*Xdata)+(H'*KsitT);
               else;
                  YHistPred = H'*KsitT;
               end;
               YSmooth(nanpos) = YHistPred(nanpos);
               if FirstPeriod+KalmanFirstObservation-1>1;
                  YSmooth = [DSGEModel.Y(:,1:FirstPeriod+KalmanFirstObservation-2) YSmooth];
               end;
               Yinitial = YSmooth(:,1:LastPeriod);
            end;
         end;
         %
         % store solution matrices
         %
         if DSGEModel.FixedBeliefsForProj==1;
            MT1 = Mtilde(:,:,T);
            FT1 = Ftilde(:,:,T);
            BT1 = Btilde(:,:,T);
         else;
            MT1 = zeros(r+1,k,h);
            FT1 = zeros(r+1,r,h);
            BT1 = zeros(r+1,q,h);
            MT1(:,:,1) = Mtilde(:,:,T);
            FT1(:,:,1) = Ftilde(:,:,T);
            BT1(:,:,1) = Btilde(:,:,T);
            InvMatT1 = zeros(r+1,r+1,h);
            InvMatT1(:,:,1) = LastSolution.InvMatT1;
         end;
         %
         % write down the solution matrices for projections using ZLB format, with Mt, Gt and C1
         %
         [Mt,Gt,C1,Mtilde,Pt] = ZLBProjectionMatrices(Mtilde(:,:,T),Ftilde(:,:,T),Btilde(:,:,T),LastSolution.InvMatT1*DA,MA);
         %
         % deal with deterministic variables
         %
         if k>0;
            Ax = A'*X;
         else;
            Ax = zeros(n,h);
            X = zeros(r,max(DSGEModel.ZLBSampleT+1,h))
         end;
         %
         % setup projections matrices for the restricted policy rate
         %
         % each row in ePG is the row vector multiplied by V(t) for tau=0,1,...,DSGEModel.ZLBSampleT
         % each element in ePGx is the deterministic term of the projection
         %
         ePG = zeros(DSGEModel.ZLBSampleT+1,size(Gt,2));
         ePGx = zeros(DSGEModel.ZLBSampleT+1,1);
         Pmats = zeros(r+1,q+r+2+DSGEModel.ZLBSampleT,max(DSGEModel.ZLBSampleT+1,h));
         Mmats = zeros(r+1,h);
         if DSGEModel.FixedBeliefsForProj==1;
            ePG(1,:) = e_pr'*Pt(1:r+1,:);
            xbart = zeros(size(Mt,1),1);
            ePGx(1) = e_pr'*Mtilde*X(:,1);
            Mmats(:,1) = Mtilde*X(:,1);
            Pmats(:,:,1) = Pt;
            for tau=1:DSGEModel.ZLBSampleT;
               ePG(tau+1,:) = ePG(tau,:)*Gt;
               xbart = (Mt*X(:,tau))+Gt*xbart;
               ePGx(tau+1) = e_pr'*((Mtilde*X(:,tau+1))+(Pt*xbart));
               Mmats(:,tau+1) = Mtilde*X(:,tau+1);
               Pmats(:,:,tau+1) = Pt;
            end;
            if h>DSGEModel.ZLBSampleT+1;
               for tau=DSGEModel.ZLBSampleT+1:h-1;
                  Pmats(:,:,tau+1) = Pt;
                  Mmats(:,tau+1) = Mtilde*X(:,tau+1);
               end;
            end;
         else;
            %
            % beliefs are now updated, take the values of the
            % solution matrices from period T that have been updated for T+1
            %
            MTiT = LastSolution.MTT;
            FTiT = LastSolution.FTT;
            BTiT = LastSolution.BTT;
            betaTiT = LastSolution.betaTT;
            InvMatT = LastSolution.InvMatT;
            MT1(:,:,2) = LastSolution.MTT;
            FT1(:,:,2) = LastSolution.FTT;
            BT1(:,:,2) = LastSolution.BTT;
            %
            % write down the solution matrices for projections using ZLB format, with Mt, Gt and C1
            %
            [Mt,Gt,C1,Mtilde,Pt] = ZLBProjectionMatrices(MTiT,FTiT,BTiT,InvMatT*DA,MA);
            ePG(1,:) = e_pr'*Pt(1:r+1,:);
            xbart = zeros(size(Mt,1),1);
            ePGx(1) = e_pr'*Mtilde*X(:,1);
            Mmats(:,1) = Mtilde*X(:,1);
            Pmats(:,:,1) = Pt;
            for tau=1:DSGEModel.ZLBSampleT;
               %
               % update betaTiT
               %
               betaTiT = betavec+(LastSolution.M*(betaTiT-betavec));
               %
               % update the solution matrices
               %
               if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
                  [MTiT,FTiT,BTiT,InvMatT] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betaTiT);
               else;
                  [MTiT,FTiT,BTiT,InvMatT] = feval(GetName(DSGEModel.AdaptiveLearningModelSolverFile),Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betaTiT,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
               end;
               %
               % obtain the new projection matrices
               %
               [Mt,Gt,C1,Mtilde,Pt] = ZLBProjectionMatrices(MTiT,FTiT,BTiT,InvMatT*DA,MA);
               ePG(tau+1,:) = ePG(tau,:)*Gt;
               xbart = (Mt*X(:,tau))+Gt*xbart;
               ePGx(tau+1) = e_pr'*((Mtilde*X(:,tau+1))+(Pt*xbart));
               Mmats(:,tau+1) = Mtilde*X(:,tau+1);
               Pmats(:,:,tau+1) = Pt;
               if tau+2<=h;
                  MT1(:,:,tau+2) = MTiT;
                  FT1(:,:,tau+2) = FTiT;
                  BT1(:,:,tau+2) = BTiT;
                  InvMatT1(:,:,tau+2) = InvMatT;
               end;
            end;
            if h>DSGEModel.ZLBSampleT+1;
               for tau=DSGEModel.ZLBSampleT+1:h-1;
                  %
                  % update betaTiT
                  %
                  betaTiT = betavec+(LastSolution.M*(betaTiT-betavec));
                  %
                  % update the solution matrices
                  %
                  if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
                     [MTiT,FTiT,BTiT,InvMatT] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betaTiT);
                  else;
                     [MTiT,FTiT,BTiT,InvMatT] = feval(GetName(DSGEModel.AdaptiveLearningModelSolverFile),Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betaTiT,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
                  end;
                  %
                  % obtain the new projection matrices
                  %
                  [Mt,Gt,C1,Mtilde,Pt] = ZLBProjectionMatrices(MTiT,FTiT,BTiT,InvMatT*DA,MA);
                  Pmats(:,:,tau+1) = Pt;
                  Mmats(:,tau+1) = Mtilde*X(:,tau+1);
                  if tau+2<=h;
                     MT1(:,:,tau+2) = MTiT;
                     FT1(:,:,tau+2) = FTiT;
                     BT1(:,:,tau+2) = BTiT;
                     InvMatT1(:,:,tau+2) = InvMatT;
                  end;
               end;
            end;
         end;
         %
         % stack the model to determine the shock distribution
         %
         Xbar = zeros(n*g,1);
         XTi = zeros(r+1,1);
         Gmat = zeros(n*g,r+1);
         DT1 = zeros(n*g,q*g);
         Fexp = eye(r+1);
         if DSGEModel.FixedBeliefsForProj==1;
            for i=1:g;
               Fexp = FT1*Fexp;
               Ind = (1+((g-i)*n):(g-i+1)*n);
               Ind1 = (1+((g-i)*q):(g-i+1)*q);
               Ind2 = (1+((g-i)*qm):(g-i+1)*qm);
               if k>0;
                  XTi = (MT1*X(:,i))+(FT1*XTi);
                  Xbar(Ind,1) = (A'*X(:,i))+(H'*XTi);
               end;
               Gmat(Ind,:) = H'*Fexp;
               DT1(Ind,Ind1) = H'*BT1;
               Fi = FT1;
               for j=1:i-1;
                  Ind1 = Ind1+q;
                  Ind3 = Ind+(j*n);
                  DT1(Ind,Ind1) = H'*Fi*BT1;
                  Fi = Fi*FT1;
               end;
            end;
         else;
            %
            % we have updated beliefs
            %
            for i=1:g;
               Fexp = FT1(:,:,i)*Fexp;
               Ind = (1+((g-i)*n):(g-i+1)*n);
               Ind1 = (1+((g-i)*q):(g-i+1)*q);
               Ind2 = (1+((g-i)*qm):(g-i+1)*qm);
               if k>0;
                  XTi = (MT1(:,:,i)*X(:,i))+(FT1(:,:,i)*XTi);
                  Xbar(Ind,1) = (A'*X(:,i))+(H'*XTi);
               end;
               Gmat(Ind,:) = H'*Fexp;
               DT1(Ind,Ind1) = H'*BT1(:,:,i);
               Fi = FT1(:,:,i);
               for j=1:i-1;
                  Ind1 = Ind1+q;
                  Ind3 = Ind+(j*n);
                  DT1(Ind,Ind1) = H'*Fi*BT1(:,:,i);
                  Fi = Fi*FT1(:,:,i-j);
               end;
            end;
         end;
         %
         % speed-up code suggested by Ingvar Strid
         %
         KpD = K'*DT1*IgM;
         KpDDpK = KpD*KpD';
         DpKInvKpDDpK = KpD'/KpDDpK;
         %
         % compute the idempotent covariance matrix for the shocks
         %
         SigmaTg = eye(q_r*g)-(DpKInvKpDDpK*KpD);
         SigmaTg = diag(abs(diag(SigmaTg)))+tril(SigmaTg,-1)+tril(SigmaTg,-1)';
         [SE,LE] = schur(SigmaTg);
         SigmaTgStd = SE*sqrt(abs(LE));
         %
         % check if we have measurement errors and if so compute a Schur decomposition of the R matrix
         %
         if rank(R)>0;
            R(:,DSGEModel.ZLBPosition) = 0;
            R(DSGEModel.ZLBPosition,:) = 0;
            [Rm,Lm] = SchurDecomposition(R);
            Rmeasure = Rm*sqrt(Lm);
            NumMeasure = size(Lm,1);
         else;
            Rmeasure = [];
            wTh = zeros(n,h);
            NumMeasure = 0;
         end;
         %
         % check if we should compute the steady-state mean
         % of the observed variables
         %
         if k==1;
            ObsVarMean = (1/NumDraws)*A';
         end;
         %
         % we are now ready to compute possible paths at theta
         %
         PredPaths = zeros(n,h,NumPaths);
         AtMean = zeros(DSGEModel.ZLBSampleT+1,h);
         XtMean = zeros(q+r+1,h);
         ShockMatrix = zeros(q,h);
         NumZLBTimes = zeros(1,h);
         NumBadValues = 0;
         NumBindingValues = 0;
         %
         for it=1:NumPaths;
            %
            % set up a loop to allow for new draws of the state vector
            % and if that fails to use the thetaMode parameter values
            %
            RunLoop = 1;
            TotalRunLoop = 1;
            while RunLoop==1;
               %
               % Draw a sequence of measurement errors
               %
               if isempty(Rmeasure)==0;
                  wTh = Rmeasure*randn(NumMeasure,h);
                  wTh(DSGEModel.ZLBPosition,:) = 0;
               else;
                  wTh = zeros(n,h);
               end;
               WTg = vec(fliplr(wTh(:,1:g)));
               %
               % Draw a state vector for time T using the N(KsiTTz,PTTz) distribution.
               %
               KsiT = KsiTT+(S*sqrt(L)*randn(size(L,2),1));
               %
               % make sure that the initial values of the unconstrained and constrained policy rate are identical
               %
               KsiT(r+1) = KsiT(DSGEModel.RtildePosition);
               %
               etaTh = zeros(q,h);
               etaTh(:,1:g) = randn(q,g);
               %
               % retrieve the q-q_r shocks eta(t,q-q_r)
               %
               NTgFree = vec(fliplr(Morth'*etaTh(:,1:g)));
               %
               % setut the mean for the shock distribution
               %
               kTg = ZTg-UT-(K'*(Xbar+(Gmat*KsiT)+WTg+(DT1*IgMorth*NTgFree)));
               muTg = DpKInvKpDDpK*kTg;
               PredictY = zeros(n,h);
               At = zeros(DSGEModel.ZLBSampleT+1,1);
               Xt = [zeros(q,1);KsiT];
               ZLBKsiPath = zeros(r,h);
               AtPath = zeros(DSGEModel.ZLBSampleT+1,h);
               XtPath = zeros(q+r+1,h);
               %
               % draw economic shocks from N(muTg,SigmaTg) for period 1 to g
               % and from N(0,I) for period g+1 to h.
               %
               ShockDraw = randn(q_r*g,1);
               etaTg = muTg+(SigmaTgStd*ShockDraw);
               %
               % overlay with restricted shocks
               %
               etaTh(DSGEModel.MixedConditionalShocks,1:g) = fliplr(reshape(etaTg,q_r,g));
               %
               gGood = 0;
               t = 1;
               K2sum = zeros(qm,1);
               BadDraw = 0;
               while t<=g;
                  ZPosi = ZPos(:,t);
                  m1 = sum(ZPosi);
                  %
                  % add shocks to Xt
                  %
                  Xt(1:q) = etaTh(:,t);
                  [AtNew,status,AlgorithmInformation] = ForwardBackShootingAlgorithmAdaptiveLearning(Xt,At,ePG,ePGx,Rzlb(t,:),Rbar,DSGEModel.AIMTolerance);
                  if status==1;
                     %
                     % check if we have non-zero anticipated shocks
                     %
                     Ksit = Mmats(:,t)+(Pmats(:,:,t)*[Xt;AtNew]);
                     if (sum(AtNew)>0)&(m1>0);
                        NumBindingValues = NumBindingValues+1;
                        %
                        % check the constraint
                        %
                        if k>0;
                           CA = Z(ZPosi,t) - (K1(:,ZPosi)'*A'*X(:,t)) - (K1(:,ZPosi)'*wTh(:,t)) - (K1(:,ZPosi)'*H'*Ksit) - K2sum(ZPosi) -U(ZPosi,g);
                        else;
                           CA = Z(ZPosi,t) - (K1(:,ZPosi)'*wTh(:,t)) - (K1(:,ZPosi)'*H'*Ksit) - K2sum(ZPosi) -U(ZPosi,g);
                        end;
                        if abs(CA)>=1.0e-06;
                           NumBadValues = NumBadValues+1;
                           status = 0;
                        end;
                     end;
                     if status==1;
                        At = AtNew;
                        AtPath(:,t) = At;
                        XtPath(:,t) = Xt;
                        if max(At)>0;
                           %
                           % add the shocks when ZLB is binding and count how many times for given horizon
                           %
                           ShockMatrix(:,t) = ShockMatrix(:,t)+Xt(1:q,1);
                           NumZLBTimes(1,t) = NumZLBTimes(1,t)+1;
                        end;
                        ZLBKsiPath(:,t) = Ksit(1:r);
                        KsiT = Ksit;
                        PredictY(:,t) = Ax(:,t)+(H'*Ksit)+wTh(:,t);
                        %
                        % test if the prediction is "reasonable"
                        %
                        TestValue = (PredictY(:,t)-ZLBYMeanOrig(:,t))./ZLBYMeanOrig(:,t);
                        if max(abs(TestValue))<100;
                           if t+1<=h;
                              At = MA*At;
                              Xt = [zeros(q,1);Ksit];
                           end;
                           %
                           % update the sum of K2 times lagged values of the observed variables
                           %
                           if K2length==2;
                              K2sum = K2sum+(K2'*PredictY(:,t));
                           else;
                              K2sum = zeros(qm,1);
                              for j=1:t;
                                 if j<=size(K2,3);
                                    K2sum = K2sum+(K2(:,:,j)'*PredictY(:,t+1-j));
                                 end;
                              end;
                           end;
                           t = t+1;
                           if t==g+1;
                              gGood = 1;
                           end;
                        else;
                           t = g+1;
                        end;
                     else;
                        t = g+1;
                     end;
                  else;
                     BadDraw = BadDraw+1;
                     if BadDraw>=100;
                        t = g+1;
                     end;
                  end;
               end;
               %
               % check if we should continue
               %
               if gGood==1;
                  if t==h+1;
                     RunLoop = 0;
                  end;
                  while t<=h;
                     Xt(1:q) = randn(q,1);
                     [AtNew,status,AlgorithmInformation] = ForwardBackShootingAlgorithmAdaptiveLearning(Xt,At,ePG,ePGx,Rzlb(t,:),Rbar,DSGEModel.AIMTolerance);
                     if status==1;
                        At = AtNew;
                        AtPath(:,t) = At;
                        XtPath(:,t) = Xt;
                        if max(At)>0;
                           %
                           % add the shocks when ZLB is binding and count how many times for given horizon
                           %
                           ShockMatrix(:,t) = ShockMatrix(:,t)+Xt(1:q,1);
                           NumZLBTimes(1,t) = NumZLBTimes(1,t)+1;
                        end;
                        Ksit = Mmats(:,t)+(Pmats(:,:,t)*[Xt;At]);
                        ZLBKsiPath(:,t) = Ksit(1:r);
                        %
                        PredictY(:,t) = Ax(:,t)+(H'*Ksit)+wTh(:,t);
                        %
                        % test if the prediction is "reasonable"
                        %
                        TestValue = (PredictY(:,t)-ZLBYMeanOrig(:,t))./ZLBYMeanOrig(:,t);
                        if max(abs(TestValue))<100;
                           if t+1<=h;
                              At = MA*At;
                              Xt = [zeros(q,1);Ksit];
                           end;
                           t = t+1;
                           if t==h+1;
                              RunLoop = 0;
                           end;
                        else;
                           t = h+1;
                        end;
                     end;
                  end;
               end;
               TotalRunLoop = TotalRunLoop+1;
               if TotalRunLoop>100;
                  %
                  % use the original parameters instead of the current vector as it doesn't work
                  %
                  KsiTT = KsiTTzmode;
                  S = Smode;
                  L = Lmode;
                  ePG = ePGmode;
                  ePGx = ePGxmode;
                  Mmats = Mmatsmode;
                  Pmats = Pmatsmode;
                  Rzlb = Rzlbmode;
                  Rbar = Rbarmode;
                  Ax = Axmode;
                  A = Amode;
                  H = Hmode;
                  Rmeasure = Rmeasuremode;
                  NumMeasure = NumMeasuremode;
                  MT1 = MT1mode;
                  FT1 = FT1mode;
                  BT1 = BT1mode;
                  MA = MAmode;
                  Xbar = Xbarmode;
                  Gmat = Gmatmode;
                  DpKInvKpDDpK = DpKInvKpDDpKmode;
                  DT1 = DT1mode;
                  SigmaTgStd = SigmaTgStdmode;
                  TotalRunLoop = 1;
               end;
            end;
            %
            % check if we should annualize or transform
            %
            if AnnualizeData==1;
               PredictY = Annualize([Yinit PredictY],DSGEModel.annual,DSGEModel.annualscale);
            elseif TransData==1;
               YTmp = [Yinitial PredictY];
               [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,YTmp',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
               PredictY = YData(Tt-h+1:Tt,:)';
            end;
            %
            % update the mean predictions for all paths and parameter draws
            %
            MeanPredictY = (Weights(ParamIndex)/(NumPaths*NumDraws))*PredictY;
            %
            % add the projected path to the variable to be saved
            %
            PredPaths(:,:,it) = PredictY;
            drawnow;
         end;
         %
         % save the calculations to disk
         %
         if IsPosterior==1;
            if AnnualizeData==0;
               if TransData==0;
                  file = FixFilePath([DSGEModel.OutputDirectory '\predictions\AdaptiveLearningDSGE-' int2str(DSGEModel.FixedBeliefsForProj) '-ZLB-CondPrediction-Original-0-Subset-' DSGEModel.NameOfModel '-' num2str(CurrChain,'%0.0f') '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '.' int2str(DSGEModel.ZLBRuleType) '-' num2str(h,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPaths,'%0.0f') '.mat']);
               else;
                  file = FixFilePath([DSGEModel.OutputDirectory '\predictions\AdaptiveLearningDSGE-' int2str(DSGEModel.FixedBeliefsForProj) '-ZLB-CondPrediction-Transformed-0-Subset-' DSGEModel.NameOfModel '-' num2str(CurrChain,'%0.0f') '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '.' int2str(DSGEModel.ZLBRuleType) '-' num2str(h,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPaths,'%0.0f') '.mat']);
               end;
            else;
               file = FixFilePath([DSGEModel.OutputDirectory '\predictions\AdaptiveLearningDSGE-' int2str(DSGEModel.FixedBeliefsForProj) '-ZLB-CondPrediction-Annual-0-Subset-' DSGEModel.NameOfModel '-' num2str(CurrChain,'%0.0f') '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '.' int2str(DSGEModel.ZLBRuleType) '-' num2str(h,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPaths,'%0.0f') '.mat']);
            end;
         else;
            if AnnualizeData==0;
               if TransData==0;
                  file = FixFilePath([DSGEModel.OutputDirectory '\predictions\AdaptiveLearningDSGE-' int2str(DSGEModel.FixedBeliefsForProj) '-ZLB-CondPrediction-Original-Prior-0-Subset-' DSGEModel.NameOfModel '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '.' int2str(DSGEModel.ZLBRuleType) '-' num2str(h,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPaths,'%0.0f') '.mat']);
               else;
                  file = FixFilePath([DSGEModel.OutputDirectory '\predictions\AdaptiveLearningDSGE-' int2str(DSGEModel.FixedBeliefsForProj) '-ZLB-CondPrediction-Transformed-Prior-0-Subset-' DSGEModel.NameOfModel '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '.' int2str(DSGEModel.ZLBRuleType) '-' num2str(h,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPaths,'%0.0f') '.mat']);
               end;
            else;
               file = FixFilePath([DSGEModel.OutputDirectory '\predictions\AdaptiveLearningDSGE-' int2str(DSGEModel.FixedBeliefsForProj) '-ZLB-CondPrediction-Annual-Prior-0-Subset-' DSGEModel.NameOfModel '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '.' int2str(DSGEModel.ZLBRuleType) '-' num2str(h,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPaths,'%0.0f') '.mat']);
            end;
         end;
         savefileparfor(file,PredPaths,MeanPredictY,ObsVarMean,ShockMatrix,NumZLBTimes);
         if DSGEModel.RunPredictionEvent==1;
            %
            % compute the prediction events results
            %
            PredEventData = CalculatePredictionEvents(PredPaths,DSGEModel.PredictionEvent,Weights(ParamIndex));
            savefileparfor(file,[],[],[],[],[],PredEventData);
         end;
      end;
   else;
      %
      % we have a time-varying H-matrix
      %
      parfor ParamIndex=1:NumDraws;
         %
         % Solve the model for the current parameter vector
         %
         theta = thetaPostSample(ParamIndex,:)';
         ModelParameters = MP;
         Gtilde = Gtildetmp;
         Kgf = Kgftmp;
         X = Xtmp;
         [A,H,R,F,B0,status,ErrMsg,ModelParameters] = SolveDSGEModel(DSGEModel,theta,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
         if status~=1;
            %
            % use posterior mode or initial values as a backup
            %
            [A,H,R,F,B0,status,ErrMsg,ModelParameters] = SolveDSGEModel(DSGEModel,thetaMode,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
         end;
         %
         % check if we can use these values.
         %
         etaMat = K1'*H(:,:,LastPeriod+1)'*B0*Mmatrix;
         if rank(etaMat)<qm;
            %
            % solve the mode for initial parameter values or for the posterior mode values
            %
            [A,H,R,F,B0,status,ErrMsg,ModelParameters] = SolveDSGEModel(DSGEModel,thetaMode,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
         end;
         %
         % Try to solve the model subject to the anticipated shocks
         %
         [Gtmp,C1tmp,Ptmp,mcode] = ZLBKleinSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.ZLBSampleT,DSGEModel.RtildePosition,DSGEModel.REqPosition,DSGEModel.ZLBRuleType,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
         if mcode~=1;
            %
            % solve the mode for initial parameter values or for the posterior mode values
            %
            [A,H,R,F,B0,status,ErrMsg,ModelParameters] = SolveDSGEModel(DSGEModel,thetaMode,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
         end;
         %
         % get the structural form matrices
         %
         [Hlag,H0,Hexp,D] = DSGEStructuralForm(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions);
         %
         % reshuffle the equations to separate the restricted from the unrestricted policy rate
         %
         [Hlag,H0,Hexp,D,DA,MA] = ZLBDSGEStructuralForm(Hlag,H0,Hexp,D,DSGEModel.ZLBSampleT,DSGEModel.RtildePosition,DSGEModel.REqPosition,DSGEModel.ZLBRuleType);
         %
         % solve the model with the new matrices
         %
         [F,B0,BA,mcode] = SolveExtendedDSGEModel(Hlag,H0,Hexp,D,DA,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
         if mcode~=1;
            %
            % use the posterior mode estimates of the parameters
            %
            [A,H,R,F,B0,status,ErrMsg,ModelParameters] = SolveDSGEModel(DSGEModel,thetaMode,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
            [Hlag,H0,Hexp,D] = DSGEStructuralForm(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions);
            [Hlag,H0,Hexp,D,DA,MA] = ZLBDSGEStructuralForm(Hlag,H0,Hexp,D,DSGEModel.ZLBSampleT,DSGEModel.RtildePosition,DSGEModel.REqPosition,DSGEModel.ZLBRuleType);
            [F,B0,BA,mcode] = SolveExtendedDSGEModel(Hlag,H0,Hexp,D,DA,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
         end;
         %
         % Compute the initial values of betavec, Sigmau, R10 and Sigmaeps
         %
         [betavec,Sigmau,R10,Sigmaeps,Gtilde,Kgf] = AdaptiveLearningInitialValuesDSGE(F,B0,ModelParameters.sigmarAL,ModelParameters.sigmaeAL,SMatrix,G,m,x,f,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,Gtilde,Kgf);
         %
         % call the transformation function
         %
         [TransStatus,Hstarlag,Hstar0,Hstar1f] = AdaptiveLearningTransformDSGEModel(Hlag,H0,Hexp,SMatrix,Sperp);
         %
         % compute initial values for the solution matrices
         %
         if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
            [Mt1,Ft1,Bt1,InvH0H1star] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betavec);
         else;
            [Mt1,Ft1,Bt1,InvH0H1star] = feval(GetName(DSGEModel.AdaptiveLearningModelSolverFile),Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betavec,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
         end;
         %
         % compute the solution matrix BAt1 on the anticipated shocks
         %
         BAt1 = InvH0H1star*DA;
         %
         % make sure that we take into account that Rbar depends on the parameters
         %
         if k>0;
            Rbar = A(:,DSGEModel.ZLBPosition)'*X;
         end;
         Rbar = Rbar(:,1:DSGEModel.ZLBSampleT+1);
         %
         % Transform the steady-state interest rate and the lower bound to the same units
         % as the unrestricted monetary policy rate
         %
         Hvalue = shiftdim(H(DSGEModel.RtildePosition,DSGEModel.ZLBPosition,DSGEModel.LastPeriod:Hlength),2)';
         Th = length(Hvalue);
         if Th>DSGEModel.ZLBSampleT;
            Hvalue = Hvalue(1,1:DSGEModel.ZLBSampleT);
         elseif Th<DSGEModel.ZLBSampleT;
            Hvalue = [Hvalue Hvalue(1,Th)*ones(1,DSGEModel.ZLBSampleT-Th)];
         end;
         Rzlb = RzlbMat./Hvalue;
         Rbar = Rbar./Hvalue;
         %
         % we need to fix the H-matrix when using the extended form of the model
         % by adding zeros to 
         %
         HOld = H;
         %
         % we need to fix the H-matrix when using the extended form of the model
         % by adding zeros to 
         %
         HT = zeros(size(H));
         for t=1:size(H,3);
            HT(:,:,t) = [H(:,:,t);zeros(1,n)];
         end;
         H = HT;
         %
         % run the Kalman filter
         %
         if IsALUpdate==1;
            if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
               [lnL,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningKalmanFilter(m,SMatrix,betavec,Sigmau,R10,Sigmaeps,ModelParameters.rhoAL,Hstar0,Hstarlag,Hstar1f,D,Mt1,Ft1,Bt1,Gtilde,Kgf,DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,KalmanFirstObservation,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs);
            else;
               [lnL,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningUserSolverKalmanFilter(GetName(DSGEModel.AdaptiveLearningModelSolverFile),m,SMatrix,betavec,Sigmau,R10,Sigmaeps,ModelParameters.rhoAL,Hstar0,Hstarlag,Hstar1f,D,Mt1,Ft1,Bt1,Gtilde,Kgf,DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,KalmanFirstObservation,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
            end;
         else;
            [lnL,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningSingleKalmanFilter(m,betavec,Mt1,Ft1,Bt1,DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,KalmanFirstObservation,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs);
            LastSolution.InvMatT1 = InvH0H1star;
            LastSolution.InvMatT = InvH0H1star;
         end;
         %
         % check if the user wants to take the conditioning infortmation into account
         % when determining the mean and covariance matrix for the states at T.
         %
         if (DSGEModel.KsiUseCondData==1)&(SCond==1);
            if (k>0);
               XPred = X(:,1:g);
            else;
               XPred = [];
            end;
            [KsiTTz,PTTz] = AdaptiveLearningCondPredictionSmoother(Ksitt1(:,T),Ptt1(:,:,T),Yhat(:,T),DSGEModel.Y(:,LastPeriod),XPred,Z,K1,A,H(:,:,LastPeriod+1:LastPeriod+g),R,Mtilde(:,:,T),Ftilde(:,:,T),Btilde(:,:,T),betavec,LastSolution.betaTT,LastSolution.M,Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,DSGEModel);
%            KsiTTz = Ksitt(:,T);
%            PTTz = Ptt(:,:,T);
         elseif (DSGEModel.KsiUseCondData==1)&(T>=g);
            %
            % we need to run the Kalman smoother
            %
            [KsitT,PtT,etatt,etatT,wtt,wtT] = AdaptiveLearningKalmanSmoother(Ksitt1,Ptt1,Yerror,InvMSEY,H,R,Ftilde,Btilde,FirstPeriod,LastPeriod,KalmanFirstObservation,DSGEModel.YNaNs);
            PtT = PtT(:,:,T-g+1:T);
            KsitT = KsitT(:,T-g+1:T);
            if isempty(wtT)==1;
               wtT = zeros(n,g);
            else;
               wtT = wtT(:,T-g+1:T);
            end;
            %
            if k>0;
               if g>1;
                  Xtra = DSGEModel.X(:,LastPeriod-g+2:LastPeriod);
               else;
                  Xtra = [];
               end;
               [KsiTTz,PTTz] = AdaptiveLearningCondPredictionKalmanSmoother(KsitT,PtT,wtT,Z,U,[Xtra X(:,1:g)],K1,K2,A,H(:,:,LastPeriod-g+2:LastPeriod+g),R,Mtilde(:,:,T),Ftilde(:,:,T),Btilde(:,:,T),betavec,LastSolution.betaTT,LastSolution.M,Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,DSGEModel);
            else;
               [KsiTTz,PTTz] = AdaptiveLearningCondPredictionKalmanSmoother(KsitT,PtT,wtT,Z,U,[],K1,K2,A,H(:,:,LastPeriod-g+2:LastPeriod+g),R,Mtilde(:,:,T),Ftilde(:,:,T),Btilde(:,:,T),betavec,LastSolution.betaTT,LastSolution.M,Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,DSGEModel);
            end;
         else;
            KsiTTz = Ksitt(:,T);
            PTTz = Ptt(:,:,T);
         end;
         PTTz = diag(abs(diag(PTTz)))+tril(PTTz,-1)+tril(PTTz,-1)';
         %
         % use the Schur decomposition for the reduced rank matrix PTTz instead of
         % the Choleski decomposition
         %
         [S,L] = SchurDecomposition(PTTz);
         KsiTT = KsiTTz;
         %
         % deal with possible missing observations in the historical sample if we
         % should annualize or transform the predictions
         %
         if AnnualizeData==1;
            if maxYinit==1;
               %
               % take into account that there are missing obs for the
               % historical data which can affect annualization
               %
               YSmooth = DSGEModel.Y(:,FirstPeriod+KalmanFirstObservation-1:LastPeriod);
               nanpos = isnan(YSmooth);
               %
               % compute the Kalman smoother
               %
               [KsitT,PtT,etatt,etatT,wtt,wtT] = AdaptiveLearningKalmanSmoother(Ksitt1,Ptt1,Yerror,InvMSEY,H,R,Ftilde,Btilde,FirstPeriod,LastPeriod,KalmanFirstObservation,DSGEModel.YNaNs);
               %
               if k>0;
                  for t=1:T;
                     YHistPred = (A'*Xdata(:,t))+(H(:,:,FirstPeriod+KalmanFirstObservation-2+t)'*KsitT(:,t));
                  end;
               else;
                  for t=1:T;
                     YHistPred = H(:,:,FirstPeriod+KalmanFirstObservation-2+t)'*KsitT(:,t);
                  end;
               end;
               YSmooth(nanpos) = YHistPred(nanpos);
               if FirstPeriod+KalmanFirstObservation-1>1;
                  YSmooth = [DSGEModel.Y(:,1:FirstPeriod+KalmanFirstObservation-2) YSmooth];
               end;
               if strcmp(DSGEModel.DataFrequency,'q')==1;
                  Yinit = YSmooth(:,LastPeriod-2:LastPeriod);
               elseif strcmp(DSGEModel.DataFrequency,'m')==1;
                  Yinit = YSmooth(:,LastPeriod-10:LastPeriod);
               else;
                  Yinit = [];
               end;
            end;
         elseif TransData==1;
            if maxYinitial==1;
               %
               % take into account that there are missing obs for the
               % historical data which can affect general transformations
               %
               YSmooth = DSGEModel.Y(:,FirstPeriod+KalmanFirstObservation-1:LastPeriod);
               nanpos = isnan(YSmooth);
               %
               % compute the Kalman smoother
               %
               [KsitT,PtT,etatt,etatT,wtt,wtT] = AdaptiveLearningKalmanSmoother(Ksitt1,Ptt1,Yerror,InvMSEY,H,R,Ftilde,Btilde,FirstPeriod,LastPeriod,KalmanFirstObservation,DSGEModel.YNaNs);
               %
               if k>0;
                  for t=1:T;
                     YHistPred = (A'*Xdata(:,t))+(H(:,:,FirstPeriod+KalmanFirstObservation-2+t)'*KsitT(:,t));
                  end;
               else;
                  for t=1:T;
                     YHistPred = H(:,:,FirstPeriod+KalmanFirstObservation-2+t)'*KsitT(:,t);
                  end;
               end;
               YSmooth(nanpos) = YHistPred(nanpos);
               if FirstPeriod+KalmanFirstObservation-1>1;
                  YSmooth = [DSGEModel.Y(:,1:FirstPeriod+KalmanFirstObservation-2) YSmooth];
               end;
               Yinitial = YSmooth(:,1:LastPeriod);
            end;
         end;
         %
         % store solution matrices
         %
         if DSGEModel.FixedBeliefsForProj==1;
            MT1 = Mtilde(:,:,T);
            FT1 = Ftilde(:,:,T);
            BT1 = Btilde(:,:,T);
         else;
            MT1 = zeros(r+1,k,h);
            FT1 = zeros(r+1,r,h);
            BT1 = zeros(r+1,q,h);
            MT1(:,:,1) = Mtilde(:,:,T);
            FT1(:,:,1) = Ftilde(:,:,T);
            BT1(:,:,1) = Btilde(:,:,T);
            InvMatT1 = zeros(r+1,r+1,h);
            InvMatT1(:,:,1) = LastSolution.InvMatT1;
         end;
         %
         % write down the solution matrices for projections using ZLB format, with Mt, Gt and C1
         %
         [Mt,Gt,C1,Mtilde,Pt] = ZLBProjectionMatrices(Mtilde(:,:,T),Ftilde(:,:,T),Btilde(:,:,T),LastSolution.InvMatT1*DA,MA);
         %
         % deal with deterministic variables
         %
         if k>0;
            Ax = A'*X;
         else;
            Ax = zeros(n,h);
            X = zeros(r,max(DSGEModel.ZLBSampleT+1,h))
         end;
         %
         % setup projections matrices for the restricted policy rate
         %
         % each row in ePG is the row vector multiplied by V(t) for tau=0,1,...,DSGEModel.ZLBSampleT
         % each element in ePGx is the deterministic term of the projection
         %
         ePG = zeros(DSGEModel.ZLBSampleT+1,size(Gt,2));
         ePGx = zeros(DSGEModel.ZLBSampleT+1,1);
         Pmats = zeros(r+1,q+r+2+DSGEModel.ZLBSampleT,max(DSGEModel.ZLBSampleT+1,h));
         Mmats = zeros(r+1,h);
         if DSGEModel.FixedBeliefsForProj==1;
            ePG(1,:) = e_pr'*Pt(1:r+1,:);
            xbart = zeros(size(Mt,1),1);
            ePGx(1) = e_pr'*Mtilde*X(:,1);
            Mmats(:,1) = Mtilde*X(:,1);
            Pmats(:,:,1) = Pt;
            for tau=1:DSGEModel.ZLBSampleT;
               ePG(tau+1,:) = ePG(tau,:)*Gt;
               xbart = (Mt*X(:,tau))+Gt*xbart;
               ePGx(tau+1) = e_pr'*((Mtilde*X(:,tau+1))+(Pt*xbart));
               Mmats(:,tau+1) = Mtilde*X(:,tau+1);
               Pmats(:,:,tau+1) = Pt;
            end;
            if h>DSGEModel.ZLBSampleT+1;
               for tau=DSGEModel.ZLBSampleT+1:h-1;
                  Pmats(:,:,tau+1) = Pt;
                  Mmats(:,tau+1) = Mtilde*X(:,tau+1);
               end;
            end;
         else;
            %
            % beliefs are now updated, take the values of the
            % solution matrices from period T that have been updated for T+1
            %
            MTiT = LastSolution.MTT;
            FTiT = LastSolution.FTT;
            BTiT = LastSolution.BTT;
            betaTiT = LastSolution.betaTT;
            InvMatT = LastSolution.InvMatT;
            MT1(:,:,2) = LastSolution.MTT;
            FT1(:,:,2) = LastSolution.FTT;
            BT1(:,:,2) = LastSolution.BTT;
            InvMatT1(:,:,2) = LastSolution.InvMatT;
            %
            % write down the solution matrices for projections using ZLB format, with Mt, Gt and C1
            %
            [Mt,Gt,C1,Mtilde,Pt] = ZLBProjectionMatrices(MTiT,FTiT,BTiT,InvMatT*DA,MA);
            ePG(1,:) = e_pr'*Pt(1:r+1,:);
            xbart = zeros(size(Mt,1),1);
            ePGx(1) = e_pr'*Mtilde*X(:,1);
            Mmats(:,1) = Mtilde*X(:,1);
            Pmats(:,:,1) = Pt;
            for tau=1:DSGEModel.ZLBSampleT;
               %
               % update betaTiT
               %
               betaTiT = betavec+(LastSolution.M*(betaTiT-betavec));
               %
               % update the solution matrices
               %
               if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
                  [MTiT,FTiT,BTiT,InvMatT] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betaTiT);
               else;
                  [MTiT,FTiT,BTiT,InvMatT] = feval(GetName(DSGEModel.AdaptiveLearningModelSolverFile),Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betaTiT,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
               end;
               %
               % obtain the new projection matrices
               %
               [Mt,Gt,C1,Mtilde,Pt] = ZLBProjectionMatrices(MTiT,FTiT,BTiT,InvMatT*DA,MA);
               ePG(tau+1,:) = ePG(tau,:)*Gt;
               xbart = (Mt*X(:,tau))+Gt*xbart;
               ePGx(tau+1) = e_pr'*((Mtilde*X(:,tau+1))+(Pt*xbart));
               Mmats(:,tau+1) = Mtilde*X(:,tau+1);
               Pmats(:,:,tau+1) = Pt;
               if tau+2<=h;
                  MT1(:,:,tau+2) = MTiT;
                  FT1(:,:,tau+2) = FTiT;
                  BT1(:,:,tau+2) = BTiT;
                  InvMatT1(:,:,tau+2) = InvMatT;
               end;
            end;
            if h>DSGEModel.ZLBSampleT+1;
               for tau=DSGEModel.ZLBSampleT+1:h-1;
                  %
                  % update betaTiT
                  %
                  betaTiT = betavec+(LastSolution.M*(betaTiT-betavec));
                  %
                  % update the solution matrices
                  %
                  if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
                     [MTiT,FTiT,BTiT,InvMatT] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betaTiT);
                  else;
                     [MTiT,FTiT,BTiT,InvMatT] = feval(GetName(DSGEModel.AdaptiveLearningModelSolverFile),Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betaTiT,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
                  end;
                  %
                  % obtain the new projection matrices
                  %
                  [Mt,Gt,C1,Mtilde,Pt] = ZLBProjectionMatrices(MTiT,FTiT,BTiT,InvMatT*DA,MA);
                  Pmats(:,:,tau+1) = Pt;
                  Mmats(:,tau+1) = Mtilde*X(:,tau+1);
                  if tau+2<=h;
                     MT1(:,:,tau+2) = MTiT;
                     FT1(:,:,tau+2) = FTiT;
                     BT1(:,:,tau+2) = BTiT;
                     InvMatT1(:,:,tau+2) = InvMatT;
                  end;
               end;
            end;
         end;
         %
         % stack the model to determine the shock distribution
         %
         Xbar = zeros(n*g,1);
         XTi = zeros(r+1,1);
         Gmat = zeros(n*g,r+1);
         DT1 = zeros(n*g,q*g);
         Fexp = eye(r+1);
         if DSGEModel.FixedBeliefsForProj==1;
            for i=1:g;
               Fexp = FT1*Fexp;
               Ind = (1+((g-i)*n):(g-i+1)*n);
               Ind1 = (1+((g-i)*q):(g-i+1)*q);
               Ind2 = (1+((g-i)*qm):(g-i+1)*qm);
               if k>0;
                  XTi = (MT1*X(:,i))+(FT1*XTi);
                  Xbar(Ind,1) = (A'*X(:,i))+(H(:,:,LastPeriod+i)'*XTi);
               end;
               Gmat(Ind,:) = H(:,:,LastPeriod+i)'*Fexp;
               DT1(Ind,Ind1) = H(:,:,LastPeriod+i)'*BT1;
               Fi = FT1;
               for j=1:i-1;
                  Ind1 = Ind1+q;
                  Ind3 = Ind+(j*n);
                  DT1(Ind,Ind1) = H(:,:,LastPeriod+i)'*Fi*BT1;
                  Fi = Fi*FT1;
               end;
            end;
         else;
            %
            % we have updated beliefs
            %
            for i=1:g;
               Fexp = FT1(:,:,i)*Fexp;
               Ind = (1+((g-i)*n):(g-i+1)*n);
               Ind1 = (1+((g-i)*q):(g-i+1)*q);
               Ind2 = (1+((g-i)*qm):(g-i+1)*qm);
               if k>0;
                  XTi = (MT1(:,:,i)*X(:,i))+(FT1(:,:,i)*XTi);
                  Xbar(Ind,1) = (A'*X(:,i))+(H(:,:,LastPeriod+i)'*XTi);
               end;
               Gmat(Ind,:) = H(:,:,LastPeriod+i)'*Fexp;
               DT1(Ind,Ind1) = H(:,:,LastPeriod+i)'*BT1(:,:,i);
               Fi = FT1(:,:,i);
               for j=1:i-1;
                  Ind1 = Ind1+q;
                  Ind3 = Ind+(j*n);
                  DT1(Ind,Ind1) = H(:,:,LastPeriod+i)'*Fi*BT1(:,:,i);
                  Fi = Fi*FT1(:,:,i-j);
               end;
            end;
         end;
         %
         % speed-up code suggested by Ingvar Strid
         %
         KpD = K'*DT1*IgM;
         KpDDpK = KpD*KpD';
         DpKInvKpDDpK = KpD'/KpDDpK;
         %
         % compute the idempotent covariance matrix for the shocks
         %
         SigmaTg = eye(q_r*g)-(DpKInvKpDDpK*KpD);
         SigmaTg = diag(abs(diag(SigmaTg)))+tril(SigmaTg,-1)+tril(SigmaTg,-1)';
         [SE,LE] = schur(SigmaTg);
         SigmaTgStd = SE*sqrt(abs(LE));
         %
         % check if we have measurement errors and if so compute a Schur decomposition of the R matrix
         %
         if rank(R)>0;
            R(:,DSGEModel.ZLBPosition) = 0;
            R(DSGEModel.ZLBPosition,:) = 0;
            [Rm,Lm] = SchurDecomposition(R);
            Rmeasure = Rm*sqrt(Lm);
            NumMeasure = size(Lm,1);
         else;
            Rmeasure = [];
            wTh = zeros(n,h);
            NumMeasure = 0;
         end;
         %
         % check if we should compute the steady-state mean
         % of the observed variables
         %
         if k==1;
            ObsVarMean = (1/NumDraws)*A';
         end;
         %
         % we are now ready to compute possible paths at theta
         %
         PredPaths = zeros(n,h,NumPaths);
         AtMean = zeros(DSGEModel.ZLBSampleT+1,h);
         XtMean = zeros(q+r+1,h);
         ShockMatrix = zeros(q,h);
         NumZLBTimes = zeros(1,h);
         NumBadValues = 0;
         NumBindingValues = 0;
         %
         for it=1:NumPaths;
            %
            % set up a loop to allow for new draws of the state vector
            % and if that fails to use the thetaMode parameter values
            %
            RunLoop = 1;
            TotalRunLoop = 1;
            while RunLoop==1;
               %
               % Draw a sequence of measurement errors
               %
               if isempty(Rmeasure)==0;
                  wTh = Rmeasure*randn(NumMeasure,h);
                  wTh(DSGEModel.ZLBPosition,:) = 0;
               else;
                  wTh = zeros(n,h);
               end;
               WTg = vec(fliplr(wTh(:,1:g)));
               %
               % Draw a state vector for time T using the N(KsiTTz,PTTz) distribution.
               %
               KsiT = KsiTT+(S*sqrt(L)*randn(size(L,2),1));
               %
               % make sure that the initial values of the unconstrained and constrained policy rate are identical
               %
               KsiT(r+1) = KsiT(DSGEModel.RtildePosition);
               %
               etaTh = zeros(q,h);
               etaTh(:,1:g) = randn(q,g);
               %
               % retrieve the q-q_r shocks eta(t,q-q_r)
               %
               NTgFree = vec(fliplr(Morth'*etaTh(:,1:g)));
               %
               % setut the mean for the shock distribution
               %
               kTg = ZTg-UT-(K'*(Xbar+(Gmat*KsiT)+WTg+(DT1*IgMorth*NTgFree)));
               muTg = DpKInvKpDDpK*kTg;
               PredictY = zeros(n,h);
               At = zeros(DSGEModel.ZLBSampleT+1,1);
               Xt = [zeros(q,1);KsiT];
               ZLBKsiPath = zeros(r,h);
               AtPath = zeros(DSGEModel.ZLBSampleT+1,h);
               XtPath = zeros(q+r+1,h);
               %
               % draw economic shocks from N(muTg,SigmaTg) for period 1 to g
               % and from N(0,I) for period g+1 to h.
               %
               ShockDraw = randn(q_r*g,1);
               etaTg = muTg+(SigmaTgStd*ShockDraw);
               %
               % overlay with restricted shocks
               %
               etaTh(DSGEModel.MixedConditionalShocks,1:g) = fliplr(reshape(etaTg,q_r,g));
               %
               gGood = 0;
               t1 = 1;
               K2sum = zeros(qm,1);
               BadDraw = 0;
               while t1<=g;
                  ZPosi = ZPos(:,t1);
                  m1 = sum(ZPosi);
                  %
                  % add shocks to Xt
                  %
                  Xt(1:q) = etaTh(:,t1);
                  [AtNew,status,AlgorithmInformation] = ForwardBackShootingAlgorithmAdaptiveLearning(Xt,At,ePG,ePGx,Rzlb(t1,:),Rbar,DSGEModel.AIMTolerance);
                  if status==1;
                     %
                     % check if we have non-zero anticipated shocks
                     %
                     Ksit = Mmats(:,t1)+(Pmats(:,:,t1)*[Xt;AtNew]);
                     if (sum(AtNew)>0)&(m1>0);
                        NumBindingValues = NumBindingValues+1;
                        %
                        % check the constraint
                        %
                        if k>0;
                           CA = Z(ZPosi,t1) - (K1(:,ZPosi)'*A'*X(:,t1)) - (K1(:,ZPosi)'*wTh(:,t1)) - (K1(:,ZPosi)'*H(:,:,LastPeriod+t1)'*Ksit) - K2sum(ZPosi) -U(ZPosi,g);
                        else;
                           CA = Z(ZPosi,t1) - (K1(:,ZPosi)'*wTh(:,t1)) - (K1(:,ZPosi)'*H(:,:,LastPeriod+t1)'*Ksit) - K2sum(ZPosi) -U(ZPosi,g);
                        end;
                        if abs(CA)>=1.0e-06;
                           NumBadValues = NumBadValues+1;
                           status = 0;
                        end;
                     end;
                     if status==1;
                        At = AtNew;
                        AtPath(:,t1) = At;
                        XtPath(:,t1) = Xt;
                        if max(At)>0;
                           %
                           % add the shocks when ZLB is binding and count how many times for given horizon
                           %
                           ShockMatrix(:,t1) = ShockMatrix(:,t1)+Xt(1:q,1);
                           NumZLBTimes(1,t1) = NumZLBTimes(1,t1)+1;
                        end;
                        ZLBKsiPath(:,t1) = Ksit(1:r);
                        KsiT = Ksit;
                        PredictY(:,t1) = Ax(:,t1)+(H(:,:,LastPeriod+t1)'*Ksit)+wTh(:,t1);
                        %
                        % test if the prediction is "reasonable"
                        %
                        TestValue = (PredictY(:,t1)-ZLBYMeanOrig(:,t1))./ZLBYMeanOrig(:,t1);
                        if max(abs(TestValue))<100;
                           if t1+1<=h;
                              At = MA*At;
                              Xt = [zeros(q,1);Ksit];
                           end;
                           %
                           % update the sum of K2 times lagged values of the observed variables
                           %
                           if K2length==2;
                              K2sum = K2sum+(K2'*PredictY(:,t));
                           else;
                              K2sum = zeros(qm,1);
                              for j=1:t;
                                 if j<=size(K2,3);
                                    K2sum = K2sum+(K2(:,:,j)'*PredictY(:,t+1-j));
                                 end;
                              end;
                           end;
                           t1 = t1+1;
                           if t1==g+1;
                              gGood = 1;
                           end;
                        else;
                           t1 = g+1;
                        end;
                     else;
                        t1 = g+1;
                     end;
                  else;
                     BadDraw = BadDraw+1;
                     if BadDraw>=100;
                        t1 = g+1;
                     end;
                  end;
               end;
               %
               % check if we should continue
               %
               if gGood==1;
                  if t1==h+1;
                     RunLoop = 0;
                  end;
                  while t1<=h;
                     Xt(1:q) = randn(q,1);
                     [AtNew,status,AlgorithmInformation] = ForwardBackShootingAlgorithmAdaptiveLearning(Xt,At,ePG,ePGx,Rzlb(t1,:),Rbar,DSGEModel.AIMTolerance);
                     if status==1;
                        At = AtNew;
                        AtPath(:,t1) = At;
                        XtPath(:,t1) = Xt;
                        if max(At)>0;
                           %
                           % add the shocks when ZLB is binding and count how many times for given horizon
                           %
                           ShockMatrix(:,t1) = ShockMatrix(:,t1)+Xt(1:q,1);
                           NumZLBTimes(1,t1) = NumZLBTimes(1,t1)+1;
                        end;
                        Ksit = Mmats(:,t1)+(Pmats(:,:,t1)*[Xt;At]);
                        ZLBKsiPath(:,t1) = Ksit(1:r);
                        %
                        PredictY(:,t1) = Ax(:,t1)+(H(:,:,LastPeriod+t1)'*Ksit)+wTh(:,t1);
                        %
                        % test if the prediction is "reasonable"
                        %
                        TestValue = (PredictY(:,t1)-ZLBYMeanOrig(:,t1))./ZLBYMeanOrig(:,t1);
                        if max(abs(TestValue))<100;
                           if t1+1<=h;
                              At = MA*At;
                              Xt = [zeros(q,1);Ksit];
                           end;
                           t1 = t1+1;
                           if t1==h+1;
                              RunLoop = 0;
                           end;
                        else;
                           t1 = h+1;
                        end;
                     end;
                  end;
               end;
               TotalRunLoop = TotalRunLoop+1;
               if TotalRunLoop>100;
                  %
                  % use the original parameters instead of the current vector as it doesn't work
                  %
                  KsiTT = KsiTTzmode;
                  S = Smode;
                  L = Lmode;
                  ePG = ePGmode;
                  ePGx = ePGxmode;
                  Mmats = Mmatsmode;
                  Pmats = Pmatsmode;
                  Rzlb = Rzlbmode;
                  Rbar = Rbarmode;
                  Ax = Axmode;
                  A = Amode;
                  H = Hmode;
                  Rmeasure = Rmeasuremode;
                  NumMeasure = NumMeasuremode;
                  MT1 = MT1mode;
                  FT1 = FT1mode;
                  BT1 = BT1mode;
                  MA = MAmode;
                  Xbar = Xbarmode;
                  Gmat = Gmatmode;
                  DpKInvKpDDpK = DpKInvKpDDpKmode;
                  DT1 = DT1mode;
                  SigmaTgStd = SigmaTgStdmode;
                  TotalRunLoop = 1;
               end;
            end;
            %
            % check if we should annualize or transform
            %
            if AnnualizeData==1;
               PredictY = Annualize([Yinit PredictY],DSGEModel.annual,DSGEModel.annualscale);
            elseif TransData==1;
               YTmp = [Yinitial PredictY];
               [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,YTmp',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
               PredictY = YData(Tt-h+1:Tt,:)';
            end;
            %
            % update the mean predictions for all paths and parameter draws
            %
            MeanPredictY = (Weights(ParamIndex)/(NumPaths*NumDraws))*PredictY;
            %
            % add the projected path to the variable to be saved
            %
            PredPaths(:,:,it) = PredictY;
            drawnow;
         end;
         %
         % save the calculations to disk
         %
         if IsPosterior==1;
            if AnnualizeData==0;
               if TransData==0;
                  file = FixFilePath([DSGEModel.OutputDirectory '\predictions\AdaptiveLearningDSGE-' int2str(DSGEModel.FixedBeliefsForProj) '-ZLB-CondPrediction-Original-0-Subset-' DSGEModel.NameOfModel '-' num2str(CurrChain,'%0.0f') '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '.' int2str(DSGEModel.ZLBRuleType) '-' num2str(h,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPaths,'%0.0f') '.mat']);
               else;
                  file = FixFilePath([DSGEModel.OutputDirectory '\predictions\AdaptiveLearningDSGE-' int2str(DSGEModel.FixedBeliefsForProj) '-ZLB-CondPrediction-Transformed-0-Subset-' DSGEModel.NameOfModel '-' num2str(CurrChain,'%0.0f') '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '.' int2str(DSGEModel.ZLBRuleType) '-' num2str(h,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPaths,'%0.0f') '.mat']);
               end;
            else;
               file = FixFilePath([DSGEModel.OutputDirectory '\predictions\AdaptiveLearningDSGE-' int2str(DSGEModel.FixedBeliefsForProj) '-ZLB-CondPrediction-Annual-0-Subset-' DSGEModel.NameOfModel '-' num2str(CurrChain,'%0.0f') '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '.' int2str(DSGEModel.ZLBRuleType) '-' num2str(h,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPaths,'%0.0f') '.mat']);
            end;
         else;
            if AnnualizeData==0;
               if TransData==0;
                  file = FixFilePath([DSGEModel.OutputDirectory '\predictions\AdaptiveLearningDSGE-' int2str(DSGEModel.FixedBeliefsForProj) '-ZLB-CondPrediction-Original-Prior-0-Subset-' DSGEModel.NameOfModel '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '.' int2str(DSGEModel.ZLBRuleType) '-' num2str(h,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPaths,'%0.0f') '.mat']);
               else;
                  file = FixFilePath([DSGEModel.OutputDirectory '\predictions\AdaptiveLearningDSGE-' int2str(DSGEModel.FixedBeliefsForProj) '-ZLB-CondPrediction-Transformed-Prior-0-Subset-' DSGEModel.NameOfModel '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '.' int2str(DSGEModel.ZLBRuleType) '-' num2str(h,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPaths,'%0.0f') '.mat']);
               end;
            else;
               file = FixFilePath([DSGEModel.OutputDirectory '\predictions\AdaptiveLearningDSGE-' int2str(DSGEModel.FixedBeliefsForProj) '-ZLB-CondPrediction-Annual-Prior-0-Subset-' DSGEModel.NameOfModel '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '.' int2str(DSGEModel.ZLBRuleType) '-' num2str(h,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPaths,'%0.0f') '.mat']);
            end;
         end;
         savefileparfor(file,PredPaths,MeanPredictY,ObsVarMean,ShockMatrix,NumZLBTimes);
         if DSGEModel.RunPredictionEvent==1;
            %
            % compute the prediction events results
            %
            PredEventData = CalculatePredictionEvents(PredPaths,DSGEModel.PredictionEvent,Weights(ParamIndex));
            savefileparfor(file,[],[],[],[],[],PredEventData);
         end;
      end;
   end;
   %
   % run additional calculations to collect estimates if required to do so PredictionEvent only
   %
   if DSGEModel.RunPredictionEvent==1;
      PredEventData = zeros(n,8);
   end;
   MeanPredictY = zeros(n,h);
   if k==1;
      ObsVarMean = zeros(n,1);
   else;
      ObsVarMean = [];
   end;
   for ParamIndex=1:NumDraws;
      if IsPosterior==1;
         if AnnualizeData==0;
            if TransData==0;
               file = FixFilePath([DSGEModel.OutputDirectory '\predictions\AdaptiveLearningDSGE-' int2str(DSGEModel.FixedBeliefsForProj) '-ZLB-CondPrediction-Original-0-Subset-' DSGEModel.NameOfModel '-' num2str(CurrChain,'%0.0f') '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '.' int2str(DSGEModel.ZLBRuleType) '-' num2str(h,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPaths,'%0.0f') '.mat']);
            else;
               file = FixFilePath([DSGEModel.OutputDirectory '\predictions\AdaptiveLearningDSGE-' int2str(DSGEModel.FixedBeliefsForProj) '-ZLB-CondPrediction-Transformed-0-Subset-' DSGEModel.NameOfModel '-' num2str(CurrChain,'%0.0f') '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '.' int2str(DSGEModel.ZLBRuleType) '-' num2str(h,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPaths,'%0.0f') '.mat']);
            end;
         else;
            file = FixFilePath([DSGEModel.OutputDirectory '\predictions\AdaptiveLearningDSGE-' int2str(DSGEModel.FixedBeliefsForProj) '-ZLB-CondPrediction-Annual-0-Subset-' DSGEModel.NameOfModel '-' num2str(CurrChain,'%0.0f') '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '.' int2str(DSGEModel.ZLBRuleType) '-' num2str(h,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPaths,'%0.0f') '.mat']);
         end;
      else;
         if AnnualizeData==0;
            if TransData==0;
               file = FixFilePath([DSGEModel.OutputDirectory '\predictions\AdaptiveLearningDSGE-' int2str(DSGEModel.FixedBeliefsForProj) '-ZLB-CondPrediction-Original-Prior-0-Subset-' DSGEModel.NameOfModel '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '.' int2str(DSGEModel.ZLBRuleType) '-' num2str(h,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPaths,'%0.0f') '.mat']);
            else;
               file = FixFilePath([DSGEModel.OutputDirectory '\predictions\AdaptiveLearningDSGE-' int2str(DSGEModel.FixedBeliefsForProj) '-ZLB-CondPrediction-Transformed-Prior-0-Subset-' DSGEModel.NameOfModel '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '.' int2str(DSGEModel.ZLBRuleType) '-' num2str(h,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPaths,'%0.0f') '.mat']);
            end;
         else;
            file = FixFilePath([DSGEModel.OutputDirectory '\predictions\AdaptiveLearningDSGE-' int2str(DSGEModel.FixedBeliefsForProj) '-ZLB-CondPrediction-Annual-Prior-0-Subset-' DSGEModel.NameOfModel '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '.' int2str(DSGEModel.ZLBRuleType) '-' num2str(h,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPaths,'%0.0f') '.mat']);
         end;
      end;
      %
      % load the file
      %
      ParamData = load(file);
      MeanPredictY = MeanPredictY+ParamData.MeanPredictY;
      if isempty(ObsVarMean)==0;
         ObsVarMean = ObsVarMean+ParamData.ObsVarMean;
      end;
      %
      % add up the PredEventData
      %
      if DSGEModel.RunPredictionEvent==1;
         PredEventData = PredEventData+ParamData.PredEventData;
      end;
   end;
   %
   % close the wait dialog
   %
   set(WaitControls.text,'UserData','done');
   delete(WaitHandle);
   drawnow;
   pause(0.02);
end;
%
if StopEstimation==1;
   return;
end;
%
% make sure we also store additional data in the last file
%
DoneCalc = 1;
if isfield(DSGEModel,'PredictionEvent')==1;
   PredictionEvent = DSGEModel.PredictionEvent;
else;
   PredictionEvent = [Inf*ones(n,1) zeros(n,1) ones(n,1)];
end;
save(file,'MeanPredictY','PredEventData','PredictionEvent','ObsVarMean','-APPEND');

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% a local function for saving to matfile from a parfor loop
%

function savefileparfor(file,PredPaths,MeanPredictY,ObsVarMean,ShockMatrix,NumZLBTimes,PredEventData)

if nargin==6;
   save(file,'PredPaths','MeanPredictY','ObsVarMean','ShockMatrix','NumZLBTimes');
elseif nargin==7;
   save(file,'PredEventData','-APPEND');
end;

%
% end of ZLBAdaptiveLearningDSGECondPredictionPathsMixed.m
%
